@@ -21,6 +21,10 @@ class ReplicaSetFixture(interface.ReplFixture):
     Fixture which provides JSTests with a replica set to run against.
     """
 
+    # Error response codes copied from mongo/base/error_codes.err.
+    _ALREADY_INITIALIZED = 23
+    _NODE_NOT_FOUND = 74
+
     def __init__(self,
                  logger,
                  job_num,
@@ -32,9 +36,11 @@ class ReplicaSetFixture(interface.ReplFixture):
                  start_initial_sync_node=False,
                  write_concern_majority_journal_default=None,
                  auth_options=None,
-                 replset_config_options=None):
+                 replset_config_options=None,
+                 voting_secondaries=False,
+                 use_replica_set_connection_string=False):
 
-        interface.ReplFixture.__init__(self, logger, job_num)
+        interface.ReplFixture.__init__(self, logger, job_num, dbpath_prefix=dbpath_prefix)
 
         self.mongod_executable = mongod_executable
         self.mongod_options = utils.default_if_none(mongod_options, {})
@@ -44,6 +50,8 @@ class ReplicaSetFixture(interface.ReplFixture):
         self.write_concern_majority_journal_default = write_concern_majority_journal_default
         self.auth_options = auth_options
         self.replset_config_options = utils.default_if_none(replset_config_options, {})
+        self.voting_secondaries = voting_secondaries
+        self.use_replica_set_connection_string = use_replica_set_connection_string
 
         # The dbpath in mongod_options is used as the dbpath prefix for replica set members and
         # takes precedence over other settings. The ShardedClusterFixture uses this parameter to
@@ -51,12 +59,7 @@ class ReplicaSetFixture(interface.ReplFixture):
         if "dbpath" in self.mongod_options:
             self._dbpath_prefix = self.mongod_options.pop("dbpath")
         else:
-            # Command line options override the YAML configuration.
-            dbpath_prefix = utils.default_if_none(config.DBPATH_PREFIX, dbpath_prefix)
-            dbpath_prefix = utils.default_if_none(dbpath_prefix, config.DEFAULT_DBPATH_PREFIX)
-            self._dbpath_prefix = os.path.join(dbpath_prefix,
-                                               "job%d" % (self.job_num),
-                                               config.FIXTURE_SUBDIR)
+            self._dbpath_prefix = os.path.join(self._dbpath_prefix, config.FIXTURE_SUBDIR)
 
         self.nodes = []
         self.replset_name = None
@@ -84,81 +87,140 @@ class ReplicaSetFixture(interface.ReplFixture):
 
         self.port = self.get_primary().port
 
-        # Call await_ready() on each of the nodes here because we want to start the election as
-        # soon as possible.
-        for node in self.nodes:
-            node.await_ready()
+        # We need only to wait to connect to the first node of the replica set because we first
+        # initiate it as a single node replica set.
+        self.get_primary().await_ready()
 
         # Initiate the replica set.
         members = []
         for (i, node) in enumerate(self.nodes):
-            member_info = {"_id": i, "host": node.get_connection_string()}
+            member_info = {"_id": i, "host": node.get_internal_connection_string()}
             if i > 0:
                 member_info["priority"] = 0
-            if i >= 7:
-                # Only 7 nodes in a replica set can vote, so the other members must be non-voting.
-                member_info["votes"] = 0
+                if i >= 7 or not self.voting_secondaries:
+                    # Only 7 nodes in a replica set can vote, so the other members must still be
+                    # non-voting when this fixture is configured to have voting secondaries.
+                    member_info["votes"] = 0
             members.append(member_info)
         if self.initial_sync_node:
             members.append({"_id": self.initial_sync_node_idx,
-                            "host": self.initial_sync_node.get_connection_string(),
+                            "host": self.initial_sync_node.get_internal_connection_string(),
                             "priority": 0,
                             "hidden": 1,
                             "votes": 0})
 
-        initiate_cmd_obj = {"replSetInitiate": {"_id": self.replset_name, "members": members}}
-
-        if self.write_concern_majority_journal_default is not None:
-            initiate_cmd_obj["replSetInitiate"]["writeConcernMajorityJournalDefault"] = self.write_concern_majority_journal_default
-
+        config = {"_id": self.replset_name}
         client = utils.new_mongo_client(port=self.port)
+
         if self.auth_options is not None:
             auth_db = client[self.auth_options["authenticationDatabase"]]
             auth_db.authenticate(self.auth_options["username"],
                                  password=self.auth_options["password"],
                                  mechanism=self.auth_options["authenticationMechanism"])
 
+        if client.local.system.replset.count():
+            # Skip initializing the replset if there is an existing configuration.
+            return
+
+        if self.write_concern_majority_journal_default is not None:
+            config["writeConcernMajorityJournalDefault"] = self.write_concern_majority_journal_default
+        else:
+            serverStatus = client.admin.command({"serverStatus": 1})
+            cmdLineOpts = client.admin.command({"getCmdLineOpts": 1})
+            if not (serverStatus["storageEngine"]["persistent"] and
+                    cmdLineOpts["parsed"].get("storage", {}).get("journal", {}).get("enabled", True)):
+                config["writeConcernMajorityJournalDefault"] = False
+
         if self.replset_config_options.get("configsvr", False):
-            initiate_cmd_obj["replSetInitiate"]["configsvr"] = True
+            config["configsvr"] = True
         if self.replset_config_options.get("settings"):
             replset_settings = self.replset_config_options["settings"]
-            initiate_cmd_obj["replSetInitiate"]["settings"] = replset_settings
+            config["settings"] = replset_settings
 
-        self.logger.info("Issuing replSetInitiate command...%s", initiate_cmd_obj)
-        client.admin.command(initiate_cmd_obj)
+        # Start up a single node replica set then reconfigure to the correct size (if the config
+        # contains more than 1 node), so the primary is elected more quickly.
+        config["members"] = [members[0]]
+        self.logger.info("Issuing replSetInitiate command: %s", config)
+        self._configure_repl_set(client, {"replSetInitiate": config})
+        self._await_primary()
+
+        if self.get_secondaries():
+            # Wait to connect to each of the secondaries before running the replSetReconfig
+            # command.
+            for node in self.get_secondaries():
+                node.await_ready()
+            config["version"] = 2
+            config["members"] = members
+            self.logger.info("Issuing replSetReconfig command: %s", config)
+            self._configure_repl_set(client, {"replSetReconfig": config})
+            self._await_secondaries()
+
+    def _configure_repl_set(self, client, cmd_obj):
+        # replSetInitiate and replSetReconfig commands can fail with a NodeNotFound error
+        # if a heartbeat times out during the quorum check. We retry three times to reduce
+        # the chance of failing this way.
+        num_initiate_attempts = 3
+        for attempt in range(1, num_initiate_attempts + 1):
+            try:
+                client.admin.command(cmd_obj)
+                break
+            except pymongo.errors.OperationFailure as err:
+                # Ignore errors from the "replSetInitiate" command when the replica set has already
+                # been initiated.
+                if err.code == ReplicaSetFixture._ALREADY_INITIALIZED:
+                    return
+
+                # Retry on NodeNotFound errors from the "replSetInitiate" command.
+                if err.code != ReplicaSetFixture._NODE_NOT_FOUND:
+                    raise
+
+                msg = "replSetInitiate failed attempt {0} of {1} with error: {2}".format(
+                    attempt, num_initiate_attempts, err)
+                self.logger.error(msg)
+                if attempt == num_initiate_attempts:
+                    raise
+                time.sleep(5)  # Wait a little bit before trying again.
 
     def await_ready(self):
+        self._await_primary()
+        self._await_secondaries()
+
+    def _await_primary(self):
         # Wait for the primary to be elected.
         client = utils.new_mongo_client(port=self.port)
         while True:
+            self.logger.info("Waiting for primary on port %d to be elected.", self.port)
             is_master = client.admin.command("isMaster")["ismaster"]
             if is_master:
                 break
-            self.logger.info("Waiting for primary on port %d to be elected.", self.port)
             time.sleep(0.1)  # Wait a little bit before trying again.
+        self.logger.info("Primary on port %d successfully elected.", self.port)
 
+    def _await_secondaries(self):
+        # Wait for the secondaries to become available.
         secondaries = self.get_secondaries()
         if self.initial_sync_node:
             secondaries.append(self.initial_sync_node)
 
-        # Wait for the secondaries to become available.
         for secondary in secondaries:
             client = utils.new_mongo_client(port=secondary.port,
                                             read_preference=pymongo.ReadPreference.SECONDARY)
             while True:
+                self.logger.info("Waiting for secondary on port %d to become available.",
+                                 secondary.port)
                 is_secondary = client.admin.command("isMaster")["secondary"]
                 if is_secondary:
                     break
-                self.logger.info("Waiting for secondary on port %d to become available.",
-                                 secondary.port)
                 time.sleep(0.1)  # Wait a little bit before trying again.
+            self.logger.info("Secondary on port %d is now available.", secondary.port)
 
-    def teardown(self):
+    def _do_teardown(self):
         running_at_start = self.is_running()
         success = True  # Still a success even if nothing is running.
 
         if not running_at_start:
-            self.logger.info("Replica set was expected to be running in teardown(), but wasn't.")
+            self.logger.info(
+                "Replica set was expected to be running in _do_teardown(), but wasn't.")
         else:
             self.logger.info("Stopping all members of the replica set...")
 
@@ -226,11 +288,27 @@ class ReplicaSetFixture(interface.ReplFixture):
 
         return logging.loggers.new_logger(logger_name, parent=self.logger)
 
-    def get_connection_string(self):
+    def get_internal_connection_string(self):
         if self.replset_name is None:
-            raise ValueError("Must call setup() before calling get_connection_string()")
+            raise ValueError("Must call setup() before calling get_internal_connection_string()")
 
-        conn_strs = [node.get_connection_string() for node in self.nodes]
+        conn_strs = [node.get_internal_connection_string() for node in self.nodes]
         if self.initial_sync_node:
-            conn_strs.append(self.initial_sync_node.get_connection_string())
+            conn_strs.append(self.initial_sync_node.get_internal_connection_string())
         return self.replset_name + "/" + ",".join(conn_strs)
+
+    def get_driver_connection_url(self):
+        if self.replset_name is None:
+            raise ValueError("Must call setup() before calling get_driver_connection_url()")
+
+        if self.use_replica_set_connection_string:
+            # We use a replica set connection string when all nodes are electable because we
+            # anticipate the client will want to gracefully handle any failovers.
+            conn_strs = [node.get_internal_connection_string() for node in self.nodes]
+            if self.initial_sync_node:
+                conn_strs.append(self.initial_sync_node.get_internal_connection_string())
+            return "mongodb://" + ",".join(conn_strs) + "/?replicaSet=" + self.replset_name
+        else:
+            # We return a direct connection to the expected pimary when only the first node is
+            # electable because we want the client to error out if a stepdown occurs.
+            return self.nodes[0].get_driver_connection_url()
