@@ -161,13 +161,14 @@ var Cluster = function(options) {
                "Both 'masterSlave' and 'sharded.enabled' cannot" + "be true");
     }
 
-    function makeReplSetTestConfig(numReplSetNodes) {
+    function makeReplSetTestConfig(numReplSetNodes, firstNodeOnlyVote = true) {
         const REPL_SET_VOTING_LIMIT = 7;
+        var firstNodeNotVoting = firstNodeOnlyVote ? 1 : REPL_SET_VOTING_LIMIT;
         // Workaround for SERVER-26893 to specify when numReplSetNodes > REPL_SET_VOTING_LIMIT.
         var rstConfig = [];
         for (var i = 0; i < numReplSetNodes; i++) {
             rstConfig[i] = {};
-            if (i >= REPL_SET_VOTING_LIMIT) {
+            if (i >= firstNodeNotVoting) {
                 rstConfig[i].rsConfig = {priority: 0, votes: 0};
             }
         }
@@ -223,11 +224,11 @@ var Cluster = function(options) {
 
             conn = st.s;  // mongos
 
-            this.teardown = function teardown() {
+            this.teardown = function teardown(opts) {
                 options.teardownFunctions.mongod.forEach(this.executeOnMongodNodes);
                 options.teardownFunctions.mongos.forEach(this.executeOnMongosNodes);
 
-                st.stop();
+                st.stop(opts);
             };
 
             // Save all mongos and mongod connections
@@ -273,10 +274,9 @@ var Cluster = function(options) {
             conn = rst.getPrimary();
             replSets = [rst];
 
-            this.teardown = function teardown() {
+            this.teardown = function teardown(opts) {
                 options.teardownFunctions.mongod.forEach(this.executeOnMongodNodes);
-
-                rst.stopSet();
+                rst.stopSet(undefined, undefined, opts);
             };
 
             this._addReplicaSetConns(rst);
@@ -573,6 +573,25 @@ var Cluster = function(options) {
 
         return data;
     };
+
+    this.isRunningWiredTigerLSM = function isRunningWiredTigerLSM() {
+        var adminDB = this.getDB('admin');
+
+        if (this.isSharded()) {
+            // Get the storage engine the sharded cluster is configured to use from one of the
+            // shards since mongos won't report it.
+            adminDB = st.shard0.getDB('admin');
+        }
+
+        var res = adminDB.runCommand({getCmdLineOpts: 1});
+        assert.commandWorked(res, 'failed to get command line options');
+
+        var wiredTigerOptions = res.parsed.storage.wiredTiger || {};
+        var wiredTigerCollectionConfig = wiredTigerOptions.collectionConfig || {};
+        var wiredTigerConfigString = wiredTigerCollectionConfig.configString || '';
+
+        return wiredTigerConfigString === 'type=lsm';
+    };
 };
 
 /**
