@@ -1,5 +1,14 @@
 // Test various cursor behaviors
-var t = db.geo_s2getmmm;
+//
+// @tags: [
+//   # This test attempts to enable profiling on a server and then get profiling data by reading
+//   # from the "system.profile" collection. The former operation must be routed to the primary in
+//   # a replica set, whereas the latter may be routed to a secondary.
+//   assumes_read_preference_unchanged,
+// ]
+
+var testDB = db.getSiblingDB("geo_s2cursorlimitskip");
+var t = testDB.geo_s2getmmm;
 t.drop();
 t.ensureIndex({geo: "2dsphere"});
 
@@ -21,9 +30,7 @@ function insertRandomPoints(num, minDist, maxDist) {
     for (var i = 0; i < num; i++) {
         var lat = sign() * (minDist + random() * (maxDist - minDist));
         var lng = sign() * (minDist + random() * (maxDist - minDist));
-        var point = {
-            geo: {type: "Point", coordinates: [lng, lat]}
-        };
+        var point = {geo: {type: "Point", coordinates: [lng, lat]}};
         assert.writeOK(t.insert(point));
     }
 }
@@ -35,16 +42,21 @@ var batchSize = 4;
 // Insert points between 0.01 and 1.0 away.
 insertRandomPoints(totalPointCount, 0.01, 1.0);
 
-var cursor = t.find({geo: {$geoNear: {$geometry: {type: "Point", coordinates: [0.0, 0.0]}}}})
-                 .batchSize(batchSize);
+var cursor = t.find({
+                  geo: {$geoNear: {$geometry: {type: "Point", coordinates: [0.0, 0.0]}}}
+              }).batchSize(batchSize);
 assert.eq(cursor.count(), totalPointCount);
 
 // Disable profiling in order to drop the system.profile collection.
 // Then enable profiling for all operations. This is acceptable because
 // our test is blacklisted from the parallel suite.
-db.setProfilingLevel(0);
-db.system.profile.drop();
-db.setProfilingLevel(2);
+testDB.setProfilingLevel(0);
+testDB.system.profile.drop();
+// Create 4MB system.profile collection to prevent the 'getmore' operations from overwriting the
+// original query.
+assert.commandWorked(
+    testDB.createCollection("system.profile", {capped: true, size: 4 * 1024 * 1024}));
+testDB.setProfilingLevel(2);
 
 for (var j = 0; j < initialAdvance; j++) {
     assert(cursor.hasNext());
@@ -54,8 +66,8 @@ for (var j = 0; j < initialAdvance; j++) {
 assert(cursor.hasNext());
 
 // Cursor was advanced 10 times, batchSize=4 => 1 query + 2 getmore.
-assert.eq(1, db.system.profile.count({op: "query", ns: t.getFullName()}));
-assert.eq(2, db.system.profile.count({op: "getmore", ns: t.getFullName()}));
+assert.eq(1, testDB.system.profile.count({op: "query", ns: t.getFullName()}));
+assert.eq(2, testDB.system.profile.count({op: "getmore", ns: t.getFullName()}));
 
 for (var k = initialAdvance; k < totalPointCount; k++) {
     assert(cursor.hasNext());
@@ -63,26 +75,28 @@ for (var k = initialAdvance; k < totalPointCount; k++) {
 }
 
 // Cursor was advanced 200 times, batchSize=4 => 1 query + 49 getmore.
-assert.eq(1, db.system.profile.count({op: "query", ns: t.getFullName()}));
-assert.eq(49, db.system.profile.count({op: "getmore", ns: t.getFullName()}));
+assert.eq(1, testDB.system.profile.count({op: "query", ns: t.getFullName()}));
+assert.eq(49, testDB.system.profile.count({op: "getmore", ns: t.getFullName()}));
 
 // Disable profiling again - no longer needed for remainder of test
-db.setProfilingLevel(0);
-db.system.profile.drop();
+testDB.setProfilingLevel(0);
+testDB.system.profile.drop();
 
 // Shouldn't be any more points to look at now.
 assert(!cursor.hasNext());
 
 var someLimit = 23;
 // Make sure limit does something.
-cursor = t.find({geo: {$geoNear: {$geometry: {type: "Point", coordinates: [0.0, 0.0]}}}})
-             .limit(someLimit);
+cursor = t.find({
+              geo: {$geoNear: {$geometry: {type: "Point", coordinates: [0.0, 0.0]}}}
+          }).limit(someLimit);
 // Count doesn't work here -- ignores limit/skip, so we use itcount.
 assert.eq(cursor.itcount(), someLimit);
 // Make sure skip works by skipping some stuff ourselves.
 var someSkip = 3;
-cursor = t.find({geo: {$geoNear: {$geometry: {type: "Point", coordinates: [0.0, 0.0]}}}})
-             .limit(someLimit + someSkip);
+cursor = t.find({
+              geo: {$geoNear: {$geometry: {type: "Point", coordinates: [0.0, 0.0]}}}
+          }).limit(someLimit + someSkip);
 for (var i = 0; i < someSkip; ++i) {
     cursor.next();
 }
