@@ -17,7 +17,7 @@ res = coll.ensureIndex({geo: "2dsphere"}, {"2dsphereIndexVersion": 0});
 assert.commandFailed(res);
 coll.drop();
 
-res = coll.ensureIndex({geo: "2dsphere"}, {"2dsphereIndexVersion": 3});
+res = coll.ensureIndex({geo: "2dsphere"}, {"2dsphereIndexVersion": 4});
 assert.commandFailed(res);
 coll.drop();
 
@@ -66,13 +66,15 @@ assert.commandWorked(res);
 coll.drop();
 
 //
-// {2dsphereIndexVersion: 2} should be the default for new indexes.
+// {2dsphereIndexVersion: 3} should be the default for new indexes.
 //
 
 res = coll.ensureIndex({geo: "2dsphere"});
 assert.commandWorked(res);
-var specObj = coll.getDB().system.indexes.findOne({ns: coll.getFullName(), name: "geo_2dsphere"});
-assert.eq(2, specObj["2dsphereIndexVersion"]);
+var specObj = coll.getIndexes().filter(function(z) {
+    return z.name == "geo_2dsphere";
+})[0];
+assert.eq(3, specObj["2dsphereIndexVersion"]);
 coll.drop();
 
 //
@@ -107,35 +109,64 @@ coll.drop();
 var pointDoc = {geo: {type: "Point", coordinates: [40, 5]}};
 var lineStringDoc = {geo: {type: "LineString", coordinates: [[40, 5], [41, 6]]}};
 var polygonDoc = {geo: {type: "Polygon", coordinates: [[[0, 0], [3, 6], [6, 1], [0, 0]]]}};
-var multiPointDoc = {geo: {type: "MultiPoint",
-                           coordinates: [[-73.9580, 40.8003], [-73.9498, 40.7968],
-                                         [-73.9737, 40.7648], [-73.9814, 40.7681]]}};
-var multiLineStringDoc = {geo: {type: "MultiLineString",
-                                coordinates: [[[-73.96943, 40.78519], [-73.96082, 40.78095]],
-                                              [[-73.96415, 40.79229], [-73.95544, 40.78854]],
-                                              [[-73.97162, 40.78205], [-73.96374, 40.77715]],
-                                              [[-73.97880, 40.77247], [-73.97036, 40.76811]]]}};
-var multiPolygonDoc = {geo: {type: "MultiPolygon",
-                             coordinates: [[[[-73.958, 40.8003], [-73.9498, 40.7968],
-                                             [-73.9737, 40.7648], [-73.9814, 40.7681],
-                                             [-73.958, 40.8003]]],
-                                           [[[-73.958, 40.8003], [-73.9498, 40.7968],
-                                             [-73.9737, 40.7648], [-73.958, 40.8003]]]]}};
-var geometryCollectionDoc = {geo: {type: "GeometryCollection",
-                                   geometries: [{type: "MultiPoint",
-                                                 coordinates: [[-73.9580, 40.8003],
-                                                               [-73.9498, 40.7968],
-                                                               [-73.9737, 40.7648],
-                                                               [-73.9814, 40.7681]]},
-                                                {type: "MultiLineString",
-                                                 coordinates: [[[-73.96943, 40.78519],
-                                                                [-73.96082, 40.78095]],
-                                                               [[-73.96415, 40.79229],
-                                                                [-73.95544, 40.78854]],
-                                                               [[-73.97162, 40.78205],
-                                                                [-73.96374, 40.77715]],
-                                                               [[-73.97880, 40.77247],
-                                                                [-73.97036, 40.76811]]]}]}};
+var multiPointDoc = {
+    geo: {
+        type: "MultiPoint",
+        coordinates:
+            [[-73.9580, 40.8003], [-73.9498, 40.7968], [-73.9737, 40.7648], [-73.9814, 40.7681]]
+    }
+};
+var multiLineStringDoc = {
+    geo: {
+        type: "MultiLineString",
+        coordinates: [
+            [[-73.96943, 40.78519], [-73.96082, 40.78095]],
+            [[-73.96415, 40.79229], [-73.95544, 40.78854]],
+            [[-73.97162, 40.78205], [-73.96374, 40.77715]],
+            [[-73.97880, 40.77247], [-73.97036, 40.76811]]
+        ]
+    }
+};
+var multiPolygonDoc = {
+    geo: {
+        type: "MultiPolygon",
+        coordinates: [
+            [[
+               [-73.958, 40.8003],
+               [-73.9498, 40.7968],
+               [-73.9737, 40.7648],
+               [-73.9814, 40.7681],
+               [-73.958, 40.8003]
+            ]],
+            [[[-73.958, 40.8003], [-73.9498, 40.7968], [-73.9737, 40.7648], [-73.958, 40.8003]]]
+        ]
+    }
+};
+var geometryCollectionDoc = {
+    geo: {
+        type: "GeometryCollection",
+        geometries: [
+            {
+              type: "MultiPoint",
+              coordinates: [
+                  [-73.9580, 40.8003],
+                  [-73.9498, 40.7968],
+                  [-73.9737, 40.7648],
+                  [-73.9814, 40.7681]
+              ]
+            },
+            {
+              type: "MultiLineString",
+              coordinates: [
+                  [[-73.96943, 40.78519], [-73.96082, 40.78095]],
+                  [[-73.96415, 40.79229], [-73.95544, 40.78854]],
+                  [[-73.97162, 40.78205], [-73.96374, 40.77715]],
+                  [[-73.97880, 40.77247], [-73.97036, 40.76811]]
+              ]
+            }
+        ]
+    }
+};
 
 // {2dsphereIndexVersion: 2} indexes allow all supported GeoJSON objects.
 res = coll.ensureIndex({geo: "2dsphere"}, {"2dsphereIndexVersion": 2});
