@@ -1,8 +1,12 @@
+// @tags: [
+//   # This test attempts to remove documents using the stageDebug command, which doesn't support
+//   # specifying a writeConcern.
+//   assumes_write_concern_unchanged,
+// ]
+
 // Test basic delete stage functionality.
 var coll = db.stages_delete;
-var collScanStage = {
-    cscan: {args: {direction: 1}, filter: {deleteMe: true}}
-};
+var collScanStage = {cscan: {args: {direction: 1}, filter: {deleteMe: true}}};
 var deleteStage;
 
 // Test delete stage with isMulti: true.
