@@ -47,7 +47,7 @@ load("jstests/replsets/rslib.js");
     });
 
     // Make sure we have a master and that that master is node A
-    replTest.waitForState(replTest.nodes[0], ReplSetTest.State.PRIMARY, 60 * 1000);
+    replTest.waitForState(replTest.nodes[0], ReplSetTest.State.PRIMARY);
     var master = replTest.getPrimary();
     var a_conn = conns[0];
     a_conn.setSlaveOk();
