@@ -1,7 +1,8 @@
 // Tests the copydb command on mongos with auth
-var runTest = function() {
+(function() {
+    'use strict';
 
-    var st = new ShardingTest({shards: 1, mongos: 1, keyFile: "jstests/libs/key1"});
+    var st = new ShardingTest({shards: 1, mongos: 1, other: {keyFile: 'jstests/libs/key1'}});
     var mongos = st.s0;
     var destAdminDB = mongos.getDB('admin');
     var destTestDB = mongos.getDB('test');
@@ -36,7 +37,4 @@ var runTest = function() {
     assert.eq(1, destTestDB.foo.findOne().a);
 
     st.stop();
-
-};
-
-runTest();
+})();
