@@ -76,9 +76,9 @@ enum {
     DATE_RESERVE_SIZE = 64
 };
 
-static const char* LBRACE = "{", * RBRACE = "}", * LBRACKET = "[", * RBRACKET = "]", * LPAREN = "(",
-                   * RPAREN = ")", * COLON = ":", * COMMA = ",", * FORWARDSLASH = "/",
-                   * SINGLEQUOTE = "'", * DOUBLEQUOTE = "\"";
+static const char *LBRACE = "{", *RBRACE = "}", *LBRACKET = "[", *RBRACKET = "]", *LPAREN = "(",
+                  *RPAREN = ")", *COLON = ":", *COMMA = ",", *FORWARDSLASH = "/",
+                  *SINGLEQUOTE = "'", *DOUBLEQUOTE = "\"";
 
 JParse::JParse(StringData str)
     : _buf(str.rawData()), _input(_buf), _input_end(_input + str.size()) {}
@@ -394,8 +394,16 @@ Status JParse::binaryObject(StringData fieldName, BSONObjBuilder& builder) {
             "Argument of $type in $bindata object must be a hex string representation of a single "
             "byte");
     }
+
+    // The fromHex function returns a signed char, but the highest
+    // BinDataType value is 128, which can only be represented as an
+    // unsigned char. If we don't coerce it to an unsigned char before
+    // wrapping it in a BinDataType (currently implicitly a signed
+    // integer), we get undefined behavior.
+    const auto binDataTypeNumeric = static_cast<unsigned char>(fromHex(binDataType));
+
     builder.appendBinData(
-        fieldName, binData.length(), BinDataType(fromHex(binDataType)), binData.data());
+        fieldName, binData.length(), BinDataType(binDataTypeNumeric), binData.data());
     return Status::OK();
 }
 
