@@ -588,6 +588,7 @@ Status MatchExpressionParser::_parseInExpression(InMatchExpression* inExpression
                                                  const BSONObj& theArray,
                                                  const CollatorInterface* collator) {
     inExpression->setCollator(collator);
+    std::vector<BSONElement> equalities;
     BSONObjIterator i(theArray);
     while (i.more()) {
         BSONElement e = i.next();
@@ -606,12 +607,10 @@ Status MatchExpressionParser::_parseInExpression(InMatchExpression* inExpression
             if (!s.isOK())
                 return s;
         } else {
-            Status s = inExpression->addEquality(e);
-            if (!s.isOK())
-                return s;
+            equalities.push_back(e);
         }
     }
-    return Status::OK();
+    return inExpression->setEqualities(std::move(equalities));
 }
 
 StatusWithMatchExpression MatchExpressionParser::_parseType(const char* name,
