@@ -107,87 +107,6 @@ TEST(MatchExpressionParserTreeTest, NOT1) {
     ASSERT(!result.getValue()->matchesBSON(BSON("x" << 8)));
 }
 
-// Test a deep match tree that is not deep enough to hit the maximum depth limit.
-TEST(MatchExpressionParserTreeTest, MaximumTreeDepthNotExceed) {
-    static const int depth = 60;
-
-    std::stringstream ss;
-    for (int i = 0; i < depth / 2; i++) {
-        ss << "{$and: [{a: 3}, {$or: [{b: 2},";
-    }
-    ss << "{b: 4}";
-    for (int i = 0; i < depth / 2; i++) {
-        ss << "]}]}";
-    }
-
-    BSONObj query = fromjson(ss.str());
-    const CollatorInterface* collator = nullptr;
-    StatusWithMatchExpression result =
-        MatchExpressionParser::parse(query, ExtensionsCallbackDisallowExtensions(), collator);
-    ASSERT(result.isOK());
-}
-
-// Test a tree that exceeds the maximum depth limit.
-TEST(MatchExpressionParserTreeTest, MaximumTreeDepthExceed) {
-    static const int depth = 105;
-
-    std::stringstream ss;
-    for (int i = 0; i < depth / 2; i++) {
-        ss << "{$and: [{a: 3}, {$or: [{b: 2},";
-    }
-    ss << "{b: 4}";
-    for (int i = 0; i < depth / 2; i++) {
-        ss << "]}]}";
-    }
-
-    BSONObj query = fromjson(ss.str());
-    const CollatorInterface* collator = nullptr;
-    StatusWithMatchExpression result =
-        MatchExpressionParser::parse(query, ExtensionsCallbackDisallowExtensions(), collator);
-    ASSERT_FALSE(result.isOK());
-}
-
-// We should also exceed the depth limit through deeply nested $not.
-TEST(MatchExpressionParserTreeTest, MaximumTreeDepthExceededNestedNots) {
-    static const int depth = 105;
-
-    std::stringstream ss;
-    ss << "{a: ";
-    for (int i = 0; i < depth; i++) {
-        ss << "{$not: ";
-    }
-    ss << "{$eq: 5}";
-    for (int i = 0; i < depth + 1; i++) {
-        ss << "}";
-    }
-
-    BSONObj query = fromjson(ss.str());
-    const CollatorInterface* collator = nullptr;
-    StatusWithMatchExpression result =
-        MatchExpressionParser::parse(query, ExtensionsCallbackDisallowExtensions(), collator);
-    ASSERT_FALSE(result.isOK());
-}
-
-// Depth limit with nested $elemMatch object.
-TEST(MatchExpressionParserTreeTest, MaximumTreeDepthExceededNestedElemMatch) {
-    static const int depth = 105;
-
-    std::stringstream ss;
-    for (int i = 0; i < depth; i++) {
-        ss << "{a: {$elemMatch: ";
-    }
-    ss << "{b: 5}";
-    for (int i = 0; i < depth; i++) {
-        ss << "}}";
-    }
-
-    BSONObj query = fromjson(ss.str());
-    const CollatorInterface* collator = nullptr;
-    StatusWithMatchExpression result =
-        MatchExpressionParser::parse(query, ExtensionsCallbackDisallowExtensions(), collator);
-    ASSERT_FALSE(result.isOK());
-}
-
 TEST(MatchExpressionParserLeafTest, NotRegex1) {
     BSONObjBuilder b;
     b.appendRegex("$not", "abc", "i");
