@@ -407,7 +407,7 @@ StatusWith<PrepareExecutionResult> prepareExecution(OperationContext* opCtx,
                 root.reset(rawRoot);
 
                 LOG(2) << "Using fast count: " << redact(canonicalQuery->toStringShort())
-                       << ", planSummary: " << redact(Explain::getPlanSummary(root.get()));
+                       << ", planSummary: " << Explain::getPlanSummary(root.get());
 
                 querySolution.reset(solutions[i]);
                 return PrepareExecutionResult(
@@ -425,7 +425,7 @@ StatusWith<PrepareExecutionResult> prepareExecution(OperationContext* opCtx,
 
         LOG(2) << "Only one plan is available; it will be run but will not be cached. "
                << redact(canonicalQuery->toStringShort())
-               << ", planSummary: " << redact(Explain::getPlanSummary(root.get()));
+               << ", planSummary: " << Explain::getPlanSummary(root.get());
 
         querySolution.reset(solutions[0]);
         return PrepareExecutionResult(
@@ -1354,21 +1354,38 @@ bool turnIxscanIntoDistinctIxscan(QuerySolution* soln, const string& field) {
         return false;
     }
 
-    // Make a new DistinctNode. We will swap this for the ixscan in the provided solution.
-    auto distinctNode = stdx::make_unique<DistinctNode>(indexScanNode->index);
-    distinctNode->direction = indexScanNode->direction;
-    distinctNode->bounds = indexScanNode->bounds;
-
     // Figure out which field we're skipping to the next value of.
-    distinctNode->fieldNo = 0;
+    int fieldNo = 0;
     BSONObjIterator it(indexScanNode->index.keyPattern);
     while (it.more()) {
         if (field == it.next().fieldName()) {
             break;
         }
-        distinctNode->fieldNo++;
+        ++fieldNo;
     }
 
+    // We should not use a distinct scan if the field over which we are computing the distinct is
+    // multikey.
+    if (indexScanNode->index.multikey) {
+        const auto& multikeyPaths = indexScanNode->index.multikeyPaths;
+        if (multikeyPaths.empty()) {
+            // We don't have path-level multikey information available.
+            return false;
+        }
+
+        if (!multikeyPaths[fieldNo].empty()) {
+            // Path-level multikey information indicates that the distinct key contains at least one
+            // array component.
+            return false;
+        }
+    }
+
+    // Make a new DistinctNode. We will swap this for the ixscan in the provided solution.
+    auto distinctNode = stdx::make_unique<DistinctNode>(indexScanNode->index);
+    distinctNode->direction = indexScanNode->direction;
+    distinctNode->bounds = indexScanNode->bounds;
+    distinctNode->fieldNo = fieldNo;
+
     if (fetchNode) {
         // If there is a fetch node, then there is no need for the projection. The fetch node should
         // become the new root, with the distinct as its child. The PROJECT=>FETCH=>IXSCAN tree
@@ -1519,7 +1536,7 @@ StatusWith<unique_ptr<PlanExecutor>> getExecutorDistinct(OperationContext* txn,
         unique_ptr<PlanStage> root(rawRoot);
 
         LOG(2) << "Using fast distinct: " << redact(cq->toStringShort())
-               << ", planSummary: " << redact(Explain::getPlanSummary(root.get()));
+               << ", planSummary: " << Explain::getPlanSummary(root.get());
 
         return PlanExecutor::make(txn,
                                   std::move(ws),
@@ -1555,7 +1572,7 @@ StatusWith<unique_ptr<PlanExecutor>> getExecutorDistinct(OperationContext* txn,
             unique_ptr<PlanStage> root(rawRoot);
 
             LOG(2) << "Using fast distinct: " << redact(cq->toStringShort())
-                   << ", planSummary: " << redact(Explain::getPlanSummary(root.get()));
+                   << ", planSummary: " << Explain::getPlanSummary(root.get());
 
             return PlanExecutor::make(txn,
                                       std::move(ws),
