@@ -50,6 +50,7 @@
 #include "mongo/db/repl/rs_sync.h"
 #include "mongo/db/repl/storage_interface.h"
 #include "mongo/db/s/shard_identity_rollback_notifier.h"
+#include "mongo/rpc/get_status_from_command_result.h"
 #include "mongo/rpc/metadata/repl_set_metadata.h"
 #include "mongo/stdx/memory.h"
 #include "mongo/util/fail_point_service.h"
@@ -79,7 +80,8 @@ public:
         ReplicationCoordinatorExternalState* replicationCoordinatorExternalState,
         BackgroundSync* bgsync);
     bool shouldStopFetching(const HostAndPort& source,
-                            const rpc::ReplSetMetadata& metadata) override;
+                            const rpc::ReplSetMetadata& replMetadata,
+                            boost::optional<rpc::OplogQueryMetadata> oqMetadata) override;
 
 private:
     BackgroundSync* _bgsync;
@@ -93,12 +95,14 @@ DataReplicatorExternalStateBackgroundSync::DataReplicatorExternalStateBackground
       _bgsync(bgsync) {}
 
 bool DataReplicatorExternalStateBackgroundSync::shouldStopFetching(
-    const HostAndPort& source, const rpc::ReplSetMetadata& metadata) {
+    const HostAndPort& source,
+    const rpc::ReplSetMetadata& replMetadata,
+    boost::optional<rpc::OplogQueryMetadata> oqMetadata) {
     if (_bgsync->shouldStopFetching()) {
         return true;
     }
 
-    return DataReplicatorExternalStateImpl::shouldStopFetching(source, metadata);
+    return DataReplicatorExternalStateImpl::shouldStopFetching(source, replMetadata, oqMetadata);
 }
 
 size_t getSize(const BSONObj& o) {
@@ -107,7 +111,8 @@ size_t getSize(const BSONObj& o) {
 }
 }  // namespace
 
-MONGO_FP_DECLARE(pauseRsBgSyncProducer);
+// Failpoint which causes rollback to hang before starting.
+MONGO_FP_DECLARE(rollbackHangBeforeStart);
 
 // The count of items in the buffer
 static Counter64 bufferCountGauge;
@@ -129,9 +134,7 @@ BackgroundSync::BackgroundSync(
     std::unique_ptr<OplogBuffer> oplogBuffer)
     : _oplogBuffer(std::move(oplogBuffer)),
       _replCoord(getGlobalReplicationCoordinator()),
-      _replicationCoordinatorExternalState(replicationCoordinatorExternalState),
-      _lastOpTimeFetched(Timestamp(std::numeric_limits<int>::max(), 0),
-                         std::numeric_limits<long long>::max()) {
+      _replicationCoordinatorExternalState(replicationCoordinatorExternalState) {
     // Update "repl.buffer.maxSizeBytes" server status metric to reflect the current oplog buffer's
     // max size.
     bufferMaxSizeGauge.increment(_oplogBuffer->getMaxSize() - bufferMaxSizeGauge.get());
@@ -151,7 +154,7 @@ void BackgroundSync::shutdown(OperationContext* txn) {
     // ensures that it won't add anything. It will also unblock the OpApplier pipeline if it is
     // waiting for an operation to be past the slaveDelay point.
     clearBuffer(txn);
-    _stopped = true;
+    _state = ProducerState::Stopped;
 
     if (_syncSourceResolver) {
         _syncSourceResolver->shutdown();
@@ -196,47 +199,25 @@ void BackgroundSync::_run() {
             fassertFailed(28546);
         }
     }
-    stop();
-}
-
-void BackgroundSync::_signalNoNewDataForApplier(OperationContext* txn) {
-    // Signal to consumers that we have entered the stopped state
-    // if the signal isn't already in the queue.
-    const boost::optional<BSONObj> lastObjectPushed = _oplogBuffer->lastObjectPushed(txn);
-    if (!lastObjectPushed || !lastObjectPushed->isEmpty()) {
-        const BSONObj sentinelDoc;
-        _oplogBuffer->pushEvenIfFull(txn, sentinelDoc);
-        bufferCountGauge.increment();
-        bufferSizeGauge.increment(sentinelDoc.objsize());
-    }
+    stop(true);
 }
 
 void BackgroundSync::_runProducer() {
-    const MemberState state = _replCoord->getMemberState();
-    // Stop when the state changes to primary.
-    //
-    // TODO(siyuan) Drain mode should imply we're the primary. Fix this condition and the one below
-    // after fixing step-down during drain mode.
-    if (!_replCoord->isCatchingUp() &&
-        (_replCoord->isWaitingForApplierToDrain() || state.primary())) {
-        if (!isStopped()) {
-            stop();
-        }
-        if (_replCoord->isWaitingForApplierToDrain()) {
-            auto txn = cc().makeOperationContext();
-            _signalNoNewDataForApplier(txn.get());
-        }
+    if (getState() == ProducerState::Stopped) {
         sleepsecs(1);
         return;
     }
 
     // TODO(spencer): Use a condition variable to await loading a config.
-    if (state.startup()) {
+    // TODO(siyuan): Control bgsync with producer state.
+    if (_replCoord->getMemberState().startup()) {
         // Wait for a config to be loaded
         sleepsecs(1);
         return;
     }
 
+    invariant(!_replCoord->getMemberState().rollback());
+
     // We need to wait until initial sync has started.
     if (_replCoord->getMyLastAppliedOpTime().isNull()) {
         sleepsecs(1);
@@ -244,21 +225,29 @@ void BackgroundSync::_runProducer() {
     }
     // we want to start when we're no longer primary
     // start() also loads _lastOpTimeFetched, which we know is set from the "if"
-    auto txn = cc().makeOperationContext();
-    if (isStopped()) {
-        start(txn.get());
+    {
+        auto opCtx = cc().makeOperationContext();
+        if (getState() == ProducerState::Starting) {
+            start(opCtx.get());
+        }
     }
-
-    _produce(txn.get());
+    _produce();
 }
 
-void BackgroundSync::_produce(OperationContext* txn) {
+void BackgroundSync::_produce() {
+    if (MONGO_FAIL_POINT(stopReplProducer)) {
+        // This log output is used in js tests so please leave it.
+        log() << "bgsync - stopReplProducer fail point "
+                 "enabled. Blocking until fail point is disabled.";
 
-    while (MONGO_FAIL_POINT(pauseRsBgSyncProducer)) {
-        if (inShutdown()) {
-            return;
-        }
-        sleepmillis(10);
+        // TODO(SERVER-27120): Remove the return statement and uncomment the while loop.
+        // Currently we cannot block here or we prevent primaries from being fully elected since
+        // we'll never call _signalNoNewDataForApplier.
+        //        while (MONGO_FAIL_POINT(stopReplProducer) && !inShutdown()) {
+        //            mongo::sleepsecs(1);
+        //        }
+        mongo::sleepsecs(1);
+        return;
     }
 
     // this oplog reader does not do a handshake because we don't want the server it's syncing
@@ -273,12 +262,7 @@ void BackgroundSync::_produce(OperationContext* txn) {
             return;
         }
 
-        if (!_replCoord->isCatchingUp() &&
-            (_replCoord->isWaitingForApplierToDrain() || _replCoord->getMemberState().primary())) {
-            return;
-        }
-
-        if (_inShutdown_inlock()) {
+        if (_state != ProducerState::Running) {
             return;
         }
     }
@@ -286,40 +270,41 @@ void BackgroundSync::_produce(OperationContext* txn) {
     // find a target to sync from the last optime fetched
     OpTime lastOpTimeFetched;
     HostAndPort source;
+    HostAndPort oldSource = _syncSourceHost;
     SyncSourceResolverResponse syncSourceResp;
-    SyncSourceResolver* syncSourceResolver;
-    OpTime minValid;
-    OpTime minValidSaved;
-    if (_replCoord->getMemberState().recovering()) {
-        minValidSaved = StorageInterface::get(txn)->getMinValid(txn);
-    }
     {
-        stdx::lock_guard<stdx::mutex> lock(_mutex);
-        if (minValidSaved > _lastOpTimeFetched) {
-            minValid = minValidSaved;
+        OpTime minValidSaved;
+        {
+            auto opCtx = cc().makeOperationContext();
+            auto storageInterface = StorageInterface::get(opCtx.get());
+            minValidSaved = storageInterface->getMinValid(opCtx.get());
         }
+        stdx::lock_guard<stdx::mutex> lock(_mutex);
+        if (_state != ProducerState::Running) {
+            return;
+        }
+        const auto requiredOpTime = (minValidSaved > _lastOpTimeFetched) ? minValidSaved : OpTime();
         lastOpTimeFetched = _lastOpTimeFetched;
         _syncSourceHost = HostAndPort();
         _syncSourceResolver = stdx::make_unique<SyncSourceResolver>(
             _replicationCoordinatorExternalState->getTaskExecutor(),
             _replCoord,
             lastOpTimeFetched,
-            minValid,
+            requiredOpTime,
             [&syncSourceResp](const SyncSourceResolverResponse& resp) { syncSourceResp = resp; });
-        syncSourceResolver = _syncSourceResolver.get();
     }
     // This may deadlock if called inside the mutex because SyncSourceResolver::startup() calls
     // ReplicationCoordinator::chooseNewSyncSource(). ReplicationCoordinatorImpl's mutex has to
     // acquired before BackgroundSync's.
     // It is safe to call startup() outside the mutex on this instance of SyncSourceResolver because
-    // we do not destroy this instance outside of this function.
+    // we do not destroy this instance outside of this function which is only called from a single
+    // thread.
     auto status = _syncSourceResolver->startup();
     if (ErrorCodes::CallbackCanceled == status || ErrorCodes::isShutdownError(status.code())) {
         return;
     }
     fassertStatusOK(40349, status);
-    syncSourceResolver->join();
-    syncSourceResolver = nullptr;
+    _syncSourceResolver->join();
     {
         stdx::lock_guard<stdx::mutex> lock(_mutex);
         _syncSourceResolver.reset();
@@ -327,19 +312,29 @@ void BackgroundSync::_produce(OperationContext* txn) {
 
     if (syncSourceResp.syncSourceStatus == ErrorCodes::OplogStartMissing) {
         // All (accessible) sync sources were too stale.
-        if (_replCoord->isCatchingUp()) {
+        if (_replCoord->getMemberState().primary()) {
             warning() << "Too stale to catch up.";
             log() << "Our newest OpTime : " << lastOpTimeFetched;
             log() << "Earliest OpTime available is " << syncSourceResp.earliestOpTimeSeen
                   << " from " << syncSourceResp.getSyncSource();
-            sleepsecs(1);
+            _replCoord->abortCatchupIfNeeded();
             return;
         }
 
+        // We only need to mark ourselves as too stale once.
+        if (_tooStale) {
+            return;
+        }
+
+        // Mark yourself as too stale.
+        _tooStale = true;
+
         error() << "too stale to catch up -- entering maintenance mode";
         log() << "Our newest OpTime : " << lastOpTimeFetched;
         log() << "Earliest OpTime available is " << syncSourceResp.earliestOpTimeSeen;
         log() << "See http://dochub.mongodb.org/core/resyncingaverystalereplicasetmember";
+
+        // Activate maintenance mode and transition to RECOVERING.
         auto status = _replCoord->setMaintenanceMode(true);
         if (!status.isOK()) {
             warning() << "Failed to transition into maintenance mode: " << status;
@@ -351,9 +346,20 @@ void BackgroundSync::_produce(OperationContext* txn) {
         }
         return;
     } else if (syncSourceResp.isOK() && !syncSourceResp.getSyncSource().empty()) {
-        stdx::lock_guard<stdx::mutex> lock(_mutex);
-        _syncSourceHost = syncSourceResp.getSyncSource();
-        source = _syncSourceHost;
+        {
+            stdx::lock_guard<stdx::mutex> lock(_mutex);
+            _syncSourceHost = syncSourceResp.getSyncSource();
+            source = _syncSourceHost;
+        }
+        // If our sync source has not changed, it is likely caused by our heartbeat data map being
+        // out of date. In that case we sleep for 1 second to reduce the amount we spin waiting
+        // for our map to update.
+        if (oldSource == source) {
+            log() << "Chose same sync source candidate as last time, " << source
+                  << ". Sleeping for 1 second to avoid immediately choosing a new sync source for "
+                     "the same reason as last time.";
+            sleepsecs(1);
+        }
     } else {
         if (!syncSourceResp.isOK()) {
             log() << "failed to find sync source, received error "
@@ -364,24 +370,43 @@ void BackgroundSync::_produce(OperationContext* txn) {
         return;
     }
 
+    // If we find a good sync source after having gone too stale, disable maintenance mode so we can
+    // transition to SECONDARY.
+    if (_tooStale) {
+
+        _tooStale = false;
+
+        log() << "No longer too stale. Able to sync from " << _syncSourceHost;
+
+        auto status = _replCoord->setMaintenanceMode(false);
+        if (!status.isOK()) {
+            warning() << "Failed to leave maintenance mode: " << status;
+        }
+    }
+
     long long lastHashFetched;
     {
         stdx::lock_guard<stdx::mutex> lock(_mutex);
-        if (_stopped) {
+        if (_state != ProducerState::Running) {
             return;
         }
         lastOpTimeFetched = _lastOpTimeFetched;
         lastHashFetched = _lastFetchedHash;
-        if (!_replCoord->isCatchingUp()) {
-            _replCoord->signalUpstreamUpdater();
-        }
+    }
+
+    if (!_replCoord->getMemberState().primary()) {
+        _replCoord->signalUpstreamUpdater();
     }
 
     // Set the applied point if unset. This is most likely the first time we've established a sync
     // source since stepping down or otherwise clearing the applied point. We need to set this here,
     // before the OplogWriter gets a chance to append to the oplog.
-    if (StorageInterface::get(txn)->getAppliedThrough(txn).isNull()) {
-        StorageInterface::get(txn)->setAppliedThrough(txn, _replCoord->getMyLastAppliedOpTime());
+    {
+        auto opCtx = cc().makeOperationContext();
+        auto storageInterface = StorageInterface::get(opCtx.get());
+        if (storageInterface->getAppliedThrough(opCtx.get()).isNull()) {
+            storageInterface->setAppliedThrough(opCtx.get(), _replCoord->getMyLastAppliedOpTime());
+        }
     }
 
     // "lastFetched" not used. Already set in _enqueueDocuments.
@@ -390,21 +415,21 @@ void BackgroundSync::_produce(OperationContext* txn) {
         _replCoord, _replicationCoordinatorExternalState, this);
     OplogFetcher* oplogFetcher;
     try {
-        auto executor = _replicationCoordinatorExternalState->getTaskExecutor();
-        auto config = _replCoord->getConfig();
         auto onOplogFetcherShutdownCallbackFn =
             [&fetcherReturnStatus](const Status& status, const OpTimeWithHash& lastFetched) {
                 fetcherReturnStatus = status;
             };
-
-        stdx::lock_guard<stdx::mutex> lock(_mutex);
-        _oplogFetcher = stdx::make_unique<OplogFetcher>(
-            executor,
+        // The construction of OplogFetcher has to be outside bgsync mutex, because it calls
+        // replication coordinator.
+        auto oplogFetcherPtr = stdx::make_unique<OplogFetcher>(
+            _replicationCoordinatorExternalState->getTaskExecutor(),
             OpTimeWithHash(lastHashFetched, lastOpTimeFetched),
             source,
             NamespaceString(rsOplogName),
-            config,
-            _replicationCoordinatorExternalState->getOplogFetcherMaxFetcherRestarts(),
+            _replCoord->getConfig(),
+            _replicationCoordinatorExternalState->getOplogFetcherSteadyStateMaxFetcherRestarts(),
+            syncSourceResp.rbid,
+            true /* requireFresherSyncSource */,
             &dataReplicatorExternalState,
             stdx::bind(&BackgroundSync::_enqueueDocuments,
                        this,
@@ -412,13 +437,19 @@ void BackgroundSync::_produce(OperationContext* txn) {
                        stdx::placeholders::_2,
                        stdx::placeholders::_3),
             onOplogFetcherShutdownCallbackFn);
+        stdx::lock_guard<stdx::mutex> lock(_mutex);
+        if (_state != ProducerState::Running) {
+            return;
+        }
+        _oplogFetcher = std::move(oplogFetcherPtr);
         oplogFetcher = _oplogFetcher.get();
     } catch (const mongo::DBException& ex) {
         fassertFailedWithStatus(34440, exceptionToStatus());
     }
 
-    LOG(1) << "scheduling fetcher to read remote oplog on " << _syncSourceHost << " starting at "
-           << oplogFetcher->getCommandObject_forTest()["filter"];
+    const auto logLevel = Command::testCommandsEnabled ? 0 : 1;
+    LOG(logLevel) << "scheduling fetcher to read remote oplog on " << _syncSourceHost
+                  << " starting at " << oplogFetcher->getCommandObject_forTest()["filter"];
     auto scheduleStatus = oplogFetcher->startup();
     if (!scheduleStatus.isOK()) {
         warning() << "unable to schedule fetcher to read remote oplog on " << source << ": "
@@ -431,7 +462,10 @@ void BackgroundSync::_produce(OperationContext* txn) {
 
     // If the background sync is stopped after the fetcher is started, we need to
     // re-evaluate our sync source and oplog common point.
-    if (isStopped()) {
+    if (getState() != ProducerState::Running) {
+        log() << "Replication producer stopped after oplog fetcher finished returning a batch from "
+                 "our sync source.  Abandoning this batch of oplog entries and re-evaluating our "
+                 "sync source.";
         return;
     }
 
@@ -443,11 +477,10 @@ void BackgroundSync::_produce(OperationContext* txn) {
         // Do not blacklist the server here, it will be blacklisted when we try to reuse it,
         // if it can't return a matching oplog start from the last fetch oplog ts field.
         return;
-    } else if (fetcherReturnStatus.code() == ErrorCodes::OplogStartMissing ||
-               fetcherReturnStatus.code() == ErrorCodes::RemoteOplogStale) {
-        if (_replCoord->isCatchingUp()) {
-            warning() << "Rollback situation detected in catch-up mode; catch-up mode will end.";
-            sleepsecs(1);
+    } else if (fetcherReturnStatus.code() == ErrorCodes::OplogStartMissing) {
+        if (_replCoord->getMemberState().primary()) {
+            warning() << "Rollback situation detected in catch-up mode. Aborting catch-up mode.";
+            _replCoord->abortCatchupIfNeeded();
             return;
         }
 
@@ -471,6 +504,9 @@ void BackgroundSync::_produce(OperationContext* txn) {
 
         log() << "Starting rollback due to " << redact(fetcherReturnStatus);
 
+        // TODO: change this to call into the Applier directly to block until the applier is
+        // drained.
+        //
         // Wait till all buffered oplog entries have drained and been applied.
         auto lastApplied = _replCoord->getMyLastAppliedOpTime();
         if (lastApplied != lastOpTimeFetched) {
@@ -478,26 +514,37 @@ void BackgroundSync::_produce(OperationContext* txn) {
                   << lastOpTimeFetched << " to be applied before starting rollback.";
             while (lastOpTimeFetched > (lastApplied = _replCoord->getMyLastAppliedOpTime())) {
                 sleepmillis(10);
-                if (isStopped() || inShutdown()) {
+                if (getState() != ProducerState::Running) {
                     return;
                 }
             }
         }
-        // check that we are at minvalid, otherwise we cannot roll back as we may be in an
-        // inconsistent state
-        const auto minValid = StorageInterface::get(txn)->getMinValid(txn);
-        if (lastApplied < minValid) {
-            fassertNoTrace(18750,
-                           Status(ErrorCodes::UnrecoverableRollbackError,
-                                  str::stream() << "need to rollback, but in inconsistent state. "
-                                                << "minvalid: "
-                                                << minValid.toString()
-                                                << " > our last optime: "
-                                                << lastApplied.toString()));
+
+        if (MONGO_FAIL_POINT(rollbackHangBeforeStart)) {
+            // This log output is used in js tests so please leave it.
+            log() << "rollback - rollbackHangBeforeStart fail point "
+                     "enabled. Blocking until fail point is disabled.";
+            while (MONGO_FAIL_POINT(rollbackHangBeforeStart) && !inShutdown()) {
+                mongo::sleepsecs(1);
+            }
         }
 
-        _rollback(txn, source, getConnection);
-        stop();
+        {
+            auto opCtx = cc().makeOperationContext();
+            OplogInterfaceLocal localOplog(opCtx.get(), rsOplogName);
+            RollbackSourceImpl rollbackSource(getConnection, source, rsOplogName);
+            auto storageInterface = StorageInterface::get(opCtx.get());
+            rollback(opCtx.get(),
+                     localOplog,
+                     rollbackSource,
+                     syncSourceResp.rbid,
+                     _replCoord,
+                     storageInterface);
+        }
+
+        // Reset the producer to clear the sync source and the last optime fetched.
+        stop(true);
+        startProducerIfStopped();
     } else if (fetcherReturnStatus == ErrorCodes::InvalidBSON) {
         Seconds blacklistDuration(60);
         warning() << "Fetcher got invalid BSON while querying oplog. Blacklisting sync source "
@@ -509,14 +556,14 @@ void BackgroundSync::_produce(OperationContext* txn) {
     }
 }
 
-void BackgroundSync::_enqueueDocuments(Fetcher::Documents::const_iterator begin,
-                                       Fetcher::Documents::const_iterator end,
-                                       const OplogFetcher::DocumentsInfo& info) {
+Status BackgroundSync::_enqueueDocuments(Fetcher::Documents::const_iterator begin,
+                                         Fetcher::Documents::const_iterator end,
+                                         const OplogFetcher::DocumentsInfo& info) {
     // If this is the first batch of operations returned from the query, "toApplyDocumentCount" will
     // be one fewer than "networkDocumentCount" because the first document (which was applied
     // previously) is skipped.
     if (info.toApplyDocumentCount == 0) {
-        return;  // Nothing to do.
+        return Status::OK();  // Nothing to do.
     }
 
     auto txn = cc().makeOperationContext();
@@ -530,8 +577,8 @@ void BackgroundSync::_enqueueDocuments(Fetcher::Documents::const_iterator begin,
         // buffer between the time we check _inShutdown and the point where we finish writing to the
         // buffer.
         stdx::unique_lock<stdx::mutex> lock(_mutex);
-        if (_inShutdown) {
-            return;
+        if (_state != ProducerState::Running) {
+            return Status::OK();
         }
 
         OCCASIONALLY {
@@ -560,6 +607,8 @@ void BackgroundSync::_enqueueDocuments(Fetcher::Documents::const_iterator begin,
         // The inference here is basically if the batch is really small, we are "caught up".
         sleepmillis(kSleepToAllowBatchingMillis);
     }
+
+    return Status::OK();
 }
 
 bool BackgroundSync::peek(OperationContext* txn, BSONObj* op) {
@@ -586,63 +635,6 @@ void BackgroundSync::consume(OperationContext* txn) {
     }
 }
 
-void BackgroundSync::_rollback(OperationContext* txn,
-                               const HostAndPort& source,
-                               stdx::function<DBClientBase*()> getConnection) {
-    // Abort only when syncRollback detects we are in a unrecoverable state.
-    // In other cases, we log the message contained in the error status and retry later.
-    auto status = syncRollback(txn,
-                               OplogInterfaceLocal(txn, rsOplogName),
-                               RollbackSourceImpl(getConnection, source, rsOplogName),
-                               _replCoord);
-    if (status.isOK()) {
-        // When the syncTail thread sees there is no new data by adding something to the buffer.
-        _signalNoNewDataForApplier(txn);
-        // Wait until the buffer is empty.
-        // This is an indication that syncTail has removed the sentinal marker from the buffer
-        // and reset its local lastAppliedOpTime via the replCoord.
-        while (!_oplogBuffer->isEmpty()) {
-            sleepmillis(10);
-            if (inShutdown()) {
-                return;
-            }
-        }
-
-        // At this point we are about to leave rollback.  Before we do, wait for any writes done
-        // as part of rollback to be durable, and then do any necessary checks that we didn't
-        // wind up rolling back something illegal.  We must wait for the rollback to be durable
-        // so that if we wind up shutting down uncleanly in response to something we rolled back
-        // we know that we won't wind up right back in the same situation when we start back up
-        // because the rollback wasn't durable.
-        txn->recoveryUnit()->waitUntilDurable();
-
-        // If we detected that we rolled back the shardIdentity document as part of this rollback
-        // then we must shut down to clear the in-memory ShardingState associated with the
-        // shardIdentity document.
-        if (ShardIdentityRollbackNotifier::get(txn)->didRollbackHappen()) {
-            severe()
-                << "shardIdentity document rollback detected.  Shutting down to clear "
-                   "in-memory sharding state.  Restarting this process should safely return it "
-                   "to a healthy state";
-            fassertFailedNoTrace(40276);
-        }
-
-        // It is now safe to clear the ROLLBACK state, which may result in the applier thread
-        // transitioning to SECONDARY.  This is safe because the applier thread has now reloaded
-        // the new rollback minValid from the database.
-        if (!_replCoord->setFollowerMode(MemberState::RS_RECOVERING)) {
-            warning() << "Failed to transition into " << MemberState(MemberState::RS_RECOVERING)
-                      << "; expected to be in state " << MemberState(MemberState::RS_ROLLBACK)
-                      << " but found self in " << _replCoord->getMemberState();
-        }
-        return;
-    }
-    if (ErrorCodes::UnrecoverableRollbackError == status.code()) {
-        fassertNoTrace(28723, status);
-    }
-    warning() << "rollback cannot proceed at this time (retrying later): " << redact(status);
-}
-
 HostAndPort BackgroundSync::getSyncTarget() const {
     stdx::unique_lock<stdx::mutex> lock(_mutex);
     return _syncSourceHost;
@@ -653,20 +645,20 @@ void BackgroundSync::clearSyncTarget() {
     _syncSourceHost = HostAndPort();
 }
 
-void BackgroundSync::cancelFetcher() {
-    stdx::lock_guard<stdx::mutex> lock(_mutex);
-    if (_oplogFetcher) {
-        _oplogFetcher->shutdown();
-    }
-}
-
-void BackgroundSync::stop() {
+void BackgroundSync::stop(bool resetLastFetchedOptime) {
     stdx::lock_guard<stdx::mutex> lock(_mutex);
 
-    _stopped = true;
+    _state = ProducerState::Stopped;
     _syncSourceHost = HostAndPort();
-    _lastOpTimeFetched = OpTime();
-    _lastFetchedHash = 0;
+    if (resetLastFetchedOptime) {
+        invariant(_oplogBuffer->isEmpty());
+        _lastOpTimeFetched = OpTime();
+        _lastFetchedHash = 0;
+    }
+
+    if (_syncSourceResolver) {
+        _syncSourceResolver->shutdown();
+    }
 
     if (_oplogFetcher) {
         _oplogFetcher->shutdown();
@@ -674,24 +666,33 @@ void BackgroundSync::stop() {
 }
 
 void BackgroundSync::start(OperationContext* txn) {
-    massert(16235, "going to start syncing, but buffer is not empty", _oplogBuffer->isEmpty());
+    OpTimeWithHash lastAppliedOpTimeWithHash;
+    do {
+        lastAppliedOpTimeWithHash = _readLastAppliedOpTimeWithHash(txn);
+        stdx::lock_guard<stdx::mutex> lk(_mutex);
+        // Double check the state after acquiring the mutex.
+        if (_state != ProducerState::Starting) {
+            return;
+        }
+        // If a node steps down during drain mode, then the buffer may not be empty at the beginning
+        // of secondary state.
+        if (!_oplogBuffer->isEmpty()) {
+            log() << "going to start syncing, but buffer is not empty";
+        }
+        _state = ProducerState::Running;
 
-    long long lastFetchedHash = _readLastAppliedHash(txn);
-    stdx::lock_guard<stdx::mutex> lk(_mutex);
-    _stopped = false;
-
-    // reset _last fields with current oplog data
-    _lastOpTimeFetched = _replCoord->getMyLastAppliedOpTime();
-    _lastFetchedHash = lastFetchedHash;
+        // When a node steps down during drain mode, the last fetched optime would be newer than
+        // the last applied.
+        if (_lastOpTimeFetched <= lastAppliedOpTimeWithHash.opTime) {
+            _lastOpTimeFetched = lastAppliedOpTimeWithHash.opTime;
+            _lastFetchedHash = lastAppliedOpTimeWithHash.value;
+        }
+        // Reload the last applied optime from disk if it has been changed.
+    } while (lastAppliedOpTimeWithHash.opTime != _replCoord->getMyLastAppliedOpTime());
 
     LOG(1) << "bgsync fetch queue set to: " << _lastOpTimeFetched << " " << _lastFetchedHash;
 }
 
-bool BackgroundSync::isStopped() const {
-    stdx::lock_guard<stdx::mutex> lock(_mutex);
-    return _stopped;
-}
-
 void BackgroundSync::clearBuffer(OperationContext* txn) {
     _oplogBuffer->clear(txn);
     const auto count = bufferCountGauge.get();
@@ -700,7 +701,7 @@ void BackgroundSync::clearBuffer(OperationContext* txn) {
     bufferSizeGauge.decrement(size);
 }
 
-long long BackgroundSync::_readLastAppliedHash(OperationContext* txn) {
+OpTimeWithHash BackgroundSync::_readLastAppliedOpTimeWithHash(OperationContext* txn) {
     BSONObj oplogEntry;
     try {
         MONGO_WRITE_CONFLICT_RETRY_LOOP_BEGIN {
@@ -710,7 +711,7 @@ long long BackgroundSync::_readLastAppliedHash(OperationContext* txn) {
             if (!success) {
                 // This can happen when we are to do an initial sync.  lastHash will be set
                 // after the initial sync is complete.
-                return 0;
+                return OpTimeWithHash(0);
             }
         }
         MONGO_WRITE_CONFLICT_RETRY_LOOP_END(txn, "readLastAppliedHash", rsOplogName);
@@ -726,36 +727,21 @@ long long BackgroundSync::_readLastAppliedHash(OperationContext* txn) {
                  << redact(status);
         fassertFailed(18902);
     }
-    return hash;
+    OplogEntry parsedEntry(oplogEntry);
+    return OpTimeWithHash(hash, parsedEntry.getOpTime());
 }
 
 bool BackgroundSync::shouldStopFetching() const {
-    if (inShutdown()) {
-        LOG(2) << "Interrupted by shutdown while checking sync source.";
-        return true;
-    }
-
-    // If we are transitioning to primary state, we need to stop fetching in order to go into
-    // bgsync-stop mode.
-    if (_replCoord->isWaitingForApplierToDrain()) {
-        LOG(2) << "Interrupted by waiting for applier to drain while checking sync source.";
-        return true;
-    }
-
-    if (_replCoord->getMemberState().primary() && !_replCoord->isCatchingUp()) {
-        LOG(2) << "Interrupted by becoming primary while checking sync source.";
-        return true;
-    }
-
     // Check if we have been stopped.
-    if (isStopped()) {
-        LOG(2) << "Interrupted by a stop request while checking sync source.";
+    if (getState() != ProducerState::Running) {
+        LOG(2) << "Stopping oplog fetcher due to stop request.";
         return true;
     }
 
-    // Check current sync target.
+    // Check current sync source.
     if (getSyncTarget().empty()) {
-        LOG(1) << "Canceling oplog query because we have no valid sync source.";
+        LOG(1) << "Stopping oplog fetcher; canceling oplog query because we have no valid sync "
+                  "source.";
         return true;
     }
 
@@ -768,5 +754,19 @@ void BackgroundSync::pushTestOpToBuffer(OperationContext* txn, const BSONObj& op
     bufferSizeGauge.increment(op.objsize());
 }
 
+BackgroundSync::ProducerState BackgroundSync::getState() const {
+    stdx::lock_guard<stdx::mutex> lock(_mutex);
+    return _state;
+}
+
+void BackgroundSync::startProducerIfStopped() {
+    stdx::lock_guard<stdx::mutex> lock(_mutex);
+    // Let producer run if it's already running.
+    if (_state == ProducerState::Stopped) {
+        _state = ProducerState::Starting;
+    }
+}
+
+
 }  // namespace repl
 }  // namespace mongo
