@@ -114,7 +114,8 @@ DatabaseCloner::DatabaseCloner(executor::TaskExecutor* executor,
                                          stdx::placeholders::_2,
                                          stdx::placeholders::_3),
                               rpc::ServerSelectionMetadata(true, boost::none).toBSON(),
-                              RemoteCommandRequest::kNoTimeout,
+                              RemoteCommandRequest::kNoTimeout /* find network timeout */,
+                              RemoteCommandRequest::kNoTimeout /* getMore network timeout */,
                               RemoteCommandRetryScheduler::makeRetryPolicy(
                                   numInitialSyncListCollectionsAttempts,
                                   executor::RemoteCommandRequest::kNoTimeout,
@@ -152,7 +153,7 @@ std::string DatabaseCloner::_getDiagnosticString_inlock() const {
     output << " source: " << _source.toString();
     output << " database: " << _dbname;
     output << " listCollections filter" << _listCollectionsFilter;
-    output << " active: " << _active;
+    output << " active: " << _isActive_inlock();
     output << " collection info objects (empty if listCollections is in progress): "
            << _collectionInfos.size();
     return output;
@@ -160,14 +161,26 @@ std::string DatabaseCloner::_getDiagnosticString_inlock() const {
 
 bool DatabaseCloner::isActive() const {
     LockGuard lk(_mutex);
-    return _active;
+    return _isActive_inlock();
 }
 
-Status DatabaseCloner::startup() {
+bool DatabaseCloner::_isActive_inlock() const {
+    return State::kRunning == _state || State::kShuttingDown == _state;
+}
+
+Status DatabaseCloner::startup() noexcept {
     LockGuard lk(_mutex);
 
-    if (_active) {
-        return Status(ErrorCodes::IllegalOperation, "database cloner already started");
+    switch (_state) {
+        case State::kPreStart:
+            _state = State::kRunning;
+            break;
+        case State::kRunning:
+            return Status(ErrorCodes::InternalError, "database cloner already started");
+        case State::kShuttingDown:
+            return Status(ErrorCodes::ShutdownInProgress, "database cloner shutting down");
+        case State::kComplete:
+            return Status(ErrorCodes::ShutdownInProgress, "database cloner completed");
     }
 
     _stats.start = _executor->now();
@@ -176,21 +189,31 @@ Status DatabaseCloner::startup() {
     if (!scheduleResult.isOK()) {
         error() << "Error scheduling listCollections for database: " << _dbname
                 << ", error:" << scheduleResult;
+        _state = State::kComplete;
         return scheduleResult;
     }
 
-    _active = true;
-
     return Status::OK();
 }
 
 void DatabaseCloner::shutdown() {
-    {
-        LockGuard lk(_mutex);
-
-        if (!_active) {
+    stdx::lock_guard<stdx::mutex> lock(_mutex);
+    switch (_state) {
+        case State::kPreStart:
+            // Transition directly from PreStart to Complete if not started yet.
+            _state = State::kComplete;
             return;
-        }
+        case State::kRunning:
+            _state = State::kShuttingDown;
+            break;
+        case State::kShuttingDown:
+        case State::kComplete:
+            // Nothing to do if we are already in ShuttingDown or Complete state.
+            return;
+    }
+
+    for (auto&& collectionCloner : _collectionCloners) {
+        collectionCloner.shutdown();
     }
 
     _listCollectionsFetcher.shutdown();
@@ -207,7 +230,7 @@ DatabaseCloner::Stats DatabaseCloner::getStats() const {
 
 void DatabaseCloner::join() {
     UniqueLock lk(_mutex);
-    _condition.wait(lk, [this]() { return !_active; });
+    _condition.wait(lk, [this]() { return !_isActive_inlock(); });
 }
 
 void DatabaseCloner::setScheduleDbWorkFn_forTest(const CollectionCloner::ScheduleDbWorkFn& work) {
@@ -221,6 +244,11 @@ void DatabaseCloner::setStartCollectionClonerFn(
     _startCollectionCloner = startCollectionCloner;
 }
 
+DatabaseCloner::State DatabaseCloner::getState_forTest() const {
+    stdx::lock_guard<stdx::mutex> lk(_mutex);
+    return _state;
+}
+
 void DatabaseCloner::_listCollectionsCallback(const StatusWith<Fetcher::QueryResponse>& result,
                                               Fetcher::NextAction* nextAction,
                                               BSONObjBuilder* getMoreBob) {
@@ -409,7 +437,8 @@ void DatabaseCloner::_collectionClonerCallback(const Status& status, const Names
 void DatabaseCloner::_finishCallback(const Status& status) {
     _onCompletion(status);
     LockGuard lk(_mutex);
-    _active = false;
+    invariant(_state != State::kComplete);
+    _state = State::kComplete;
     _condition.notify_all();
     _stats.end = _executor->now();
     LOG(1) << "    database: " << _dbname << ", stats: " << _stats.toString();
@@ -457,5 +486,19 @@ void DatabaseCloner::Stats::append(BSONObjBuilder* builder) const {
     }
 }
 
+std::ostream& operator<<(std::ostream& os, const DatabaseCloner::State& state) {
+    switch (state) {
+        case DatabaseCloner::State::kPreStart:
+            return os << "PreStart";
+        case DatabaseCloner::State::kRunning:
+            return os << "Running";
+        case DatabaseCloner::State::kShuttingDown:
+            return os << "ShuttingDown";
+        case DatabaseCloner::State::kComplete:
+            return os << "Complete";
+    }
+    MONGO_UNREACHABLE;
+}
+
 }  // namespace repl
 }  // namespace mongo
