@@ -36,7 +36,9 @@
 #include "mongo/db/commands/server_status_metric.h"
 #include "mongo/db/jsobj.h"
 #include "mongo/db/repl/replication_coordinator.h"
+#include "mongo/db/server_parameters.h"
 #include "mongo/db/stats/timer_stats.h"
+#include "mongo/rpc/metadata/oplog_query_metadata.h"
 #include "mongo/rpc/metadata/server_selection_metadata.h"
 #include "mongo/stdx/memory.h"
 #include "mongo/stdx/mutex.h"
@@ -51,14 +53,42 @@ namespace repl {
 
 Seconds OplogFetcher::kDefaultProtocolZeroAwaitDataTimeout(2);
 
+MONGO_FP_DECLARE(stopReplProducer);
+
 namespace {
 
-MONGO_FP_DECLARE(stopOplogFetcher);
+// Number of seconds for the `maxTimeMS` on the initial `find` command.
+//
+// For the initial 'find' request, we provide a generous timeout, to account for the potentially
+// slow process of a sync source finding the lastApplied optime provided in a node's query in its
+// oplog.
+MONGO_EXPORT_SERVER_PARAMETER(oplogInitialFindMaxSeconds, int, 60);
+
+// Number of seconds for the `maxTimeMS` on any retried `find` commands.
+MONGO_EXPORT_SERVER_PARAMETER(oplogRetriedFindMaxSeconds, int, 2);
+
+// Number of milliseconds to add to the `find` and `getMore` timeouts to calculate the network
+// timeout for the requests.
+const Milliseconds kNetworkTimeoutBufferMS{5000};
+
+Counter64 readersCreatedStats;
+ServerStatusMetricField<Counter64> displayReadersCreated("repl.network.readersCreated",
+                                                         &readersCreatedStats);
+// The number and time spent reading batches off the network
+TimerStats getmoreReplStats;
+ServerStatusMetricField<TimerStats> displayBatchesRecieved("repl.network.getmores",
+                                                           &getmoreReplStats);
+// The oplog entries read via the oplog reader
+Counter64 opsReadStats;
+ServerStatusMetricField<Counter64> displayOpsRead("repl.network.ops", &opsReadStats);
+// The bytes read via the oplog reader
+Counter64 networkByteStats;
+ServerStatusMetricField<Counter64> displayBytesRead("repl.network.bytes", &networkByteStats);
 
 /**
  * Calculates await data timeout based on the current replica set configuration.
  */
-Milliseconds calculateAwaitDataTimeout(const ReplicaSetConfig& config) {
+Milliseconds calculateAwaitDataTimeout(const ReplSetConfig& config) {
     // Under protocol version 1, make the awaitData timeout (maxTimeMS) dependent on the election
     // timeout. This enables the sync source to communicate liveness of the primary to secondaries.
     // Under protocol version 0, use a default timeout of 2 seconds for awaitData.
@@ -71,20 +101,23 @@ Milliseconds calculateAwaitDataTimeout(const ReplicaSetConfig& config) {
 /**
  * Returns find command object suitable for tailing remote oplog.
  */
-BSONObj makeFindCommandObject(DataReplicatorExternalState* dataReplicatorExternalState,
-                              const NamespaceString& nss,
-                              OpTime lastOpTimeFetched) {
-    invariant(dataReplicatorExternalState);
+BSONObj makeFindCommandObject(const NamespaceString& nss,
+                              long long currentTerm,
+                              OpTime lastOpTimeFetched,
+                              Milliseconds fetcherMaxTimeMS) {
     BSONObjBuilder cmdBob;
     cmdBob.append("find", nss.coll());
     cmdBob.append("filter", BSON("ts" << BSON("$gte" << lastOpTimeFetched.getTimestamp())));
     cmdBob.append("tailable", true);
     cmdBob.append("oplogReplay", true);
     cmdBob.append("awaitData", true);
-    cmdBob.append("maxTimeMS", durationCount<Milliseconds>(Minutes(1)));  // 1 min initial find.
-    auto opTimeWithTerm = dataReplicatorExternalState->getCurrentTermAndLastCommittedOpTime();
-    if (opTimeWithTerm.value != OpTime::kUninitializedTerm) {
-        cmdBob.append("term", opTimeWithTerm.value);
+    cmdBob.append("maxTimeMS", durationCount<Milliseconds>(fetcherMaxTimeMS));
+    if (currentTerm != OpTime::kUninitializedTerm) {
+        cmdBob.append("term", currentTerm);
+    }
+    if (serverGlobalParams.featureCompatibility.version.load() ==
+        ServerGlobalParams::FeatureCompatibility::Version::k34) {
+        cmdBob.append("readConcern", BSON("afterOpTime" << lastOpTimeFetched.toBSON()));
     }
     return cmdBob.obj();
 }
@@ -92,18 +125,17 @@ BSONObj makeFindCommandObject(DataReplicatorExternalState* dataReplicatorExterna
 /**
  * Returns getMore command object suitable for tailing remote oplog.
  */
-BSONObj makeGetMoreCommandObject(DataReplicatorExternalState* dataReplicatorExternalState,
-                                 const NamespaceString& nss,
+BSONObj makeGetMoreCommandObject(const NamespaceString& nss,
                                  CursorId cursorId,
+                                 OpTimeWithTerm lastCommittedWithCurrentTerm,
                                  Milliseconds fetcherMaxTimeMS) {
     BSONObjBuilder cmdBob;
     cmdBob.append("getMore", cursorId);
     cmdBob.append("collection", nss.coll());
     cmdBob.append("maxTimeMS", durationCount<Milliseconds>(fetcherMaxTimeMS));
-    auto opTimeWithTerm = dataReplicatorExternalState->getCurrentTermAndLastCommittedOpTime();
-    if (opTimeWithTerm.value != OpTime::kUninitializedTerm) {
-        cmdBob.append("term", opTimeWithTerm.value);
-        opTimeWithTerm.opTime.append(&cmdBob, "lastKnownCommittedOpTime");
+    if (lastCommittedWithCurrentTerm.value != OpTime::kUninitializedTerm) {
+        cmdBob.append("term", lastCommittedWithCurrentTerm.value);
+        lastCommittedWithCurrentTerm.opTime.append(&cmdBob, "lastKnownCommittedOpTime");
     }
     return cmdBob.obj();
 }
@@ -114,6 +146,8 @@ BSONObj makeGetMoreCommandObject(DataReplicatorExternalState* dataReplicatorExte
 StatusWith<BSONObj> makeMetadataObject(bool isV1ElectionProtocol) {
     return isV1ElectionProtocol
         ? BSON(rpc::kReplSetMetadataFieldName
+               << 1
+               << rpc::kOplogQueryMetadataFieldName
                << 1
                << rpc::ServerSelectionMetadata::fieldName()
                << BSON(rpc::ServerSelectionMetadata::kSecondaryOkFieldName << true))
@@ -124,21 +158,85 @@ StatusWith<BSONObj> makeMetadataObject(bool isV1ElectionProtocol) {
  * Checks the first batch of results from query.
  * 'documents' are the first batch of results returned from tailing the remote oplog.
  * 'lastFetched' optime and hash should be consistent with the predicate in the query.
- * Returns RemoteOplogStale if the oplog query has no results.
+ * 'remoteLastOpApplied' is the last OpTime applied on the sync source. This is optional for
+ * compatibility with 3.4 servers that do not send OplogQueryMetadata.
+ * 'requiredRBID' is a RollbackID received when we chose the sync source that we use here to
+ * guarantee we have not rolled back since we confirmed the sync source had our minValid.
+ * 'remoteRBID' is a RollbackId for the sync source returned in this oplog query. This is optional
+ * for compatibility with 3.4 servers that do not send OplogQueryMetadata.
+ * 'requireFresherSyncSource' is a boolean indicating whether we should require the sync source's
+ * oplog to be ahead of ours. If false, the sync source's oplog is allowed to be at the same point
+ * as ours, but still cannot be behind ours.
+ *
+ * TODO (SERVER-27668): Make remoteLastOpApplied and remoteRBID non-optional in mongodb 3.8.
+ *
  * Returns OplogStartMissing if we cannot find the optime of the last fetched operation in
  * the remote oplog.
  */
-Status checkRemoteOplogStart(const Fetcher::Documents& documents, OpTimeWithHash lastFetched) {
-    if (documents.empty()) {
-        // The GTE query from upstream returns nothing, so we're ahead of the upstream.
-        return Status(ErrorCodes::RemoteOplogStale,
-                      str::stream() << "We are ahead of the sync source. Our last op time fetched: "
-                                    << lastFetched.opTime.toString());
+Status checkRemoteOplogStart(const Fetcher::Documents& documents,
+                             OpTimeWithHash lastFetched,
+                             boost::optional<OpTime> remoteLastOpApplied,
+                             int requiredRBID,
+                             boost::optional<int> remoteRBID,
+                             bool requireFresherSyncSource) {
+    // Once we establish our cursor, we need to ensure that our upstream node hasn't rolled back
+    // since that could cause it to not have our required minValid point. The cursor will be
+    // killed if the upstream node rolls back so we don't need to keep checking once the cursor
+    // is established.
+    if (remoteRBID && (*remoteRBID != requiredRBID)) {
+        return Status(ErrorCodes::InvalidSyncSource,
+                      "Upstream node rolled back after choosing it as a sync source. Choosing "
+                      "new sync source.");
     }
+
+    // Sometimes our remoteLastOpApplied may be stale; if we received a document with an
+    // opTime later than remoteLastApplied, we can assume the remote is at least up to that
+    // opTime.
+    if (remoteLastOpApplied && !documents.empty()) {
+        const auto docOpTime = OpTime::parseFromOplogEntry(documents.back());
+        if (docOpTime.isOK()) {
+            remoteLastOpApplied = std::max(*remoteLastOpApplied, docOpTime.getValue());
+        }
+    }
+
+    // The SyncSourceResolver never checks that the sync source candidate is actually ahead of
+    // us. Rather than have it check there with an extra network roundtrip, we check here.
+    if (requireFresherSyncSource && remoteLastOpApplied &&
+        (*remoteLastOpApplied <= lastFetched.opTime)) {
+        return Status(ErrorCodes::InvalidSyncSource,
+                      str::stream() << "Sync source's last applied OpTime "
+                                    << remoteLastOpApplied->toString()
+                                    << " is not greater than our last fetched OpTime "
+                                    << lastFetched.opTime.toString()
+                                    << ". Choosing new sync source.");
+    } else if (remoteLastOpApplied && (*remoteLastOpApplied < lastFetched.opTime)) {
+        // In initial sync, the lastFetched OpTime will almost always equal the remoteLastOpApplied
+        // since we fetch the sync source's last applied OpTime to determine where to start our
+        // OplogFetcher. This is fine since no other node can sync off of an initial syncing node
+        // and thus cannot form a sync source cycle. To account for this, we must relax the
+        // constraint on our sync source being fresher.
+        return Status(ErrorCodes::InvalidSyncSource,
+                      str::stream() << "Sync source's last applied OpTime "
+                                    << remoteLastOpApplied->toString()
+                                    << " is older than our last fetched OpTime "
+                                    << lastFetched.opTime.toString()
+                                    << ". Choosing new sync source.");
+    }
+
+    // At this point we know that our sync source has our minValid and is ahead of us, so if our
+    // history diverges from our sync source's we should prefer its history and roll back ours.
+
+    // Since we checked for rollback and our sync source is ahead of us, an empty batch means that
+    // we have a higher timestamp on our last fetched OpTime than our sync source's last applied
+    // OpTime, but a lower term. When this occurs, we must roll back our inconsistent oplog entry.
+    if (documents.empty()) {
+        return Status(ErrorCodes::OplogStartMissing, "Received an empty batch from sync source.");
+    }
+
     const auto& o = documents.front();
     auto opTimeResult = OpTime::parseFromOplogEntry(o);
     if (!opTimeResult.isOK()) {
-        return Status(ErrorCodes::OplogStartMissing,
+        return Status(ErrorCodes::InvalidBSON,
                       str::stream() << "our last op time fetched: " << lastFetched.opTime.toString()
                                     << " (hash: "
                                     << lastFetched.value
@@ -164,20 +262,30 @@ Status checkRemoteOplogStart(const Fetcher::Documents& documents, OpTimeWithHash
     return Status::OK();
 }
 
-Counter64 readersCreatedStats;
-ServerStatusMetricField<Counter64> displayReadersCreated("repl.network.readersCreated",
-                                                         &readersCreatedStats);
-// The number and time spent reading batches off the network
-TimerStats getmoreReplStats;
-ServerStatusMetricField<TimerStats> displayBatchesRecieved("repl.network.getmores",
-                                                           &getmoreReplStats);
-// The oplog entries read via the oplog reader
-Counter64 opsReadStats;
-ServerStatusMetricField<Counter64> displayOpsRead("repl.network.ops", &opsReadStats);
-// The bytes read via the oplog reader
-Counter64 networkByteStats;
-ServerStatusMetricField<Counter64> displayBytesRead("repl.network.bytes", &networkByteStats);
-
+/**
+ * Parses a QueryResponse for the OplogQueryMetadata. If there is an error it returns it. If
+ * no OplogQueryMetadata is provided then it returns boost::none.
+ *
+ * OplogQueryMetadata is made optional for backwards compatibility.
+ * TODO (SERVER-27668): Make this non-optional in mongodb 3.8. When this stops being optional
+ * we can remove the duplicated fields in both metadata types and begin to always use
+ * OplogQueryMetadata's data.
+ */
+StatusWith<boost::optional<rpc::OplogQueryMetadata>> parseOplogQueryMetadata(
+    Fetcher::QueryResponse queryResponse) {
+    boost::optional<rpc::OplogQueryMetadata> oqMetadata = boost::none;
+    bool receivedOplogQueryMetadata =
+        queryResponse.otherFields.metadata.hasElement(rpc::kOplogQueryMetadataFieldName);
+    if (receivedOplogQueryMetadata) {
+        const auto& metadataObj = queryResponse.otherFields.metadata;
+        auto metadataResult = rpc::OplogQueryMetadata::readFromMetadata(metadataObj);
+        if (!metadataResult.isOK()) {
+            return metadataResult.getStatus();
+        }
+        oqMetadata = boost::make_optional<rpc::OplogQueryMetadata>(metadataResult.getValue());
+    }
+    return oqMetadata;
+}
 }  // namespace
 
 StatusWith<OplogFetcher::DocumentsInfo> OplogFetcher::validateDocuments(
@@ -245,8 +353,10 @@ OplogFetcher::OplogFetcher(executor::TaskExecutor* executor,
                            OpTimeWithHash lastFetched,
                            HostAndPort source,
                            NamespaceString nss,
-                           ReplicaSetConfig config,
+                           ReplSetConfig config,
                            std::size_t maxFetcherRestarts,
+                           int requiredRBID,
+                           bool requireFresherSyncSource,
                            DataReplicatorExternalState* dataReplicatorExternalState,
                            EnqueueDocumentsFn enqueueDocumentsFn,
                            OnShutdownCallbackFn onShutdownCallbackFn)
@@ -254,20 +364,23 @@ OplogFetcher::OplogFetcher(executor::TaskExecutor* executor,
       _source(source),
       _nss(nss),
       _metadataObject(uassertStatusOK(makeMetadataObject(config.getProtocolVersion() == 1LL))),
-      _remoteCommandTimeout(config.getElectionTimeoutPeriod()),
       _maxFetcherRestarts(maxFetcherRestarts),
+      _requiredRBID(requiredRBID),
+      _requireFresherSyncSource(requireFresherSyncSource),
       _dataReplicatorExternalState(dataReplicatorExternalState),
       _enqueueDocumentsFn(enqueueDocumentsFn),
       _awaitDataTimeout(calculateAwaitDataTimeout(config)),
       _onShutdownCallbackFn(onShutdownCallbackFn),
-      _lastFetched(lastFetched),
-      _fetcher(_makeFetcher(_lastFetched.opTime)) {
+      _lastFetched(lastFetched) {
     uassert(ErrorCodes::BadValue, "null last optime fetched", !_lastFetched.opTime.isNull());
     uassert(ErrorCodes::InvalidReplicaSetConfig,
             "uninitialized replica set configuration",
             config.isInitialized());
     uassert(ErrorCodes::BadValue, "null enqueueDocuments function", enqueueDocumentsFn);
     uassert(ErrorCodes::BadValue, "null onShutdownCallback function", onShutdownCallbackFn);
+
+    auto currentTerm = dataReplicatorExternalState->getCurrentTermAndLastCommittedOpTime().value;
+    _fetcher = _makeFetcher(currentTerm, _lastFetched.opTime, _getInitialFindMaxTime());
 }
 
 OplogFetcher::~OplogFetcher() {
@@ -283,21 +396,30 @@ std::string OplogFetcher::toString() const {
 
 bool OplogFetcher::isActive() const {
     stdx::lock_guard<stdx::mutex> lock(_mutex);
-    return _active;
+    return _isActive_inlock();
+}
+
+bool OplogFetcher::_isActive_inlock() const {
+    return State::kRunning == _state || State::kShuttingDown == _state;
 }
 
 Status OplogFetcher::startup() {
     stdx::lock_guard<stdx::mutex> lock(_mutex);
-    if (_active) {
-        return Status(ErrorCodes::IllegalOperation, "oplog fetcher already active");
-    }
-    if (_inShutdown) {
-        return Status(ErrorCodes::ShutdownInProgress, "oplog fetcher shutting down");
+    switch (_state) {
+        case State::kPreStart:
+            _state = State::kRunning;
+            break;
+        case State::kRunning:
+            return Status(ErrorCodes::InternalError, "oplog fetcher already started");
+        case State::kShuttingDown:
+            return Status(ErrorCodes::ShutdownInProgress, "oplog fetcher shutting down");
+        case State::kComplete:
+            return Status(ErrorCodes::ShutdownInProgress, "oplog fetcher completed");
     }
 
     auto status = _scheduleFetcher_inlock();
-    if (status.isOK()) {
-        _active = true;
+    if (!status.isOK()) {
+        _state = State::kComplete;
     }
     return status;
 }
@@ -309,13 +431,25 @@ Status OplogFetcher::_scheduleFetcher_inlock() {
 
 void OplogFetcher::shutdown() {
     stdx::lock_guard<stdx::mutex> lock(_mutex);
-    _inShutdown = true;
+    switch (_state) {
+        case State::kPreStart:
+            // Transition directly from PreStart to Complete if not started yet.
+            _state = State::kComplete;
+            return;
+        case State::kRunning:
+            _state = State::kShuttingDown;
+            break;
+        case State::kShuttingDown:
+        case State::kComplete:
+            // Nothing to do if we are already in ShuttingDown or Complete state.
+            return;
+    }
     _fetcher->shutdown();
 }
 
 void OplogFetcher::join() {
     stdx::unique_lock<stdx::mutex> lock(_mutex);
-    _condition.wait(lock, [this]() { return !_active; });
+    _condition.wait(lock, [this]() { return !_isActive_inlock(); });
 }
 
 OpTimeWithHash OplogFetcher::getLastOpTimeWithHashFetched() const {
@@ -332,16 +466,25 @@ BSONObj OplogFetcher::getMetadataObject_forTest() const {
     return _metadataObject;
 }
 
-Milliseconds OplogFetcher::getRemoteCommandTimeout_forTest() const {
-    return _remoteCommandTimeout;
+Milliseconds OplogFetcher::getAwaitDataTimeout_forTest() const {
+    return _getGetMoreMaxTime();
 }
 
-Milliseconds OplogFetcher::getAwaitDataTimeout_forTest() const {
+Milliseconds OplogFetcher::_getInitialFindMaxTime() const {
+    return Milliseconds(oplogInitialFindMaxSeconds.load() * 1000);
+}
+
+Milliseconds OplogFetcher::_getRetriedFindMaxTime() const {
+    return Milliseconds(oplogRetriedFindMaxSeconds.load() * 1000);
+}
+
+Milliseconds OplogFetcher::_getGetMoreMaxTime() const {
     return _awaitDataTimeout;
 }
 
-bool OplogFetcher::inShutdown_forTest() const {
-    return _isInShutdown();
+OplogFetcher::State OplogFetcher::getState_forTest() const {
+    stdx::lock_guard<stdx::mutex> lk(_mutex);
+    return _state;
 }
 
 void OplogFetcher::_callback(const Fetcher::QueryResponseStatus& result,
@@ -357,8 +500,13 @@ void OplogFetcher::_callback(const Fetcher::QueryResponseStatus& result,
     // example, because it stepped down) we might not have a cursor.
     if (!responseStatus.isOK()) {
         {
+            // We have to call into replication coordinator outside oplog fetcher's mutex.
+            // It is OK if the current term becomes stale after this line since requests
+            // to remote nodes are asynchronous anyway.
+            auto currentTerm =
+                _dataReplicatorExternalState->getCurrentTermAndLastCommittedOpTime().value;
             stdx::lock_guard<stdx::mutex> lock(_mutex);
-            if (_inShutdown) {
+            if (_isShuttingDown_inlock()) {
                 log() << "Error returned from oplog query while canceling query: "
                       << redact(responseStatus);
             } else if (_fetcherRestarts == _maxFetcherRestarts) {
@@ -373,8 +521,8 @@ void OplogFetcher::_callback(const Fetcher::QueryResponseStatus& result,
                 _shuttingDownFetcher.reset();
                 // Move the old fetcher into the shutting down instance.
                 _shuttingDownFetcher.swap(_fetcher);
-                // Create and start fetcher with new starting optime.
-                _fetcher = _makeFetcher(_lastFetched.opTime);
+                // Create and start fetcher with current term and new starting optime.
+                _fetcher = _makeFetcher(currentTerm, _lastFetched.opTime, _getRetriedFindMaxTime());
                 auto scheduleStatus = _scheduleFetcher_inlock();
                 if (scheduleStatus.isOK()) {
                     log() << "Scheduled new oplog query " << _fetcher->toString();
@@ -391,44 +539,24 @@ void OplogFetcher::_callback(const Fetcher::QueryResponseStatus& result,
     // Reset fetcher restart counter on successful response.
     {
         stdx::lock_guard<stdx::mutex> lock(_mutex);
-        invariant(_active);
+        invariant(_isActive_inlock());
         _fetcherRestarts = 0;
     }
 
-    if (_isInShutdown()) {
+    if (_isShuttingDown()) {
         _finishCallback(Status(ErrorCodes::CallbackCanceled, "oplog fetcher shutting down"));
         return;
     }
 
     // Stop fetching and return on fail point.
-    // This fail point is intended to make the oplog fetcher ignore the downloaded batch of
-    // operations and not error out.
-    if (MONGO_FAIL_POINT(stopOplogFetcher)) {
+    // This fail point makes the oplog fetcher ignore the downloaded batch of operations and not
+    // error out.
+    if (MONGO_FAIL_POINT(stopReplProducer)) {
         _finishCallback(Status::OK());
-        // Wait for a while, otherwise, it will keep busy waiting.
-        sleepmillis(100);
         return;
     }
 
     const auto& queryResponse = result.getValue();
-    rpc::ReplSetMetadata metadata;
-
-    // Forward metadata (containing liveness information) to data replicator external state.
-    bool receivedMetadata =
-        queryResponse.otherFields.metadata.hasElement(rpc::kReplSetMetadataFieldName);
-    if (receivedMetadata) {
-        const auto& metadataObj = queryResponse.otherFields.metadata;
-        auto metadataResult = rpc::ReplSetMetadata::readFromMetadata(metadataObj);
-        if (!metadataResult.isOK()) {
-            error() << "invalid replication metadata from sync source " << _fetcher->getSource()
-                    << ": " << metadataResult.getStatus() << ": " << metadataObj;
-            _finishCallback(metadataResult.getStatus());
-            return;
-        }
-        metadata = metadataResult.getValue();
-        _dataReplicatorExternalState->processMetadata(metadata);
-    }
-
     const auto& documents = queryResponse.documents;
     auto firstDocToApply = documents.cbegin();
 
@@ -442,11 +570,28 @@ void OplogFetcher::_callback(const Fetcher::QueryResponseStatus& result,
 
     auto opTimeWithHash = getLastOpTimeWithHashFetched();
 
+    auto oqMetadataResult = parseOplogQueryMetadata(queryResponse);
+    if (!oqMetadataResult.isOK()) {
+        error() << "invalid oplog query metadata from sync source " << _fetcher->getSource() << ": "
+                << oqMetadataResult.getStatus() << ": " << queryResponse.otherFields.metadata;
+        _finishCallback(oqMetadataResult.getStatus());
+        return;
+    }
+    auto oqMetadata = oqMetadataResult.getValue();
+
     // Check start of remote oplog and, if necessary, stop fetcher to execute rollback.
     if (queryResponse.first) {
-        auto status = checkRemoteOplogStart(documents, opTimeWithHash);
+        auto remoteRBID = oqMetadata ? boost::make_optional(oqMetadata->getRBID()) : boost::none;
+        auto remoteLastApplied =
+            oqMetadata ? boost::make_optional(oqMetadata->getLastOpApplied()) : boost::none;
+        auto status = checkRemoteOplogStart(documents,
+                                            opTimeWithHash,
+                                            remoteLastApplied,
+                                            _requiredRBID,
+                                            remoteRBID,
+                                            _requireFresherSyncSource);
         if (!status.isOK()) {
-            // Stop oplog fetcher and execute rollback.
+            // Stop oplog fetcher and execute rollback if necessary.
             _finishCallback(status, opTimeWithHash);
             return;
         }
@@ -465,6 +610,28 @@ void OplogFetcher::_callback(const Fetcher::QueryResponseStatus& result,
     }
     auto info = validateResult.getValue();
 
+    // Process replset metadata.  It is important that this happen after we've validated the
+    // first batch, so we don't progress our knowledge of the commit point from a
+    // response that triggers a rollback.
+    rpc::ReplSetMetadata replSetMetadata;
+    bool receivedReplMetadata =
+        queryResponse.otherFields.metadata.hasElement(rpc::kReplSetMetadataFieldName);
+    if (receivedReplMetadata) {
+        const auto& metadataObj = queryResponse.otherFields.metadata;
+        auto metadataResult = rpc::ReplSetMetadata::readFromMetadata(metadataObj);
+        if (!metadataResult.isOK()) {
+            error() << "invalid replication metadata from sync source " << _fetcher->getSource()
+                    << ": " << metadataResult.getStatus() << ": " << metadataObj;
+            _finishCallback(metadataResult.getStatus());
+            return;
+        }
+        replSetMetadata = metadataResult.getValue();
+
+        // We will only ever have OplogQueryMetadata if we have ReplSetMetadata, so it is safe
+        // to call processMetadata() in this if block.
+        _dataReplicatorExternalState->processMetadata(replSetMetadata, oqMetadata);
+    }
+
     // Increment stats. We read all of the docs in the query.
     opsReadStats.increment(info.networkDocumentCount);
     networkByteStats.increment(info.networkDocumentBytes);
@@ -473,7 +640,11 @@ void OplogFetcher::_callback(const Fetcher::QueryResponseStatus& result,
     getmoreReplStats.recordMillis(durationCount<Milliseconds>(queryResponse.elapsedMillis));
 
     // TODO: back pressure handling will be added in SERVER-23499.
-    _enqueueDocumentsFn(firstDocToApply, documents.cend(), info);
+    auto status = _enqueueDocumentsFn(firstDocToApply, documents.cend(), info);
+    if (!status.isOK()) {
+        _finishCallback(status);
+        return;
+    }
 
     // Update last fetched info.
     if (firstDocToApply != documents.cend()) {
@@ -485,17 +656,24 @@ void OplogFetcher::_callback(const Fetcher::QueryResponseStatus& result,
         _lastFetched = opTimeWithHash;
     }
 
-    if (_dataReplicatorExternalState->shouldStopFetching(_fetcher->getSource(), metadata)) {
-        _finishCallback(Status(ErrorCodes::InvalidSyncSource,
-                               str::stream() << "sync source " << _fetcher->getSource().toString()
-                                             << " (last optime: "
-                                             << metadata.getLastOpVisible().toString()
-                                             << "; sync source index: "
-                                             << metadata.getSyncSourceIndex()
-                                             << "; primary index: "
-                                             << metadata.getPrimaryIndex()
-                                             << ") is no longer valid"),
-                        opTimeWithHash);
+    if (_dataReplicatorExternalState->shouldStopFetching(
+            _fetcher->getSource(), replSetMetadata, oqMetadata)) {
+        str::stream errMsg;
+        errMsg << "sync source " << _fetcher->getSource().toString();
+        errMsg << " (config version: " << replSetMetadata.getConfigVersion();
+        // If OplogQueryMetadata was provided, its values were used to determine if we should
+        // stop fetching from this sync source.
+        if (oqMetadata) {
+            errMsg << "; last applied optime: " << oqMetadata->getLastOpApplied().toString();
+            errMsg << "; sync source index: " << oqMetadata->getSyncSourceIndex();
+            errMsg << "; primary index: " << oqMetadata->getPrimaryIndex();
+        } else {
+            errMsg << "; last visible optime: " << replSetMetadata.getLastOpVisible().toString();
+            errMsg << "; sync source index: " << replSetMetadata.getSyncSourceIndex();
+            errMsg << "; primary index: " << replSetMetadata.getPrimaryIndex();
+        }
+        errMsg << ") is no longer valid";
+        _finishCallback(Status(ErrorCodes::InvalidSyncSource, errMsg), opTimeWithHash);
         return;
     }
 
@@ -506,10 +684,12 @@ void OplogFetcher::_callback(const Fetcher::QueryResponseStatus& result,
         return;
     }
 
-    getMoreBob->appendElements(makeGetMoreCommandObject(_dataReplicatorExternalState,
-                                                        queryResponse.nss,
+    auto lastCommittedWithCurrentTerm =
+        _dataReplicatorExternalState->getCurrentTermAndLastCommittedOpTime();
+    getMoreBob->appendElements(makeGetMoreCommandObject(queryResponse.nss,
                                                         queryResponse.cursorId,
-                                                        _awaitDataTimeout));
+                                                        lastCommittedWithCurrentTerm,
+                                                        _getGetMoreMaxTime()));
 }
 
 void OplogFetcher::_finishCallback(Status status) {
@@ -521,25 +701,54 @@ void OplogFetcher::_finishCallback(Status status, OpTimeWithHash opTimeWithHash)
 
     _onShutdownCallbackFn(status, opTimeWithHash);
 
+    decltype(_onShutdownCallbackFn) onShutdownCallbackFn;
     stdx::lock_guard<stdx::mutex> lock(_mutex);
-    _active = false;
+    invariant(State::kComplete != _state);
+    _state = State::kComplete;
     _condition.notify_all();
+
+    // Release any resources that might be held by the '_onShutdownCallbackFn' function object.
+    // The function object will be destroyed outside the lock since the temporary variable
+    // 'onShutdownCallbackFn' is declared before 'lock'.
+    invariant(_onShutdownCallbackFn);
+    std::swap(_onShutdownCallbackFn, onShutdownCallbackFn);
 }
 
-std::unique_ptr<Fetcher> OplogFetcher::_makeFetcher(OpTime lastFetchedOpTime) {
+std::unique_ptr<Fetcher> OplogFetcher::_makeFetcher(long long currentTerm,
+                                                    OpTime lastFetchedOpTime,
+                                                    Milliseconds findMaxTime) {
     return stdx::make_unique<Fetcher>(
         _executor,
         _source,
         _nss.db().toString(),
-        makeFindCommandObject(_dataReplicatorExternalState, _nss, lastFetchedOpTime),
+        makeFindCommandObject(_nss, currentTerm, lastFetchedOpTime, findMaxTime),
         stdx::bind(&OplogFetcher::_callback, this, stdx::placeholders::_1, stdx::placeholders::_3),
         _metadataObject,
-        _remoteCommandTimeout);
+        findMaxTime + kNetworkTimeoutBufferMS,
+        _getGetMoreMaxTime() + kNetworkTimeoutBufferMS);
 }
 
-bool OplogFetcher::_isInShutdown() const {
+bool OplogFetcher::_isShuttingDown() const {
     stdx::lock_guard<stdx::mutex> lock(_mutex);
-    return _inShutdown;
+    return _isShuttingDown_inlock();
+}
+
+bool OplogFetcher::_isShuttingDown_inlock() const {
+    return State::kShuttingDown == _state;
+}
+
+std::ostream& operator<<(std::ostream& os, const OplogFetcher::State& state) {
+    switch (state) {
+        case OplogFetcher::State::kPreStart:
+            return os << "PreStart";
+        case OplogFetcher::State::kRunning:
+            return os << "Running";
+        case OplogFetcher::State::kShuttingDown:
+            return os << "ShuttingDown";
+        case OplogFetcher::State::kComplete:
+            return os << "Complete";
+    }
+    MONGO_UNREACHABLE;
 }
 
 }  // namespace repl
