@@ -34,6 +34,7 @@
 #include "mongo/db/namespace_string.h"
 #include "mongo/db/repl/sync_source_resolver.h"
 #include "mongo/db/repl/sync_source_selector.h"
+#include "mongo/db/repl/sync_source_selector_mock.h"
 #include "mongo/executor/network_interface_mock.h"
 #include "mongo/executor/thread_pool_task_executor_test_fixture.h"
 #include "mongo/stdx/functional.h"
@@ -67,30 +68,6 @@ private:
     ShouldFailRequestFn _shouldFailRequest;
 };
 
-class SyncSourceSelectorMock : public SyncSourceSelector {
-public:
-    void clearSyncSourceBlacklist() override {}
-    HostAndPort chooseNewSyncSource(const OpTime& ot) override {
-        chooseNewSyncSourceHook();
-        lastOpTimeFetched = ot;
-        return syncSource;
-    }
-    void blacklistSyncSource(const HostAndPort& host, Date_t until) override {
-        blacklistHost = host;
-        blacklistUntil = until;
-    }
-    bool shouldChangeSyncSource(const HostAndPort&, const rpc::ReplSetMetadata&) {
-        return false;
-    }
-
-    HostAndPort syncSource = HostAndPort("host1", 1234);
-    OpTime lastOpTimeFetched;
-    stdx::function<void()> chooseNewSyncSourceHook = []() {};
-
-    HostAndPort blacklistHost;
-    Date_t blacklistUntil;
-};
-
 class SyncSourceResolverTest : public executor::ThreadPoolExecutorTest {
 private:
     void setUp() override;
@@ -241,7 +218,7 @@ TEST_F(SyncSourceResolverTest, StartupReturnsIllegalOperationIfAlreadyActive) {
 }
 
 TEST_F(SyncSourceResolverTest, StartupReturnsShutdownInProgressIfResolverIsShuttingDown) {
-    _selector->syncSource = HostAndPort("node1", 12345);
+    _selector->setChooseNewSyncSourceResult_forTest(HostAndPort("node1", 12345));
     ASSERT_OK(_resolver->startup());
     ASSERT_TRUE(executor::NetworkInterfaceMock::InNetworkGuard(getNet())->hasReadyRequests());
     _resolver->shutdown();
@@ -258,13 +235,13 @@ TEST_F(SyncSourceResolverTest, StartupReturnsShutdownInProgressIfExecutorIsShutd
 
 TEST_F(SyncSourceResolverTest,
        SyncSourceResolverReturnsStatusOkAndAnEmptyHostWhenNoViableHostExists) {
-    _selector->syncSource = HostAndPort();
+    _selector->setChooseNewSyncSourceResult_forTest(HostAndPort());
     ASSERT_OK(_resolver->startup());
 
     // Resolver invokes callback with empty host and becomes inactive immediately.
     ASSERT_FALSE(_resolver->isActive());
     ASSERT_EQUALS(HostAndPort(), unittest::assertGet(_response.syncSourceStatus));
-    ASSERT_EQUALS(lastOpTimeFetched, _selector->lastOpTimeFetched);
+    ASSERT_EQUALS(lastOpTimeFetched, _selector->getChooseNewSyncSourceOpTime_forTest());
 
     // Cannot restart a completed resolver.
     ASSERT_EQUALS(ErrorCodes::ShutdownInProgress, _resolver->startup());
@@ -273,17 +250,16 @@ TEST_F(SyncSourceResolverTest,
 TEST_F(
     SyncSourceResolverTest,
     SyncSourceResolverReturnsCallbackCanceledIfResolverIsShutdownBeforeReturningEmptySyncSource) {
-    _selector->syncSource = HostAndPort();
-    _selector->chooseNewSyncSourceHook = [this]() { _resolver->shutdown(); };
+    _selector->setChooseNewSyncSourceResult_forTest(HostAndPort());
+    _selector->setChooseNewSyncSourceHook_forTest([this]() { _resolver->shutdown(); });
     ASSERT_EQUALS(ErrorCodes::CallbackCanceled, _resolver->startup());
     ASSERT_EQUALS(ErrorCodes::CallbackCanceled, _response.syncSourceStatus);
 }
 
 TEST_F(SyncSourceResolverTest,
        SyncSourceResolverConvertsExceptionToStatusIfChoosingViableSyncSourceThrowsException) {
-    _selector->chooseNewSyncSourceHook = [this]() {
-        uassert(ErrorCodes::InternalError, "", false);
-    };
+    _selector->setChooseNewSyncSourceHook_forTest(
+        [this]() { uassert(ErrorCodes::InternalError, "", false); });
     ASSERT_EQUALS(ErrorCodes::InternalError, _resolver->startup());
     ASSERT_EQUALS(ErrorCodes::InternalError, _response.syncSourceStatus);
 }
@@ -313,7 +289,7 @@ void _scheduleFirstOplogEntryFetcherResponse(executor::NetworkInterfaceMock* net
     ASSERT_BSONOBJ_EQ(BSON("$natural" << 1), request.cmdObj.getObjectField("sort"));
 
     // Change next sync source candidate before delivering scheduled response.
-    selector->syncSource = nextSyncSource;
+    selector->setChooseNewSyncSourceResult_forTest(nextSyncSource);
 
     net->runReadyNetworkOperations();
 }
@@ -327,15 +303,29 @@ void _scheduleFirstOplogEntryFetcherResponse(executor::NetworkInterfaceMock* net
         net, selector, currentSyncSource, nextSyncSource, {BSON("ts" << ts << "t" << 0)});
 }
 
+void _scheduleRBIDResponse(executor::NetworkInterfaceMock* net,
+                           HostAndPort currentSyncSource,
+                           const BSONObj& reply = BSON("ok" << 1 << "rbid" << 1)) {
+    executor::NetworkInterfaceMock::InNetworkGuard networkGuard(net);
+    ASSERT_TRUE(net->hasReadyRequests());
+    auto request = net->scheduleSuccessfulResponse(reply);
+    ASSERT_EQUALS(currentSyncSource, request.target);
+    ASSERT_EQUALS("admin", request.dbname);
+    ASSERT_EQUALS(SyncSourceResolver::kFetcherTimeout, request.timeout);
+    ASSERT_BSONOBJ_EQ(BSON("replSetGetRBID" << 1), request.cmdObj);
+    net->runReadyNetworkOperations();
+}
+
 TEST_F(SyncSourceResolverTest,
        SyncSourceResolverReturnsStatusOkAndTheFoundHostWhenAnEligibleSyncSourceExists) {
     HostAndPort candidate1("node1", 12345);
-    _selector->syncSource = candidate1;
+    _selector->setChooseNewSyncSourceResult_forTest(candidate1);
     ASSERT_OK(_resolver->startup());
     ASSERT_TRUE(_resolver->isActive());
 
     _scheduleFirstOplogEntryFetcherResponse(
         getNet(), _selector.get(), candidate1, HostAndPort(), Timestamp(10, 2));
+    _scheduleRBIDResponse(getNet(), candidate1);
 
     _resolver->join();
     ASSERT_FALSE(_resolver->isActive());
@@ -345,7 +335,7 @@ TEST_F(SyncSourceResolverTest,
 TEST_F(SyncSourceResolverTest,
        SyncSourceResolverTransitionsToCompleteWhenFinishCallbackThrowsException) {
     HostAndPort candidate1("node1", 12345);
-    _selector->syncSource = candidate1;
+    _selector->setChooseNewSyncSourceResult_forTest(candidate1);
     _onCompletion = [this](const SyncSourceResolverResponse& response) {
         _response = response;
         uassert(ErrorCodes::InternalError, "", false);
@@ -356,6 +346,7 @@ TEST_F(SyncSourceResolverTest,
 
     _scheduleFirstOplogEntryFetcherResponse(
         getNet(), _selector.get(), candidate1, HostAndPort(), Timestamp(10, 2));
+    _scheduleRBIDResponse(getNet(), candidate1);
 
     _resolver->join();
     ASSERT_FALSE(_resolver->isActive());
@@ -391,7 +382,7 @@ TEST_F(SyncSourceResolverTest,
        SyncSourceResolverWillTryOtherSourcesWhenTheFirstNodeDoesNotHaveOldEnoughData) {
     HostAndPort candidate1("node1", 12345);
     HostAndPort candidate2("node2", 12345);
-    _selector->syncSource = candidate1;
+    _selector->setChooseNewSyncSourceResult_forTest(candidate1);
 
     ASSERT_OK(_resolver->startup());
     ASSERT_TRUE(_resolver->isActive());
@@ -400,12 +391,13 @@ TEST_F(SyncSourceResolverTest,
         getNet(), _selector.get(), candidate1, candidate2, Timestamp(200, 2));
 
     ASSERT_TRUE(_resolver->isActive());
-    ASSERT_EQUALS(candidate1, _selector->blacklistHost);
+    ASSERT_EQUALS(candidate1, _selector->getLastBlacklistedSyncSource_forTest());
     ASSERT_EQUALS(getExecutor().now() + SyncSourceResolver::kTooStaleBlacklistDuration,
-                  _selector->blacklistUntil);
+                  _selector->getLastBlacklistExpiration_forTest());
 
     _scheduleFirstOplogEntryFetcherResponse(
         getNet(), _selector.get(), candidate2, HostAndPort(), Timestamp(10, 2));
+    _scheduleRBIDResponse(getNet(), candidate2);
 
     _resolver->join();
     ASSERT_FALSE(_resolver->isActive());
@@ -417,7 +409,7 @@ TEST_F(SyncSourceResolverTest,
     HostAndPort candidate1("node1", 12345);
     HostAndPort candidate2("node2", 12345);
     HostAndPort candidate3("node3", 12345);
-    _selector->syncSource = candidate1;
+    _selector->setChooseNewSyncSourceResult_forTest(candidate1);
 
     ASSERT_OK(_resolver->startup());
     ASSERT_TRUE(_resolver->isActive());
@@ -448,7 +440,7 @@ void _scheduleNetworkErrorForFirstNode(executor::NetworkInterfaceMock* net,
     ASSERT_EQUALS(currentSyncSource, request.target);
 
     // Change next sync source candidate before delivering error to callback.
-    selector->syncSource = nextSyncSource;
+    selector->setChooseNewSyncSourceResult_forTest(nextSyncSource);
 
     net->runReadyNetworkOperations();
 }
@@ -457,7 +449,7 @@ TEST_F(SyncSourceResolverTest,
        SyncSourceResolverWillTryOtherSourcesWhenTheFirstNodeHasANetworkError) {
     HostAndPort candidate1("node1", 12345);
     HostAndPort candidate2("node2", 12345);
-    _selector->syncSource = candidate1;
+    _selector->setChooseNewSyncSourceResult_forTest(candidate1);
 
     ASSERT_OK(_resolver->startup());
     ASSERT_TRUE(_resolver->isActive());
@@ -465,12 +457,13 @@ TEST_F(SyncSourceResolverTest,
     _scheduleNetworkErrorForFirstNode(getNet(), _selector.get(), candidate1, candidate2);
 
     ASSERT_TRUE(_resolver->isActive());
-    ASSERT_EQUALS(candidate1, _selector->blacklistHost);
+    ASSERT_EQUALS(candidate1, _selector->getLastBlacklistedSyncSource_forTest());
     ASSERT_EQUALS(getExecutor().now() + SyncSourceResolver::kFetcherErrorBlacklistDuration,
-                  _selector->blacklistUntil);
+                  _selector->getLastBlacklistExpiration_forTest());
 
     _scheduleFirstOplogEntryFetcherResponse(
         getNet(), _selector.get(), candidate2, HostAndPort(), Timestamp(10, 2));
+    _scheduleRBIDResponse(getNet(), candidate2);
 
     _resolver->join();
     ASSERT_FALSE(_resolver->isActive());
@@ -480,7 +473,7 @@ TEST_F(SyncSourceResolverTest,
 TEST_F(SyncSourceResolverTest,
        SyncSourceResolverReturnsEmptyHostWhenNoViableNodeExistsAfterNetworkErrorOnFirstNode) {
     HostAndPort candidate1("node1", 12345);
-    _selector->syncSource = candidate1;
+    _selector->setChooseNewSyncSourceResult_forTest(candidate1);
 
     ASSERT_OK(_resolver->startup());
     ASSERT_TRUE(_resolver->isActive());
@@ -488,9 +481,9 @@ TEST_F(SyncSourceResolverTest,
     _scheduleNetworkErrorForFirstNode(getNet(), _selector.get(), candidate1, HostAndPort());
 
     ASSERT_FALSE(_resolver->isActive());
-    ASSERT_EQUALS(candidate1, _selector->blacklistHost);
+    ASSERT_EQUALS(candidate1, _selector->getLastBlacklistedSyncSource_forTest());
     ASSERT_EQUALS(getExecutor().now() + SyncSourceResolver::kFetcherErrorBlacklistDuration,
-                  _selector->blacklistUntil);
+                  _selector->getLastBlacklistExpiration_forTest());
 
     ASSERT_EQUALS(HostAndPort(), unittest::assertGet(_response.syncSourceStatus));
 }
@@ -499,7 +492,7 @@ TEST_F(SyncSourceResolverTest,
        SyncSourceResolverReturnsScheduleErrorWhenTheSchedulingCommandToSecondNodeFails) {
     HostAndPort candidate1("node1", 12345);
     HostAndPort candidate2("node2", 12345);
-    _selector->syncSource = candidate1;
+    _selector->setChooseNewSyncSourceResult_forTest(candidate1);
 
     ASSERT_OK(_resolver->startup());
     ASSERT_TRUE(_resolver->isActive());
@@ -511,9 +504,9 @@ TEST_F(SyncSourceResolverTest,
     _scheduleNetworkErrorForFirstNode(getNet(), _selector.get(), candidate1, candidate2);
 
     ASSERT_FALSE(_resolver->isActive());
-    ASSERT_EQUALS(candidate1, _selector->blacklistHost);
+    ASSERT_EQUALS(candidate1, _selector->getLastBlacklistedSyncSource_forTest());
     ASSERT_EQUALS(getExecutor().now() + SyncSourceResolver::kFetcherErrorBlacklistDuration,
-                  _selector->blacklistUntil);
+                  _selector->getLastBlacklistExpiration_forTest());
 
     ASSERT_EQUALS(ErrorCodes::OperationFailed, _response.syncSourceStatus);
 }
@@ -522,7 +515,7 @@ TEST_F(SyncSourceResolverTest,
        SyncSourceResolverWillTryOtherSourcesWhenTheFirstNodeHasAnEmptyOplog) {
     HostAndPort candidate1("node1", 12345);
     HostAndPort candidate2("node1", 12345);
-    _selector->syncSource = candidate1;
+    _selector->setChooseNewSyncSourceResult_forTest(candidate1);
     ASSERT_OK(_resolver->startup());
     ASSERT_TRUE(_resolver->isActive());
 
@@ -530,12 +523,13 @@ TEST_F(SyncSourceResolverTest,
         getNet(), _selector.get(), candidate1, candidate2, std::vector<BSONObj>());
 
     ASSERT_TRUE(_resolver->isActive());
-    ASSERT_EQUALS(candidate1, _selector->blacklistHost);
+    ASSERT_EQUALS(candidate1, _selector->getLastBlacklistedSyncSource_forTest());
     ASSERT_EQUALS(getExecutor().now() + SyncSourceResolver::kOplogEmptyBlacklistDuration,
-                  _selector->blacklistUntil);
+                  _selector->getLastBlacklistExpiration_forTest());
 
     _scheduleFirstOplogEntryFetcherResponse(
         getNet(), _selector.get(), candidate2, HostAndPort(), Timestamp(10, 2));
+    _scheduleRBIDResponse(getNet(), candidate2);
 
     _resolver->join();
     ASSERT_FALSE(_resolver->isActive());
@@ -546,7 +540,7 @@ TEST_F(SyncSourceResolverTest,
        SyncSourceResolverWillTryOtherSourcesWhenTheFirstNodeHasAnEmptyFirstOplogEntry) {
     HostAndPort candidate1("node1", 12345);
     HostAndPort candidate2("node1", 12345);
-    _selector->syncSource = candidate1;
+    _selector->setChooseNewSyncSourceResult_forTest(candidate1);
     ASSERT_OK(_resolver->startup());
     ASSERT_TRUE(_resolver->isActive());
 
@@ -554,12 +548,13 @@ TEST_F(SyncSourceResolverTest,
         getNet(), _selector.get(), candidate1, candidate2, {BSONObj()});
 
     ASSERT_TRUE(_resolver->isActive());
-    ASSERT_EQUALS(candidate1, _selector->blacklistHost);
+    ASSERT_EQUALS(candidate1, _selector->getLastBlacklistedSyncSource_forTest());
     ASSERT_EQUALS(getExecutor().now() + SyncSourceResolver::kFirstOplogEntryEmptyBlacklistDuration,
-                  _selector->blacklistUntil);
+                  _selector->getLastBlacklistExpiration_forTest());
 
     _scheduleFirstOplogEntryFetcherResponse(
         getNet(), _selector.get(), candidate2, HostAndPort(), Timestamp(10, 2));
+    _scheduleRBIDResponse(getNet(), candidate2);
 
     _resolver->join();
     ASSERT_FALSE(_resolver->isActive());
@@ -570,7 +565,7 @@ TEST_F(SyncSourceResolverTest,
        SyncSourceResolverWillTryOtherSourcesWhenFirstNodeContainsOplogEntryWithNullTimestamp) {
     HostAndPort candidate1("node1", 12345);
     HostAndPort candidate2("node1", 12345);
-    _selector->syncSource = candidate1;
+    _selector->setChooseNewSyncSourceResult_forTest(candidate1);
     ASSERT_OK(_resolver->startup());
     ASSERT_TRUE(_resolver->isActive());
 
@@ -578,13 +573,14 @@ TEST_F(SyncSourceResolverTest,
         getNet(), _selector.get(), candidate1, candidate2, Timestamp(0, 0));
 
     ASSERT_TRUE(_resolver->isActive());
-    ASSERT_EQUALS(candidate1, _selector->blacklistHost);
+    ASSERT_EQUALS(candidate1, _selector->getLastBlacklistedSyncSource_forTest());
     ASSERT_EQUALS(getExecutor().now() +
                       SyncSourceResolver::kFirstOplogEntryNullTimestampBlacklistDuration,
-                  _selector->blacklistUntil);
+                  _selector->getLastBlacklistExpiration_forTest());
 
     _scheduleFirstOplogEntryFetcherResponse(
         getNet(), _selector.get(), candidate2, HostAndPort(), Timestamp(10, 2));
+    _scheduleRBIDResponse(getNet(), candidate2);
 
     _resolver->join();
     ASSERT_FALSE(_resolver->isActive());
@@ -646,7 +642,7 @@ TEST_F(
     _resolver = _makeResolver(lastOpTimeFetched, requiredOpTime);
 
     HostAndPort candidate1("node1", 12345);
-    _selector->syncSource = candidate1;
+    _selector->setChooseNewSyncSourceResult_forTest(candidate1);
     ASSERT_OK(_resolver->startup());
     ASSERT_TRUE(_resolver->isActive());
 
@@ -655,11 +651,14 @@ TEST_F(
 
     ASSERT_TRUE(_resolver->isActive());
 
+    _scheduleRBIDResponse(getNet(), candidate1, BSON("ok" << 1 << "rbid" << 7));
     _scheduleRequiredOpTimeFetcherResponse(getNet(), _selector.get(), candidate1, requiredOpTime);
 
     _resolver->join();
     ASSERT_FALSE(_resolver->isActive());
     ASSERT_EQUALS(candidate1, unittest::assertGet(_response.syncSourceStatus));
+    ASSERT(_response.rbid);
+    ASSERT_EQ(_response.rbid, 7);
 }
 
 TEST_F(SyncSourceResolverTest,
@@ -668,7 +667,7 @@ TEST_F(SyncSourceResolverTest,
 
     HostAndPort candidate1("node1", 12345);
     HostAndPort candidate2("node2", 12345);
-    _selector->syncSource = candidate1;
+    _selector->setChooseNewSyncSourceResult_forTest(candidate1);
     ASSERT_OK(_resolver->startup());
     ASSERT_TRUE(_resolver->isActive());
 
@@ -677,6 +676,7 @@ TEST_F(SyncSourceResolverTest,
 
     ASSERT_TRUE(_resolver->isActive());
 
+    _scheduleRBIDResponse(getNet(), candidate1);
     _scheduleRequiredOpTimeFetcherResponse(
         getNet(),
         _selector.get(),
@@ -685,12 +685,13 @@ TEST_F(SyncSourceResolverTest,
         {BSON("ts" << requiredOpTime.getTimestamp() << "t" << OpTime::kUninitializedTerm)});
 
     ASSERT_TRUE(_resolver->isActive());
-    ASSERT_EQUALS(candidate1, _selector->blacklistHost);
+    ASSERT_EQUALS(candidate1, _selector->getLastBlacklistedSyncSource_forTest());
     ASSERT_EQUALS(getExecutor().now() + SyncSourceResolver::kNoRequiredOpTimeBlacklistDuration,
-                  _selector->blacklistUntil);
+                  _selector->getLastBlacklistExpiration_forTest());
 
     _scheduleFirstOplogEntryFetcherResponse(
         getNet(), _selector.get(), candidate2, HostAndPort(), Timestamp(10, 0));
+    _scheduleRBIDResponse(getNet(), candidate2);
     _scheduleRequiredOpTimeFetcherResponse(getNet(), _selector.get(), candidate2, requiredOpTime);
 
     _resolver->join();
@@ -707,7 +708,7 @@ TEST_F(
 
     HostAndPort candidate1("node1", 12345);
     HostAndPort candidate2("node2", 12345);
-    _selector->syncSource = candidate1;
+    _selector->setChooseNewSyncSourceResult_forTest(candidate1);
     ASSERT_OK(_resolver->startup());
     ASSERT_TRUE(_resolver->isActive());
 
@@ -716,15 +717,17 @@ TEST_F(
 
     ASSERT_TRUE(_resolver->isActive());
 
+    _scheduleRBIDResponse(getNet(), candidate1);
     _scheduleRequiredOpTimeFetcherResponse(getNet(), _selector.get(), candidate1, requiredOpTime);
 
     ASSERT_TRUE(_resolver->isActive());
-    ASSERT_EQUALS(candidate1, _selector->blacklistHost);
+    ASSERT_EQUALS(candidate1, _selector->getLastBlacklistedSyncSource_forTest());
     ASSERT_EQUALS(getExecutor().now() + SyncSourceResolver::kNoRequiredOpTimeBlacklistDuration,
-                  _selector->blacklistUntil);
+                  _selector->getLastBlacklistExpiration_forTest());
 
     _scheduleFirstOplogEntryFetcherResponse(
         getNet(), _selector.get(), candidate2, HostAndPort(), Timestamp(10, 0));
+    _scheduleRBIDResponse(getNet(), candidate2);
     _scheduleRequiredOpTimeFetcherResponse(
         getNet(), _selector.get(), candidate2, requireOpTimeWithUninitializedTerm);
 
@@ -739,7 +742,7 @@ TEST_F(SyncSourceResolverTest,
 
     HostAndPort candidate1("node1", 12345);
     HostAndPort candidate2("node2", 12345);
-    _selector->syncSource = candidate1;
+    _selector->setChooseNewSyncSourceResult_forTest(candidate1);
     ASSERT_OK(_resolver->startup());
     ASSERT_TRUE(_resolver->isActive());
 
@@ -748,16 +751,18 @@ TEST_F(SyncSourceResolverTest,
 
     ASSERT_TRUE(_resolver->isActive());
 
+    _scheduleRBIDResponse(getNet(), candidate1);
     _scheduleRequiredOpTimeFetcherResponse(
         getNet(), _selector.get(), candidate1, requiredOpTime, {});
 
     ASSERT_TRUE(_resolver->isActive());
-    ASSERT_EQUALS(candidate1, _selector->blacklistHost);
+    ASSERT_EQUALS(candidate1, _selector->getLastBlacklistedSyncSource_forTest());
     ASSERT_EQUALS(getExecutor().now() + SyncSourceResolver::kNoRequiredOpTimeBlacklistDuration,
-                  _selector->blacklistUntil);
+                  _selector->getLastBlacklistExpiration_forTest());
 
     _scheduleFirstOplogEntryFetcherResponse(
         getNet(), _selector.get(), candidate2, HostAndPort(), Timestamp(10, 0));
+    _scheduleRBIDResponse(getNet(), candidate2);
     _scheduleRequiredOpTimeFetcherResponse(getNet(), _selector.get(), candidate2, requiredOpTime);
 
     _resolver->join();
@@ -771,7 +776,7 @@ TEST_F(SyncSourceResolverTest,
 
     HostAndPort candidate1("node1", 12345);
     HostAndPort candidate2("node2", 12345);
-    _selector->syncSource = candidate1;
+    _selector->setChooseNewSyncSourceResult_forTest(candidate1);
     ASSERT_OK(_resolver->startup());
     ASSERT_TRUE(_resolver->isActive());
 
@@ -780,6 +785,7 @@ TEST_F(SyncSourceResolverTest,
 
     ASSERT_TRUE(_resolver->isActive());
 
+    _scheduleRBIDResponse(getNet(), candidate1);
     _scheduleRequiredOpTimeFetcherResponse(
         getNet(),
         _selector.get(),
@@ -788,12 +794,13 @@ TEST_F(SyncSourceResolverTest,
         {BSON("ts" << requiredOpTime.getTimestamp() << "t" << requiredOpTime.getTerm() + 1)});
 
     ASSERT_TRUE(_resolver->isActive());
-    ASSERT_EQUALS(candidate1, _selector->blacklistHost);
+    ASSERT_EQUALS(candidate1, _selector->getLastBlacklistedSyncSource_forTest());
     ASSERT_EQUALS(getExecutor().now() + SyncSourceResolver::kNoRequiredOpTimeBlacklistDuration,
-                  _selector->blacklistUntil);
+                  _selector->getLastBlacklistExpiration_forTest());
 
     _scheduleFirstOplogEntryFetcherResponse(
         getNet(), _selector.get(), candidate2, HostAndPort(), Timestamp(10, 0));
+    _scheduleRBIDResponse(getNet(), candidate2);
     _scheduleRequiredOpTimeFetcherResponse(getNet(), _selector.get(), candidate2, requiredOpTime);
 
     _resolver->join();
@@ -801,6 +808,27 @@ TEST_F(SyncSourceResolverTest,
     ASSERT_EQUALS(candidate2, unittest::assertGet(_response.syncSourceStatus));
 }
 
+TEST_F(SyncSourceResolverTest,
+       SyncSourceResolverReturnsScheduleErrorWhenSchedulingRBIDCommandFails) {
+    _resolver = _makeResolver(lastOpTimeFetched, requiredOpTime);
+
+    _shouldFailRequest = [](const executor::RemoteCommandRequest& request) {
+        return request.cmdObj.firstElementFieldName() == "replSetGetRBID"_sd;
+    };
+
+    HostAndPort candidate1("node1", 12345);
+    _selector->setChooseNewSyncSourceResult_forTest(candidate1);
+    ASSERT_OK(_resolver->startup());
+    ASSERT_TRUE(_resolver->isActive());
+
+    _scheduleFirstOplogEntryFetcherResponse(
+        getNet(), _selector.get(), candidate1, HostAndPort(), Timestamp(10, 0));
+
+    _resolver->join();
+    ASSERT_FALSE(_resolver->isActive());
+    ASSERT_EQUALS(ErrorCodes::OperationFailed, _response.syncSourceStatus);
+}
+
 TEST_F(SyncSourceResolverTest,
        SyncSourceResolverReturnsScheduleErrorWhenSchedulingRequiredOpTimeFindCommandFails) {
     _resolver = _makeResolver(lastOpTimeFetched, requiredOpTime);
@@ -810,12 +838,13 @@ TEST_F(SyncSourceResolverTest,
     };
 
     HostAndPort candidate1("node1", 12345);
-    _selector->syncSource = candidate1;
+    _selector->setChooseNewSyncSourceResult_forTest(candidate1);
     ASSERT_OK(_resolver->startup());
     ASSERT_TRUE(_resolver->isActive());
 
     _scheduleFirstOplogEntryFetcherResponse(
         getNet(), _selector.get(), candidate1, HostAndPort(), Timestamp(10, 0));
+    _scheduleRBIDResponse(getNet(), candidate1);
 
     _resolver->join();
     ASSERT_FALSE(_resolver->isActive());
@@ -828,7 +857,7 @@ TEST_F(
     _resolver = _makeResolver(lastOpTimeFetched, requiredOpTime);
 
     HostAndPort candidate1("node1", 12345);
-    _selector->syncSource = candidate1;
+    _selector->setChooseNewSyncSourceResult_forTest(candidate1);
     ASSERT_OK(_resolver->startup());
     ASSERT_TRUE(_resolver->isActive());
 
@@ -851,7 +880,7 @@ TEST_F(
     _resolver = _makeResolver(lastOpTimeFetched, requiredOpTime);
 
     HostAndPort candidate1("node1", 12345);
-    _selector->syncSource = candidate1;
+    _selector->setChooseNewSyncSourceResult_forTest(candidate1);
     ASSERT_OK(_resolver->startup());
     ASSERT_TRUE(_resolver->isActive());
 
@@ -867,13 +896,12 @@ TEST_F(
     ASSERT_EQUALS(ErrorCodes::CallbackCanceled, _response.syncSourceStatus);
 }
 
-TEST_F(
-    SyncSourceResolverTest,
-    SyncSourceResolverReturnsEmptyHostIfNoViableHostExistsAfterNetworkErrorOnRequiredOpTimeCommand) {
+TEST_F(SyncSourceResolverTest,
+       SyncSourceResolverReturnsEmptyHostIfNoViableHostExistsAfterNetworkErrorOnRBIDCommand) {
     _resolver = _makeResolver(lastOpTimeFetched, requiredOpTime);
 
     HostAndPort candidate1("node1", 12345);
-    _selector->syncSource = candidate1;
+    _selector->setChooseNewSyncSourceResult_forTest(candidate1);
     ASSERT_OK(_resolver->startup());
     ASSERT_TRUE(_resolver->isActive());
 
@@ -883,9 +911,65 @@ TEST_F(
     _scheduleNetworkErrorForFirstNode(getNet(), _selector.get(), candidate1, HostAndPort());
 
     ASSERT_FALSE(_resolver->isActive());
-    ASSERT_EQUALS(candidate1, _selector->blacklistHost);
+    ASSERT_EQUALS(candidate1, _selector->getLastBlacklistedSyncSource_forTest());
     ASSERT_EQUALS(getExecutor().now() + SyncSourceResolver::kFetcherErrorBlacklistDuration,
-                  _selector->blacklistUntil);
+                  _selector->getLastBlacklistExpiration_forTest());
+
+    _resolver->join();
+    ASSERT_FALSE(_resolver->isActive());
+    ASSERT_EQUALS(HostAndPort(), unittest::assertGet(_response.syncSourceStatus));
+}
+
+TEST_F(SyncSourceResolverTest,
+       SyncSourceResolverReturnsEmptyHostIfNoViableHostExistsAfterRBIDCommandNotOk) {
+    _resolver = _makeResolver(lastOpTimeFetched, requiredOpTime);
+
+    HostAndPort candidate1("node1", 12345);
+    _selector->setChooseNewSyncSourceResult_forTest(candidate1);
+    ASSERT_OK(_resolver->startup());
+    ASSERT_TRUE(_resolver->isActive());
+
+    _scheduleFirstOplogEntryFetcherResponse(
+        getNet(), _selector.get(), candidate1, candidate1, Timestamp(10, 0));
+
+    // _scheduleNetworkErrorForFirstNode does this for us.
+    _selector->setChooseNewSyncSourceResult_forTest(HostAndPort());
+
+    _scheduleRBIDResponse(getNet(),
+                          candidate1,
+                          BSON("ok" << 0 << "code" << 9000 << "errmsg"
+                                    << "I'm sorry Dave, I'm afraid I can't do that."));
+
+    ASSERT_FALSE(_resolver->isActive());
+    ASSERT_EQUALS(candidate1, _selector->getLastBlacklistedSyncSource_forTest());
+    ASSERT_EQUALS(getExecutor().now() + SyncSourceResolver::kFetcherErrorBlacklistDuration,
+                  _selector->getLastBlacklistExpiration_forTest());
+
+    _resolver->join();
+    ASSERT_FALSE(_resolver->isActive());
+    ASSERT_EQUALS(HostAndPort(), unittest::assertGet(_response.syncSourceStatus));
+}
+
+TEST_F(
+    SyncSourceResolverTest,
+    SyncSourceResolverReturnsEmptyHostIfNoViableHostExistsAfterNetworkErrorOnRequiredOpTimeCommand) {
+    _resolver = _makeResolver(lastOpTimeFetched, requiredOpTime);
+
+    HostAndPort candidate1("node1", 12345);
+    _selector->setChooseNewSyncSourceResult_forTest(candidate1);
+    ASSERT_OK(_resolver->startup());
+    ASSERT_TRUE(_resolver->isActive());
+
+    _scheduleFirstOplogEntryFetcherResponse(
+        getNet(), _selector.get(), candidate1, candidate1, Timestamp(10, 0));
+
+    _scheduleRBIDResponse(getNet(), candidate1);
+    _scheduleNetworkErrorForFirstNode(getNet(), _selector.get(), candidate1, HostAndPort());
+
+    ASSERT_FALSE(_resolver->isActive());
+    ASSERT_EQUALS(candidate1, _selector->getLastBlacklistedSyncSource_forTest());
+    ASSERT_EQUALS(getExecutor().now() + SyncSourceResolver::kFetcherErrorBlacklistDuration,
+                  _selector->getLastBlacklistExpiration_forTest());
 
     _resolver->join();
     ASSERT_FALSE(_resolver->isActive());
