@@ -75,9 +75,11 @@ bool isInRange(const BSONObj& obj,
 
 BSONObj createRequestWithSessionId(StringData commandName,
                                    const NamespaceString& nss,
-                                   const MigrationSessionId& sessionId) {
+                                   const MigrationSessionId& sessionId,
+                                   bool waitForSteadyOrDone = false) {
     BSONObjBuilder builder;
     builder.append(commandName, nss.ns());
+    builder.append("waitForSteadyOrDone", waitForSteadyOrDone);
     sessionId.append(&builder);
     return builder.obj();
 }
@@ -231,13 +233,8 @@ Status MigrationChunkClonerSourceLegacy::awaitUntilCriticalSectionIsAppropriate(
 
     int iteration = 0;
     while ((Date_t::now() - startTime) < maxTimeToWait) {
-        // Exponential sleep backoff, up to 1024ms. Don't sleep much on the first few iterations,
-        // since we want empty chunk migrations to be fast.
-        sleepmillis(1LL << std::min(iteration, 10));
-        iteration++;
-
         auto responseStatus = _callRecipient(
-            createRequestWithSessionId(kRecvChunkStatus, _args.getNss(), _sessionId));
+            createRequestWithSessionId(kRecvChunkStatus, _args.getNss(), _sessionId, true));
         if (!responseStatus.isOK()) {
             return {responseStatus.getStatus().code(),
                     str::stream()
@@ -247,6 +244,11 @@ Status MigrationChunkClonerSourceLegacy::awaitUntilCriticalSectionIsAppropriate(
 
         const BSONObj& res = responseStatus.getValue();
 
+        if (!res["waited"].boolean()) {
+            sleepmillis(1LL << std::min(iteration, 10));
+        }
+        iteration++;
+
         stdx::lock_guard<stdx::mutex> sl(_mutex);
 
         const std::size_t cloneLocsRemaining = _cloneLocs.size();
@@ -306,7 +308,7 @@ Status MigrationChunkClonerSourceLegacy::awaitUntilCriticalSectionIsAppropriate(
     return {ErrorCodes::ExceededTimeLimit, "Timed out waiting for the cloner to catch up"};
 }
 
-Status MigrationChunkClonerSourceLegacy::commitClone(OperationContext* txn) {
+StatusWith<BSONObj> MigrationChunkClonerSourceLegacy::commitClone(OperationContext* txn) {
     invariant(_state == kCloning);
     invariant(!txn->lockState()->isLocked());
 
@@ -314,7 +316,7 @@ Status MigrationChunkClonerSourceLegacy::commitClone(OperationContext* txn) {
         _callRecipient(createRequestWithSessionId(kRecvChunkCommit, _args.getNss(), _sessionId));
     if (responseStatus.isOK()) {
         _cleanup(txn);
-        return Status::OK();
+        return responseStatus;
     }
 
     cancelClone(txn);
@@ -471,12 +473,17 @@ void MigrationChunkClonerSourceLegacy::_cleanup(OperationContext* txn) {
         _reload.clear();
         _deleted.clear();
     }
+    // Implicitly resets _deleteNotifyExec to avoid possible invariant failure
+    // in on destruction of MigrationChunkClonerSourceLegacy, and will always
+    // call deleteNotifyExec destructor on scope exit even if something in the
+    // below if statement fails
+    auto deleteNotifyExec = std::move(_deleteNotifyExec);
 
-    if (_deleteNotifyExec) {
+    if (deleteNotifyExec) {
         ScopedTransaction scopedXact(txn, MODE_IS);
         AutoGetCollection autoColl(txn, _args.getNss(), MODE_IS);
 
-        _deleteNotifyExec.reset();
+        deleteNotifyExec.reset();
     }
 }
 
@@ -576,8 +583,7 @@ Status MigrationChunkClonerSourceLegacy::_storeCurrentLocs(OperationContext* txn
     if (totalRecs > 0) {
         avgRecSize = collection->dataSize(txn) / totalRecs;
         maxRecsWhenFull = _args.getMaxChunkSizeBytes() / avgRecSize;
-        maxRecsWhenFull = std::min((unsigned long long)(kMaxObjectPerChunk + 1),
-                                   130 * maxRecsWhenFull / 100 /* slack */);
+        maxRecsWhenFull = 130 * maxRecsWhenFull / 100;  // pad some slack
     } else {
         avgRecSize = 0;
         maxRecsWhenFull = kMaxObjectPerChunk + 1;
