@@ -47,6 +47,8 @@
 #include "mongo/db/namespace_string.h"
 #include "mongo/db/operation_context.h"
 #include "mongo/db/ops/delete.h"
+#include "mongo/db/ops/write_ops.h"
+#include "mongo/db/ops/write_ops_exec.h"
 #include "mongo/db/range_deleter_service.h"
 #include "mongo/db/repl/repl_client_info.h"
 #include "mongo/db/repl/replication_coordinator_global.h"
@@ -56,8 +58,8 @@
 #include "mongo/db/s/move_timing_helper.h"
 #include "mongo/db/s/sharded_connection_info.h"
 #include "mongo/db/s/sharding_state.h"
+#include "mongo/db/server_parameters.h"
 #include "mongo/db/service_context.h"
-#include "mongo/db/storage/mmap_v1/dur.h"
 #include "mongo/logger/ramlog.h"
 #include "mongo/s/catalog/type_chunk.h"
 #include "mongo/s/shard_key_pattern.h"
@@ -65,6 +67,8 @@
 #include "mongo/util/fail_point_service.h"
 #include "mongo/util/log.h"
 #include "mongo/util/mongoutils/str.h"
+#include "mongo/util/producer_consumer_queue.h"
+#include "mongo/util/scopeguard.h"
 
 namespace mongo {
 
@@ -75,6 +79,15 @@ namespace {
 
 Tee* migrateLog = RamLog::get("migrate");
 
+const WriteConcernOptions kMajorityWriteConcern(WriteConcernOptions::kMajority,
+                                                // Note: Even though we're setting UNSET here,
+                                                // kMajority implies JOURNAL if journaling is
+                                                // supported by mongod and
+                                                // writeConcernMajorityJournalDefault is set to true
+                                                // in the ReplSetConfig.
+                                                WriteConcernOptions::SyncMode::UNSET,
+                                                -1);
+
 /**
  * Returns a human-readabale name of the migration manager's state.
  */
@@ -139,26 +152,33 @@ bool willOverrideLocalId(OperationContext* txn,
 bool opReplicatedEnough(OperationContext* txn,
                         const repl::OpTime& lastOpApplied,
                         const WriteConcernOptions& writeConcern) {
-    WriteConcernOptions majorityWriteConcern;
-    majorityWriteConcern.wTimeout = -1;
-    majorityWriteConcern.wMode = WriteConcernOptions::kMajority;
-    Status majorityStatus = repl::getGlobalReplicationCoordinator()
-                                ->awaitReplication(txn, lastOpApplied, majorityWriteConcern)
-                                .status;
+    WriteConcernResult writeConcernResult;
+    writeConcernResult.wTimedOut = false;
 
-    if (!writeConcern.shouldWaitForOtherNodes()) {
-        return majorityStatus.isOK();
+    Status majorityStatus =
+        waitForWriteConcern(txn, lastOpApplied, kMajorityWriteConcern, &writeConcernResult);
+    if (!majorityStatus.isOK()) {
+        if (!writeConcernResult.wTimedOut) {
+            uassertStatusOK(majorityStatus);
+        }
+        return false;
     }
 
     // Enforce the user specified write concern after "majority" so it covers the union of the 2
-    // write concerns
+    // write concerns in case the user's write concern is stronger than majority
     WriteConcernOptions userWriteConcern(writeConcern);
     userWriteConcern.wTimeout = -1;
-    Status userStatus = repl::getGlobalReplicationCoordinator()
-                            ->awaitReplication(txn, lastOpApplied, userWriteConcern)
-                            .status;
+    writeConcernResult.wTimedOut = false;
 
-    return majorityStatus.isOK() && userStatus.isOK();
+    Status userStatus =
+        waitForWriteConcern(txn, lastOpApplied, userWriteConcern, &writeConcernResult);
+    if (!userStatus.isOK()) {
+        if (!writeConcernResult.wTimedOut) {
+            uassertStatusOK(userStatus);
+        }
+        return false;
+    }
+    return true;
 }
 
 /**
@@ -212,6 +232,7 @@ MigrationDestinationManager::State MigrationDestinationManager::getState() const
 void MigrationDestinationManager::setState(State newState) {
     stdx::lock_guard<stdx::mutex> sl(_mutex);
     _state = newState;
+    _stateChangedCV.notify_all();
 }
 
 bool MigrationDestinationManager::isActive() const {
@@ -223,7 +244,21 @@ bool MigrationDestinationManager::_isActive_inlock() const {
     return _sessionId.is_initialized();
 }
 
-void MigrationDestinationManager::report(BSONObjBuilder& b) {
+void MigrationDestinationManager::report(BSONObjBuilder& b,
+                                         OperationContext* opCtx,
+                                         bool waitForSteadyOrDone) {
+    if (waitForSteadyOrDone) {
+        stdx::unique_lock<stdx::mutex> lock(_mutex);
+        try {
+            opCtx->waitForConditionOrInterruptFor(_stateChangedCV, lock, Seconds(1), [&]() -> bool {
+                return _state != READY && _state != CLONE && _state != CATCHUP;
+            });
+        } catch (...) {
+            // Ignoring this error because this is an optional parameter and we catch timeout
+            // exceptions later.
+        }
+        b.append("waited", true);
+    }
     stdx::lock_guard<stdx::mutex> sl(_mutex);
 
     b.appendBool("active", _sessionId.is_initialized());
@@ -278,6 +313,7 @@ Status MigrationDestinationManager::start(const NamespaceString& nss,
     invariant(!_scopedRegisterReceiveChunk);
 
     _state = READY;
+    _stateChangedCV.notify_all();
     _errmsg = "";
 
     _nss = nss;
@@ -314,6 +350,53 @@ Status MigrationDestinationManager::start(const NamespaceString& nss,
     return Status::OK();
 }
 
+void MigrationDestinationManager::cloneDocumentsFromDonor(
+    OperationContext* txn,
+    stdx::function<void(OperationContext*, BSONObj)> insertBatchFn,
+    stdx::function<BSONObj(OperationContext*)> fetchBatchFn) {
+
+    ProducerConsumerQueue<BSONObj> batches(1);
+    stdx::thread inserterThread{[&] {
+        Client::initThreadIfNotAlready("chunkInserter");
+        auto inserterTxn = Client::getCurrent()->makeOperationContext();
+        auto consumerGuard = MakeGuard([&] { batches.closeConsumerEnd(); });
+        try {
+            while (true) {
+                auto nextBatch = batches.pop(inserterTxn.get());
+                auto arr = nextBatch["objects"].Obj();
+                if (arr.isEmpty()) {
+                    return;
+                }
+                insertBatchFn(inserterTxn.get(), arr);
+            }
+        } catch (...) {
+            stdx::lock_guard<Client> lk(*txn->getClient());
+            txn->getServiceContext()->killOperation(txn, exceptionToStatus().code());
+            log() << "Batch insertion failed " << causedBy(redact(exceptionToStatus()));
+        }
+    }};
+    auto inserterThreadJoinGuard = MakeGuard([&] {
+        batches.closeProducerEnd();
+        inserterThread.join();
+    });
+
+    while (true) {
+        txn->checkForInterrupt();
+
+        auto res = fetchBatchFn(txn);
+
+        txn->checkForInterrupt();
+        batches.push(res.getOwned(), txn);
+        auto arr = res["objects"].Obj();
+        if (arr.isEmpty()) {
+            inserterThreadJoinGuard.Dismiss();
+            inserterThread.join();
+            txn->checkForInterrupt();
+            break;
+        }
+    }
+}
+
 bool MigrationDestinationManager::abort(const MigrationSessionId& sessionId) {
     stdx::lock_guard<stdx::mutex> sl(_mutex);
 
@@ -328,6 +411,7 @@ bool MigrationDestinationManager::abort(const MigrationSessionId& sessionId) {
     }
 
     _state = ABORT;
+    _stateChangedCV.notify_all();
     _errmsg = "aborted";
 
     return true;
@@ -336,6 +420,7 @@ bool MigrationDestinationManager::abort(const MigrationSessionId& sessionId) {
 void MigrationDestinationManager::abortWithoutSessionIdCheck() {
     stdx::lock_guard<stdx::mutex> sl(_mutex);
     _state = ABORT;
+    _stateChangedCV.notify_all();
     _errmsg = "aborted without session id check";
 }
 
@@ -360,6 +445,7 @@ bool MigrationDestinationManager::startCommit(const MigrationSessionId& sessionI
     }
 
     _state = COMMIT_START;
+    _stateChangedCV.notify_all();
 
     const auto deadline = Date_t::now() + Seconds(30);
 
@@ -367,6 +453,8 @@ bool MigrationDestinationManager::startCommit(const MigrationSessionId& sessionI
         if (stdx::cv_status::timeout ==
             _isActiveCV.wait_until(lock, deadline.toSystemTimePoint())) {
             _state = FAIL;
+            _stateChangedCV.notify_all();
+
             log() << "startCommit never finished!" << migrateLog;
             return false;
         }
@@ -397,22 +485,14 @@ void MigrationDestinationManager::_migrateThread(BSONObj min,
     try {
         _migrateDriver(
             opCtx.get(), min, max, shardKeyPattern, fromShardConnString, epoch, writeConcern);
-    } catch (std::exception& e) {
-        {
-            stdx::lock_guard<stdx::mutex> sl(_mutex);
-            _state = FAIL;
-            _errmsg = e.what();
-        }
-
-        error() << "migrate failed: " << redact(e.what()) << migrateLog;
     } catch (...) {
         {
             stdx::lock_guard<stdx::mutex> sl(_mutex);
             _state = FAIL;
-            _errmsg = "UNKNOWN ERROR";
+            _errmsg = redact(exceptionToStatus().toString());
         }
 
-        error() << "migrate failed with unknown exception" << migrateLog;
+        error() << "migrate failed: " << redact(exceptionToStatus()) << migrateLog;
     }
 
     if (getState() != DONE) {
@@ -429,6 +509,17 @@ void MigrationDestinationManager::_migrateThread(BSONObj min,
     _isActiveCV.notify_all();
 }
 
+// The maximum number of documents to insert in a single batch during migration clone.
+// secondaryThrottle and migrateCloneInsertionBatchDelayMS apply between each batch.
+// 0 or negative values (the default) means no limit to batch size.
+// 1 corresponds to 3.4.16 (and earlier) behavior.
+MONGO_EXPORT_SERVER_PARAMETER(migrateCloneInsertionBatchSize, int, 0);
+
+// Time in milliseconds between batches of insertions during migration clone.
+// This is in addition to any time spent waiting for replication (secondaryThrottle).
+// Defaults to 0.
+MONGO_EXPORT_SERVER_PARAMETER(migrateCloneInsertionBatchDelayMS, int, 0);
+
 void MigrationDestinationManager::_migrateDriver(OperationContext* txn,
                                                  const BSONObj& min,
                                                  const BSONObj& max,
@@ -588,6 +679,12 @@ void MigrationDestinationManager::_migrateDriver(OperationContext* txn,
             wunit.commit();
         }
 
+        Status status = _notePending(txn, _nss, min, max, epoch);
+        if (!status.isOK()) {
+            setState(FAIL);
+            return;
+        }
+
         timing.done(1);
         MONGO_FAIL_POINT_PAUSE_WHILE_SET(migrateThreadHangAtStep1);
     }
@@ -603,7 +700,12 @@ void MigrationDestinationManager::_migrateDriver(OperationContext* txn,
         // results
         deleterOptions.waitForOpenCursors = false;
         deleterOptions.fromMigrate = true;
-        deleterOptions.onlyRemoveOrphanedDocs = true;
+
+        // There is no need to perform checking for orphaned docs as part of the range deletion,
+        // because the call above (to _notePending) will ensure that the chunk which is coming in is
+        // not currently owned by this shard and the scopedRegisterReceiveChunk would prevent this
+        // chunk from getting received while range deletion is running.
+        deleterOptions.onlyRemoveOrphanedDocs = false;
         deleterOptions.removeSaverReason = "preCleanup";
 
         if (!getDeleter()->deleteNow(txn, deleterOptions, &errmsg)) {
@@ -612,12 +714,6 @@ void MigrationDestinationManager::_migrateDriver(OperationContext* txn,
             return;
         }
 
-        Status status = _notePending(txn, _nss, min, max, epoch);
-        if (!status.isOK()) {
-            setState(FAIL);
-            return;
-        }
-
         timing.done(2);
         MONGO_FAIL_POINT_PAUSE_WHILE_SET(migrateThreadHangAtStep2);
     }
@@ -628,84 +724,78 @@ void MigrationDestinationManager::_migrateDriver(OperationContext* txn,
 
         const BSONObj migrateCloneRequest = createMigrateCloneRequest(_nss, *_sessionId);
 
-        while (true) {
-            BSONObj res;
-            if (!conn->runCommand("admin",
-                                  migrateCloneRequest,
-                                  res)) {  // gets array of objects to copy, in disk order
-                setState(FAIL);
-                errmsg = "_migrateClone failed: ";
-                errmsg += redact(res.toString());
-                error() << errmsg << migrateLog;
-                conn.done();
-                return;
-            }
+        auto assertNotAborted = [&](OperationContext* opCtx) {
+            opCtx->checkForInterrupt();
+            uassert(40655, "Migration aborted while copying documents", getState() != ABORT);
+        };
 
-            BSONObj arr = res["objects"].Obj();
-            int thisTime = 0;
+        auto insertBatchFn = [&](OperationContext* opCtx, BSONObj arr) {
+            auto it = arr.begin();
+            while (it != arr.end()) {
+                int batchNumCloned = 0;
+                int batchClonedBytes = 0;
+                int batchMaxCloned = migrateCloneInsertionBatchSize.load();
 
-            BSONObjIterator i(arr);
-            while (i.more()) {
-                txn->checkForInterrupt();
+                assertNotAborted(opCtx);
 
-                if (getState() == ABORT) {
-                    errmsg = "Migration aborted while copying documents";
-                    error() << errmsg << migrateLog;
-                    return;
+                std::vector<BSONObj> toInsert;
+                while (it != arr.end() &&
+                       (batchMaxCloned <= 0 || batchNumCloned < batchMaxCloned)) {
+                    const auto& doc = *it;
+                    BSONObj docToClone = doc.Obj();
+                    toInsert.push_back(docToClone);
+                    batchNumCloned++;
+                    batchClonedBytes += docToClone.objsize();
+                    it++;
                 }
+                InsertOp insertOp;
+                insertOp.ns = _nss;
+                insertOp.documents = toInsert;
 
-                BSONObj docToClone = i.next().Obj();
-                {
-                    OldClientWriteContext cx(txn, _nss.ns());
+                const WriteResult reply = performInserts(opCtx, insertOp, true);
 
-                    BSONObj localDoc;
-                    if (willOverrideLocalId(txn,
-                                            _nss.ns(),
-                                            min,
-                                            max,
-                                            shardKeyPattern,
-                                            cx.db(),
-                                            docToClone,
-                                            &localDoc)) {
-                        string errMsg = str::stream() << "cannot migrate chunk, local document "
-                                                      << redact(localDoc)
-                                                      << " has same _id as cloned "
-                                                      << "remote document " << redact(docToClone);
-
-                        warning() << errMsg;
-
-                        // Exception will abort migration cleanly
-                        uasserted(16976, errMsg);
-                    }
-
-                    Helpers::upsert(txn, _nss.ns(), docToClone, true);
+                for (unsigned long i = 0; i < reply.results.size(); ++i) {
+                    uassertStatusOK(reply.results[i]);
                 }
-                thisTime++;
 
                 {
                     stdx::lock_guard<stdx::mutex> statsLock(_mutex);
-                    _numCloned++;
-                    _clonedBytes += docToClone.objsize();
+                    _numCloned += batchNumCloned;
+                    _clonedBytes += batchClonedBytes;
                 }
 
                 if (writeConcern.shouldWaitForOtherNodes()) {
                     repl::ReplicationCoordinator::StatusAndDuration replStatus =
-                        repl::getGlobalReplicationCoordinator()->awaitReplication(
-                            txn,
-                            repl::ReplClientInfo::forClient(txn->getClient()).getLastOp(),
+                        repl::ReplicationCoordinator::get(opCtx)->awaitReplication(
+                            opCtx,
+                            repl::ReplClientInfo::forClient(opCtx->getClient()).getLastOp(),
                             writeConcern);
                     if (replStatus.status.code() == ErrorCodes::WriteConcernFailed) {
                         warning() << "secondaryThrottle on, but doc insert timed out; "
                                      "continuing";
                     } else {
-                        massertStatusOK(replStatus.status);
+                        uassertStatusOK(replStatus.status);
                     }
                 }
-            }
 
-            if (thisTime == 0)
-                break;
-        }
+                sleepmillis(migrateCloneInsertionBatchDelayMS.load());
+            }
+        };
+
+        auto fetchBatchFn = [&](OperationContext* txn) {
+            BSONObj res;
+            if (!conn->runCommand("admin",
+                                  migrateCloneRequest,
+                                  res)) {  // gets array of objects to copy, in disk order
+                conn.done();
+                const std::string errMsg = str::stream() << "_migrateClone failed: "
+                                                         << redact(res.toString());
+                uasserted(40656, errMsg);
+            }
+            return res;
+        };
+
+        cloneDocumentsFromDonor(txn, insertBatchFn, fetchBatchFn);
 
         timing.done(3);
         MONGO_FAIL_POINT_PAUSE_WHILE_SET(migrateThreadHangAtStep3);
@@ -944,9 +1034,9 @@ bool MigrationDestinationManager::_applyMigrateOp(OperationContext* txn,
             BSONObj localDoc;
             if (willOverrideLocalId(
                     txn, ns, min, max, shardKeyPattern, cx.db(), updatedDoc, &localDoc)) {
-                string errMsg = str::stream() << "cannot migrate chunk, local document " << localDoc
-                                              << " has same _id as reloaded remote document "
-                                              << updatedDoc;
+                string errMsg = str::stream()
+                    << "cannot migrate chunk, local document " << redact(localDoc)
+                    << " has same _id as reloaded remote document " << redact(updatedDoc);
 
                 warning() << errMsg;
 
@@ -979,20 +1069,8 @@ bool MigrationDestinationManager::_flushPendingWrites(OperationContext* txn,
         return false;
     }
 
-    log() << "migrate commit succeeded flushing to secondaries for '" << ns << "' " << min << " -> "
-          << max << migrateLog;
-
-    {
-        // Get global lock to wait for write to be commited to journal.
-        ScopedTransaction scopedXact(txn, MODE_S);
-        Lock::GlobalRead lk(txn->lockState());
-
-        // if durability is on, force a write to journal
-        if (getDur().commitNow(txn)) {
-            log() << "migrate commit flushed to journal for '" << ns << "' " << redact(min)
-                  << " -> " << redact(max) << migrateLog;
-        }
-    }
+    log() << "migrate commit succeeded flushing to secondaries for '" << ns << "' " << redact(min)
+          << " -> " << redact(max) << migrateLog;
 
     return true;
 }
@@ -1014,9 +1092,9 @@ Status MigrationDestinationManager::_notePending(OperationContext* txn,
         return {ErrorCodes::StaleShardVersion,
                 str::stream() << "could not note chunk "
                               << "["
-                              << min
+                              << redact(min)
                               << ","
-                              << max
+                              << redact(max)
                               << ")"
                               << " as pending because the epoch for "
                               << nss.ns()
