@@ -53,9 +53,6 @@ namespace {
 // This is a global resource, which protects accesses to the catalog metadata (instance-wide).
 // It is never used with KVEngines that support doc-level locking so this should never conflict
 // with anything else.
-//
-// NOTE: Must be locked *before* _identLock.
-const ResourceId resourceIdCatalogMetadata(RESOURCE_METADATA, 1ULL);
 
 const char kIsFeatureDocumentFieldName[] = "isFeatureDoc";
 const char kNamespaceFieldName[] = "ns";
@@ -123,12 +120,6 @@ bool KVCatalog::FeatureTracker::isFeatureDocument(BSONObj obj) {
 }
 
 Status KVCatalog::FeatureTracker::isCompatibleWithCurrentCode(OperationContext* opCtx) const {
-    std::unique_ptr<Lock::ResourceLock> rLk;
-    if (!_catalog->_isRsThreadSafe && opCtx->lockState()) {
-        rLk = stdx::make_unique<Lock::ResourceLock>(
-            opCtx->lockState(), resourceIdCatalogMetadata, MODE_S);
-    }
-
     FeatureBits versionInfo = getInfo(opCtx);
 
     uint64_t unrecognizedNonRepairableFeatures =
@@ -173,22 +164,12 @@ std::unique_ptr<KVCatalog::FeatureTracker> KVCatalog::FeatureTracker::create(
 
 bool KVCatalog::FeatureTracker::isNonRepairableFeatureInUse(OperationContext* opCtx,
                                                             NonRepairableFeature feature) const {
-    std::unique_ptr<Lock::ResourceLock> rLk;
-    if (!_catalog->_isRsThreadSafe && opCtx->lockState()) {
-        rLk.reset(new Lock::ResourceLock(opCtx->lockState(), resourceIdCatalogMetadata, MODE_S));
-    }
-
     FeatureBits versionInfo = getInfo(opCtx);
     return versionInfo.nonRepairableFeatures & static_cast<NonRepairableFeatureMask>(feature);
 }
 
 void KVCatalog::FeatureTracker::markNonRepairableFeatureAsInUse(OperationContext* opCtx,
                                                                 NonRepairableFeature feature) {
-    std::unique_ptr<Lock::ResourceLock> rLk;
-    if (!_catalog->_isRsThreadSafe && opCtx->lockState()) {
-        rLk.reset(new Lock::ResourceLock(opCtx->lockState(), resourceIdCatalogMetadata, MODE_X));
-    }
-
     FeatureBits versionInfo = getInfo(opCtx);
     versionInfo.nonRepairableFeatures |= static_cast<NonRepairableFeatureMask>(feature);
     putInfo(opCtx, versionInfo);
@@ -196,11 +177,6 @@ void KVCatalog::FeatureTracker::markNonRepairableFeatureAsInUse(OperationContext
 
 void KVCatalog::FeatureTracker::markNonRepairableFeatureAsNotInUse(OperationContext* opCtx,
                                                                    NonRepairableFeature feature) {
-    std::unique_ptr<Lock::ResourceLock> rLk;
-    if (!_catalog->_isRsThreadSafe && opCtx->lockState()) {
-        rLk.reset(new Lock::ResourceLock(opCtx->lockState(), resourceIdCatalogMetadata, MODE_X));
-    }
-
     FeatureBits versionInfo = getInfo(opCtx);
     versionInfo.nonRepairableFeatures &= ~static_cast<NonRepairableFeatureMask>(feature);
     putInfo(opCtx, versionInfo);
@@ -208,22 +184,12 @@ void KVCatalog::FeatureTracker::markNonRepairableFeatureAsNotInUse(OperationCont
 
 bool KVCatalog::FeatureTracker::isRepairableFeatureInUse(OperationContext* opCtx,
                                                          RepairableFeature feature) const {
-    std::unique_ptr<Lock::ResourceLock> rLk;
-    if (!_catalog->_isRsThreadSafe && opCtx->lockState()) {
-        rLk.reset(new Lock::ResourceLock(opCtx->lockState(), resourceIdCatalogMetadata, MODE_S));
-    }
-
     FeatureBits versionInfo = getInfo(opCtx);
     return versionInfo.repairableFeatures & static_cast<RepairableFeatureMask>(feature);
 }
 
 void KVCatalog::FeatureTracker::markRepairableFeatureAsInUse(OperationContext* opCtx,
                                                              RepairableFeature feature) {
-    std::unique_ptr<Lock::ResourceLock> rLk;
-    if (!_catalog->_isRsThreadSafe && opCtx->lockState()) {
-        rLk.reset(new Lock::ResourceLock(opCtx->lockState(), resourceIdCatalogMetadata, MODE_X));
-    }
-
     FeatureBits versionInfo = getInfo(opCtx);
     versionInfo.repairableFeatures |= static_cast<RepairableFeatureMask>(feature);
     putInfo(opCtx, versionInfo);
@@ -231,11 +197,6 @@ void KVCatalog::FeatureTracker::markRepairableFeatureAsInUse(OperationContext* o
 
 void KVCatalog::FeatureTracker::markRepairableFeatureAsNotInUse(OperationContext* opCtx,
                                                                 RepairableFeature feature) {
-    std::unique_ptr<Lock::ResourceLock> rLk;
-    if (!_catalog->_isRsThreadSafe && opCtx->lockState()) {
-        rLk.reset(new Lock::ResourceLock(opCtx->lockState(), resourceIdCatalogMetadata, MODE_X));
-    }
-
     FeatureBits versionInfo = getInfo(opCtx);
     versionInfo.repairableFeatures &= ~static_cast<RepairableFeatureMask>(feature);
     putInfo(opCtx, versionInfo);
@@ -243,10 +204,6 @@ void KVCatalog::FeatureTracker::markRepairableFeatureAsNotInUse(OperationContext
 
 KVCatalog::FeatureTracker::FeatureBits KVCatalog::FeatureTracker::getInfo(
     OperationContext* opCtx) const {
-    if (!_catalog->_isRsThreadSafe && opCtx->lockState()) {
-        invariant(opCtx->lockState()->isLockHeldForMode(resourceIdCatalogMetadata, MODE_S));
-    }
-
     if (_rid.isNull()) {
         return {};
     }
@@ -274,10 +231,6 @@ KVCatalog::FeatureTracker::FeatureBits KVCatalog::FeatureTracker::getInfo(
 }
 
 void KVCatalog::FeatureTracker::putInfo(OperationContext* opCtx, const FeatureBits& versionInfo) {
-    if (!_catalog->_isRsThreadSafe && opCtx->lockState()) {
-        invariant(opCtx->lockState()->isLockHeldForMode(resourceIdCatalogMetadata, MODE_X));
-    }
-
     BSONObjBuilder bob;
     bob.appendBool(kIsFeatureDocumentFieldName, true);
     // We intentionally include the "ns" field with a null value in the feature document to prevent
@@ -306,12 +259,8 @@ void KVCatalog::FeatureTracker::putInfo(OperationContext* opCtx, const FeatureBi
     }
 }
 
-KVCatalog::KVCatalog(RecordStore* rs,
-                     bool isRsThreadSafe,
-                     bool directoryPerDb,
-                     bool directoryForIndexes)
+KVCatalog::KVCatalog(RecordStore* rs, bool directoryPerDb, bool directoryForIndexes)
     : _rs(rs),
-      _isRsThreadSafe(isRsThreadSafe),
       _directoryPerDb(directoryPerDb),
       _directoryForIndexes(directoryForIndexes),
       _rand(_newRand()) {}
@@ -392,11 +341,6 @@ Status KVCatalog::newCollection(OperationContext* opCtx,
     invariant(opCtx->lockState() == NULL ||
               opCtx->lockState()->isDbLockedForMode(nsToDatabaseSubstring(ns), MODE_X));
 
-    std::unique_ptr<Lock::ResourceLock> rLk;
-    if (!_isRsThreadSafe && opCtx->lockState()) {
-        rLk.reset(new Lock::ResourceLock(opCtx->lockState(), resourceIdCatalogMetadata, MODE_X));
-    }
-
     const string ident = _newUniqueIdent(ns, "collection");
 
     stdx::lock_guard<stdx::mutex> lk(_identsLock);
@@ -444,11 +388,6 @@ std::string KVCatalog::getIndexIdent(OperationContext* opCtx,
 }
 
 BSONObj KVCatalog::_findEntry(OperationContext* opCtx, StringData ns, RecordId* out) const {
-    std::unique_ptr<Lock::ResourceLock> rLk;
-    if (!_isRsThreadSafe && opCtx->lockState()) {
-        rLk.reset(new Lock::ResourceLock(opCtx->lockState(), resourceIdCatalogMetadata, MODE_S));
-    }
-
     RecordId dl;
     {
         stdx::lock_guard<stdx::mutex> lk(_identsLock);
@@ -488,11 +427,6 @@ const BSONCollectionCatalogEntry::MetaData KVCatalog::getMetaData(OperationConte
 void KVCatalog::putMetaData(OperationContext* opCtx,
                             StringData ns,
                             BSONCollectionCatalogEntry::MetaData& md) {
-    std::unique_ptr<Lock::ResourceLock> rLk;
-    if (!_isRsThreadSafe && opCtx->lockState()) {
-        rLk.reset(new Lock::ResourceLock(opCtx->lockState(), resourceIdCatalogMetadata, MODE_X));
-    }
-
     RecordId loc;
     BSONObj obj = _findEntry(opCtx, ns, &loc);
 
@@ -533,11 +467,6 @@ Status KVCatalog::renameCollection(OperationContext* opCtx,
                                    StringData fromNS,
                                    StringData toNS,
                                    bool stayTemp) {
-    std::unique_ptr<Lock::ResourceLock> rLk;
-    if (!_isRsThreadSafe && opCtx->lockState()) {
-        rLk.reset(new Lock::ResourceLock(opCtx->lockState(), resourceIdCatalogMetadata, MODE_X));
-    }
-
     RecordId loc;
     BSONObj old = _findEntry(opCtx, fromNS, &loc).getOwned();
     {
@@ -575,11 +504,6 @@ Status KVCatalog::renameCollection(OperationContext* opCtx,
 Status KVCatalog::dropCollection(OperationContext* opCtx, StringData ns) {
     invariant(opCtx->lockState() == NULL ||
               opCtx->lockState()->isDbLockedForMode(nsToDatabaseSubstring(ns), MODE_X));
-    std::unique_ptr<Lock::ResourceLock> rLk;
-    if (!_isRsThreadSafe && opCtx->lockState()) {
-        rLk.reset(new Lock::ResourceLock(opCtx->lockState(), resourceIdCatalogMetadata, MODE_X));
-    }
-
     stdx::lock_guard<stdx::mutex> lk(_identsLock);
     const NSToIdentMap::iterator it = _idents.find(ns.toString());
     if (it == _idents.end()) {
