@@ -46,6 +46,7 @@
 #include "mongo/rpc/metadata/metadata_hook.h"
 #include "mongo/stdx/chrono.h"
 #include "mongo/stdx/memory.h"
+#include "mongo/util/concurrency/idle_thread_block.h"
 #include "mongo/util/concurrency/thread_name.h"
 #include "mongo/util/log.h"
 #include "mongo/util/net/sock.h"
@@ -185,6 +186,7 @@ void NetworkInterfaceASIO::waitForWork() {
     stdx::unique_lock<stdx::mutex> lk(_executorMutex);
     // TODO: This can be restructured with a lambda.
     while (!_isExecutorRunnable) {
+        MONGO_IDLE_THREAD_BLOCK;
         _isExecutorRunnableCondition.wait(lk);
     }
     _isExecutorRunnable = false;
@@ -198,6 +200,7 @@ void NetworkInterfaceASIO::waitForWorkUntil(Date_t when) {
         if (waitTime <= Milliseconds(0)) {
             break;
         }
+        MONGO_IDLE_THREAD_BLOCK;
         _isExecutorRunnableCondition.wait_for(lk, waitTime.toSystemDuration());
     }
     _isExecutorRunnable = false;
@@ -477,7 +480,17 @@ Status NetworkInterfaceASIO::setAlarm(Date_t when, const stdx::function<void()>&
         return exceptionToStatus();
     }
 
-    alarm->async_wait([alarm, this, action](std::error_code ec) {
+    alarm->async_wait([alarm, this, action, when](std::error_code ec) {
+        const auto nowValue = now();
+        if (nowValue < when) {
+            warning() << "ASIO alarm returned early. Expected at: " << when
+                      << ", fired at: " << nowValue;
+            const auto status = setAlarm(when, action);
+            if ((!status.isOK()) && (status.code() != ErrorCodes::ShutdownInProgress)) {
+                fassertFailedWithStatus(40383, status);
+            }
+            return;
+        }
         if (!ec) {
             return action();
         } else if (ec != asio::error::operation_aborted) {
@@ -520,5 +533,9 @@ void NetworkInterfaceASIO::_failWithInfo_inlock(const char* file,
     fassertFailedWithStatus(34429, status);
 }
 
+void NetworkInterfaceASIO::dropConnections(const HostAndPort& hostAndPort) {
+    _connectionPool.dropConnections(hostAndPort);
+}
+
 }  // namespace executor
 }  // namespace mongo
