@@ -46,16 +46,16 @@
 #include "mongo/db/client.h"
 #include "mongo/db/commands.h"
 #include "mongo/db/hasher.h"
+#include "mongo/db/index/index_descriptor.h"
 #include "mongo/db/operation_context.h"
 #include "mongo/db/query/collation/collator_factory_interface.h"
 #include "mongo/db/write_concern_options.h"
 #include "mongo/s/balancer_configuration.h"
-#include "mongo/s/catalog/catalog_cache.h"
 #include "mongo/s/catalog/sharding_catalog_client.h"
-#include "mongo/s/chunk_manager.h"
+#include "mongo/s/catalog/type_database.h"
+#include "mongo/s/catalog_cache.h"
 #include "mongo/s/client/shard_registry.h"
 #include "mongo/s/commands/cluster_write.h"
-#include "mongo/s/config.h"
 #include "mongo/s/config_server_client.h"
 #include "mongo/s/grid.h"
 #include "mongo/s/migration_secondary_throttle_options.h"
@@ -63,40 +63,105 @@
 #include "mongo/util/log.h"
 
 namespace mongo {
-
-using std::shared_ptr;
-using std::list;
-using std::set;
-using std::string;
-using std::vector;
-
 namespace {
 
+/**
+ * Constructs the BSON specification document for the given namespace, index key and options.
+ */
+BSONObj createIndexDoc(const std::string& ns,
+                       const BSONObj& keys,
+                       const BSONObj& collation,
+                       bool unique) {
+    BSONObjBuilder indexDoc;
+    indexDoc.append("ns", ns);
+    indexDoc.append("key", keys);
+
+    StringBuilder indexName;
+
+    bool isFirstKey = true;
+    for (BSONObjIterator keyIter(keys); keyIter.more();) {
+        BSONElement currentKey = keyIter.next();
+
+        if (isFirstKey) {
+            isFirstKey = false;
+        } else {
+            indexName << "_";
+        }
+
+        indexName << currentKey.fieldName() << "_";
+        if (currentKey.isNumber()) {
+            indexName << currentKey.numberInt();
+        } else {
+            indexName << currentKey.str();  // this should match up with shell command
+        }
+    }
+
+    indexDoc.append("name", indexName.str());
+
+    if (!collation.isEmpty()) {
+        // Creating an index with the "collation" option requires a v=2 index.
+        indexDoc.append("v", static_cast<int>(IndexDescriptor::IndexVersion::kV2));
+        indexDoc.append("collation", collation);
+    }
+
+    if (unique && !IndexDescriptor::isIdIndexPattern(keys)) {
+        indexDoc.appendBool("unique", unique);
+    }
+
+    return indexDoc.obj();
+}
+
+/**
+ * Used only for writes to the config server, config and admin databases.
+ */
+Status clusterCreateIndex(OperationContext* txn,
+                          const std::string& ns,
+                          const BSONObj& keys,
+                          const BSONObj& collation,
+                          bool unique) {
+    const NamespaceString nss(ns);
+
+    // Go through the shard insert path
+    std::unique_ptr<BatchedInsertRequest> insert(new BatchedInsertRequest());
+    insert->addToDocuments(createIndexDoc(ns, keys, collation, unique));
+
+    BatchedCommandRequest request(insert.release());
+    request.setNS(NamespaceString(nss.getSystemIndexesCollection()));
+    request.setWriteConcern(WriteConcernOptions::Acknowledged);
+
+    BatchedCommandResponse response;
+
+    ClusterWriter writer(false, 0);
+    writer.write(txn, request, &response);
+
+    return response.toStatus();
+}
+
 class ShardCollectionCmd : public Command {
 public:
     ShardCollectionCmd() : Command("shardCollection", false, "shardcollection") {}
 
-    virtual bool slaveOk() const {
+    bool slaveOk() const override {
         return true;
     }
 
-    virtual bool adminOnly() const {
+    bool adminOnly() const override {
         return true;
     }
 
-    virtual bool supportsWriteConcern(const BSONObj& cmd) const override {
+    bool supportsWriteConcern(const BSONObj& cmd) const override {
         return false;
     }
 
-    virtual void help(std::stringstream& help) const {
+    void help(std::stringstream& help) const override {
         help << "Shard a collection. Requires key. Optional unique."
              << " Sharding must already be enabled for the database.\n"
              << "   { enablesharding : \"<dbname>\" }\n";
     }
 
-    virtual Status checkAuthForCommand(Client* client,
-                                       const std::string& dbname,
-                                       const BSONObj& cmdObj) {
+    Status checkAuthForCommand(Client* client,
+                               const std::string& dbname,
+                               const BSONObj& cmdObj) override {
         if (!AuthorizationSession::get(client)->isAuthorizedForActionsOnResource(
                 ResourcePattern::forExactNamespace(NamespaceString(parseNs(dbname, cmdObj))),
                 ActionType::enableSharding)) {
@@ -106,30 +171,43 @@ public:
         return Status::OK();
     }
 
-    virtual std::string parseNs(const std::string& dbname, const BSONObj& cmdObj) const {
+    std::string parseNs(const std::string& dbname, const BSONObj& cmdObj) const override {
         return parseNsFullyQualified(dbname, cmdObj);
     }
 
-    virtual bool run(OperationContext* txn,
-                     const std::string& dbname,
-                     BSONObj& cmdObj,
-                     int options,
-                     std::string& errmsg,
-                     BSONObjBuilder& result) {
+    bool run(OperationContext* opCtx,
+             const std::string& dbname,
+             BSONObj& cmdObj,
+             int options,
+             std::string& errmsg,
+             BSONObjBuilder& result) override {
         const NamespaceString nss(parseNs(dbname, cmdObj));
-        uassert(ErrorCodes::InvalidNamespace, "Invalid namespace", nss.isValid());
 
-        auto config = uassertStatusOK(grid.catalogCache()->getDatabase(txn, nss.db().toString()));
-        uassert(ErrorCodes::IllegalOperation,
-                str::stream() << "sharding not enabled for db " << nss.db(),
-                config->isShardingEnabled());
+        auto const catalogClient = Grid::get(opCtx)->catalogClient(opCtx);
+        auto const shardRegistry = Grid::get(opCtx)->shardRegistry();
+        auto const catalogCache = Grid::get(opCtx)->catalogCache();
 
+        // Ensure sharding is allowed on the database by reading directly from the config server,
+        // because reading through the cache might produce a stale entry if the "enableSharding" was
+        // called through a different mongos
+        {
+            const auto opTimeWithDbt =
+                uassertStatusOK(catalogClient->getDatabase(opCtx, nss.db().toString()));
+            const auto& dbt = opTimeWithDbt.value;
+            uassert(ErrorCodes::IllegalOperation,
+                    str::stream() << "sharding not enabled for db " << nss.db(),
+                    dbt.getSharded());
+        }
+
+        auto routingInfo = uassertStatusOK(catalogCache->getCollectionRoutingInfo(opCtx, nss));
+
+        // Ensure that the collection is not sharded already
         uassert(ErrorCodes::IllegalOperation,
                 str::stream() << "sharding already enabled for collection " << nss.ns(),
-                !config->isSharded(nss.ns()));
+                !routingInfo.cm());
 
-        // NOTE: We *must* take ownership of the key here - otherwise the shared BSONObj
-        // becomes corrupt as soon as the command ends.
+        // NOTE: We *must* take ownership of the key here - otherwise the shared BSONObj becomes
+        // corrupt as soon as the command ends.
         BSONObj proposedKey = cmdObj.getObjectField("key").getOwned();
         if (proposedKey.isEmpty()) {
             errmsg = "no shard key";
@@ -165,7 +243,7 @@ public:
                 bsonExtractTypedField(cmdObj, "collation", BSONType::Object, &collationElement);
             if (collationStatus.isOK()) {
                 // Ensure that the collation is valid. Currently we only allow the simple collation.
-                auto collator = CollatorFactoryInterface::get(txn->getServiceContext())
+                auto collator = CollatorFactoryInterface::get(opCtx->getServiceContext())
                                     ->makeFromBSON(collationElement.Obj());
                 if (!collator.getStatus().isOK()) {
                     return appendCommandStatus(result, collator.getStatus());
@@ -187,9 +265,10 @@ public:
             }
         }
 
-        vector<ShardId> shardIds;
-        grid.shardRegistry()->getAllShardIds(&shardIds);
-        int numShards = shardIds.size();
+        std::vector<ShardId> shardIds;
+        shardRegistry->getAllShardIds(&shardIds);
+
+        const int numShards = shardIds.size();
 
         // Cannot have more than 8192 initial chunks per shard. Setting a maximum of 1,000,000
         // chunks in total to limit the amount of memory this command consumes so there is less
@@ -205,21 +284,14 @@ public:
         }
 
         // The rest of the checks require a connection to the primary db
-        ConnectionString shardConnString;
-        {
-            const auto shard =
-                uassertStatusOK(grid.shardRegistry()->getShard(txn, config->getPrimaryId()));
-            shardConnString = shard->getConnString();
-        }
-
-        ScopedDbConnection conn(shardConnString);
+        ScopedDbConnection conn(routingInfo.primary()->getConnString());
 
         // Retrieve the collection metadata in order to verify that it is legal to shard this
         // collection.
         BSONObj res;
         {
-            list<BSONObj> all =
-                conn->getCollectionInfos(config->name(), BSON("name" << nss.coll()));
+            std::list<BSONObj> all =
+                conn->getCollectionInfos(nss.db().toString(), BSON("name" << nss.coll()));
             if (!all.empty()) {
                 res = all.front().getOwned();
             }
@@ -322,12 +394,12 @@ public:
         // 5. If the collection is empty, and it's still possible to create an index
         //    on the proposed key, we go ahead and do so.
 
-        list<BSONObj> indexes = conn->getIndexSpecs(nss.ns());
+        std::list<BSONObj> indexes = conn->getIndexSpecs(nss.ns());
 
         // 1.  Verify consistency with existing unique indexes
         ShardKeyPattern proposedShardKey(proposedKey);
-        for (list<BSONObj>::iterator it = indexes.begin(); it != indexes.end(); ++it) {
-            BSONObj idx = *it;
+
+        for (const auto& idx : indexes) {
             BSONObj currentKey = idx["key"].embeddedObject();
             bool isUnique = idx["unique"].trueValue();
 
@@ -345,8 +417,7 @@ public:
         // 2. Check for a useful index
         bool hasUsefulIndexForKey = false;
 
-        for (list<BSONObj>::iterator it = indexes.begin(); it != indexes.end(); ++it) {
-            BSONObj idx = *it;
+        for (const auto& idx : indexes) {
             BSONObj currentKey = idx["key"].embeddedObject();
             // Check 2.i. and 2.ii.
             if (!idx["sparse"].trueValue() && idx["filter"].eoo() && idx["collation"].eoo() &&
@@ -371,12 +442,12 @@ public:
 
         // 3. If proposed key is required to be unique, additionally check for exact match.
         bool careAboutUnique = cmdObj["unique"].trueValue();
+
         if (hasUsefulIndexForKey && careAboutUnique) {
             BSONObj eqQuery = BSON("ns" << nss.ns() << "key" << proposedKey);
             BSONObj eqQueryResult;
 
-            for (list<BSONObj>::iterator it = indexes.begin(); it != indexes.end(); ++it) {
-                BSONObj idx = *it;
+            for (const auto& idx : indexes) {
                 if (SimpleBSONObjComparator::kInstance.evaluate(idx["key"].embeddedObject() ==
                                                                 proposedKey)) {
                     eqQueryResult = idx;
@@ -431,7 +502,7 @@ public:
             BSONObj collationArg =
                 !defaultCollation.isEmpty() ? CollationSpec::kSimpleSpec : BSONObj();
             Status status =
-                clusterCreateIndex(txn, nss.ns(), proposedKey, collationArg, careAboutUnique);
+                clusterCreateIndex(opCtx, nss.ns(), proposedKey, collationArg, careAboutUnique);
             if (!status.isOK()) {
                 errmsg = str::stream() << "ensureIndex failed to create index on "
                                        << "primary shard: " << status.reason();
@@ -453,8 +524,8 @@ public:
         // 2. move them one at a time
         // 3. split the big chunks to achieve the desired total number of initial chunks
 
-        vector<BSONObj> initSplits;  // there will be at most numShards-1 of these
-        vector<BSONObj> allSplits;   // all of the initial desired split points
+        std::vector<BSONObj> initSplits;  // there will be at most numShards-1 of these
+        std::vector<BSONObj> allSplits;   // all of the initial desired split points
 
         // only pre-split when using a hashed shard key and collection is still empty
         if (isHashedShardKey && isEmpty) {
@@ -510,41 +581,41 @@ public:
 
         audit::logShardCollection(Client::getCurrent(), nss.ns(), proposedKey, careAboutUnique);
 
-        Status status = grid.catalogClient(txn)->shardCollection(txn,
-                                                                 nss.ns(),
-                                                                 proposedShardKey,
-                                                                 defaultCollation,
-                                                                 careAboutUnique,
-                                                                 initSplits,
-                                                                 std::set<ShardId>{});
-        if (!status.isOK()) {
-            return appendCommandStatus(result, status);
-        }
-
-        // Make sure the cached metadata for the collection knows that we are now sharded
-        config = uassertStatusOK(grid.catalogCache()->getDatabase(txn, nss.db().toString()));
-        config->getChunkManager(txn, nss.ns(), true /* force */);
+        uassertStatusOK(catalogClient->shardCollection(opCtx,
+                                                       nss.ns(),
+                                                       proposedShardKey,
+                                                       defaultCollation,
+                                                       careAboutUnique,
+                                                       initSplits,
+                                                       std::set<ShardId>{}));
 
         result << "collectionsharded" << nss.ns();
 
+        // Make sure the cached metadata for the collection knows that we are now sharded
+        catalogCache->invalidateShardedCollection(nss);
+
         // Only initially move chunks when using a hashed shard key
         if (isHashedShardKey && isEmpty) {
-            // Reload the new config info.  If we created more than one initial chunk, then
-            // we need to move them around to balance.
-            shared_ptr<ChunkManager> chunkManager = config->getChunkManager(txn, nss.ns(), true);
-            ChunkMap chunkMap = chunkManager->getChunkMap();
+            routingInfo = uassertStatusOK(catalogCache->getCollectionRoutingInfo(opCtx, nss));
+            uassert(ErrorCodes::ConflictingOperationInProgress,
+                    "Collection was successfully written as sharded but got dropped before it "
+                    "could be evenly distributed",
+                    routingInfo.cm());
+            auto chunkManager = routingInfo.cm();
+
+            const auto chunkMap = chunkManager->chunkMap();
 
             // 2. Move and commit each "big chunk" to a different shard.
             int i = 0;
             for (ChunkMap::const_iterator c = chunkMap.begin(); c != chunkMap.end(); ++c, ++i) {
                 const ShardId& shardId = shardIds[i % numShards];
-                const auto toStatus = grid.shardRegistry()->getShard(txn, shardId);
+                const auto toStatus = shardRegistry->getShard(opCtx, shardId);
                 if (!toStatus.isOK()) {
                     continue;
                 }
                 const auto to = toStatus.getValue();
 
-                shared_ptr<Chunk> chunk = c->second;
+                auto chunk = c->second;
 
                 // Can't move chunk to shard it's already on
                 if (to->getId() == chunk->getShardId()) {
@@ -559,10 +630,10 @@ public:
                 chunkType.setVersion(chunkManager->getVersion());
 
                 Status moveStatus = configsvr_client::moveChunk(
-                    txn,
+                    opCtx,
                     chunkType,
                     to->getId(),
-                    Grid::get(txn)->getBalancerConfiguration()->getMaxChunkSizeBytes(),
+                    Grid::get(opCtx)->getBalancerConfiguration()->getMaxChunkSizeBytes(),
                     MigrationSecondaryThrottleOptions::create(
                         MigrationSecondaryThrottleOptions::kOff),
                     true);
@@ -578,26 +649,30 @@ public:
             }
 
             // Reload the config info, after all the migrations
-            chunkManager = config->getChunkManager(txn, nss.ns(), true);
+            catalogCache->invalidateShardedCollection(nss);
+            routingInfo = uassertStatusOK(catalogCache->getCollectionRoutingInfo(opCtx, nss));
+            uassert(ErrorCodes::ConflictingOperationInProgress,
+                    "Collection was successfully written as sharded but got dropped before it "
+                    "could be evenly distributed",
+                    routingInfo.cm());
+            chunkManager = routingInfo.cm();
 
             // 3. Subdivide the big chunks by splitting at each of the points in "allSplits"
             //    that we haven't already split by.
-            shared_ptr<Chunk> currentChunk =
-                chunkManager->findIntersectingChunkWithSimpleCollation(txn, allSplits[0]);
+            auto currentChunk =
+                chunkManager->findIntersectingChunkWithSimpleCollation(allSplits[0]);
 
-            vector<BSONObj> subSplits;
+            std::vector<BSONObj> subSplits;
             for (unsigned i = 0; i <= allSplits.size(); i++) {
                 if (i == allSplits.size() || !currentChunk->containsKey(allSplits[i])) {
                     if (!subSplits.empty()) {
                         auto splitStatus = shardutil::splitChunkAtMultiplePoints(
-                            txn,
+                            opCtx,
                             currentChunk->getShardId(),
                             nss,
                             chunkManager->getShardKeyPattern(),
                             chunkManager->getVersion(),
-                            currentChunk->getMin(),
-                            currentChunk->getMax(),
-                            currentChunk->getLastmod(),
+                            ChunkRange(currentChunk->getMin(), currentChunk->getMax()),
                             subSplits);
                         if (!splitStatus.isOK()) {
                             warning() << "couldn't split chunk " << redact(currentChunk->toString())
@@ -609,8 +684,8 @@ public:
                     }
 
                     if (i < allSplits.size()) {
-                        currentChunk = chunkManager->findIntersectingChunkWithSimpleCollation(
-                            txn, allSplits[i]);
+                        currentChunk =
+                            chunkManager->findIntersectingChunkWithSimpleCollation(allSplits[i]);
                     }
                 } else {
                     BSONObj splitPoint(allSplits[i]);
@@ -623,10 +698,6 @@ public:
                     subSplits.push_back(splitPoint);
                 }
             }
-
-            // Proactively refresh the chunk manager. Not really necessary, but this way it's
-            // immediately up-to-date the next time it's used.
-            config->getChunkManager(txn, nss.ns(), true);
         }
 
         return true;
