@@ -1,4 +1,4 @@
-/*
+/**
  *    Copyright (C) 2010 10gen Inc.
  *
  *    This program is free software: you can redistribute it and/or  modify
@@ -37,40 +37,32 @@
 #include "mongo/base/status.h"
 #include "mongo/bson/util/bson_extract.h"
 #include "mongo/bson/util/builder.h"
-#include "mongo/client/connpool.h"
-#include "mongo/client/dbclientcursor.h"
-#include "mongo/client/parallel.h"
 #include "mongo/db/audit.h"
 #include "mongo/db/auth/action_type.h"
 #include "mongo/db/auth/authorization_session.h"
 #include "mongo/db/commands.h"
+#include "mongo/db/lasterror.h"
 #include "mongo/db/matcher/extensions_callback_noop.h"
-#include "mongo/db/max_time.h"
 #include "mongo/db/namespace_string.h"
 #include "mongo/db/query/find_common.h"
 #include "mongo/db/query/getmore_request.h"
 #include "mongo/db/query/query_request.h"
-#include "mongo/db/server_parameters.h"
 #include "mongo/db/stats/counters.h"
 #include "mongo/db/views/resolved_view.h"
 #include "mongo/rpc/get_status_from_command_result.h"
 #include "mongo/rpc/metadata/server_selection_metadata.h"
-#include "mongo/s/bson_serializable.h"
-#include "mongo/s/catalog/catalog_cache.h"
-#include "mongo/s/chunk_manager.h"
-#include "mongo/s/chunk_version.h"
+#include "mongo/s/catalog_cache.h"
+#include "mongo/s/client/parallel.h"
 #include "mongo/s/client/shard_connection.h"
 #include "mongo/s/client/shard_registry.h"
-#include "mongo/s/client/version_manager.h"
 #include "mongo/s/commands/cluster_explain.h"
-#include "mongo/s/commands/request.h"
-#include "mongo/s/config.h"
 #include "mongo/s/grid.h"
 #include "mongo/s/query/cluster_cursor_manager.h"
 #include "mongo/s/query/cluster_find.h"
 #include "mongo/s/stale_exception.h"
 #include "mongo/s/write_ops/batch_upconvert.h"
 #include "mongo/s/write_ops/batched_command_request.h"
+#include "mongo/s/write_ops/batched_command_response.h"
 #include "mongo/util/log.h"
 #include "mongo/util/mongoutils/str.h"
 #include "mongo/util/timer.h"
@@ -82,11 +74,10 @@ using std::shared_ptr;
 using std::set;
 using std::string;
 using std::stringstream;
-using std::vector;
 
 namespace {
 
-void runAgainstRegistered(OperationContext* txn,
+void runAgainstRegistered(OperationContext* opCtx,
                           const char* ns,
                           BSONObj& jsobj,
                           BSONObjBuilder& anObjBuilder,
@@ -108,116 +99,73 @@ void runAgainstRegistered(OperationContext* txn,
         return;
     }
 
-    Command::execCommandClient(txn, c, queryOptions, ns, jsobj, anObjBuilder);
-}
-
-/**
- * This code handles the $cmd.sys.inprog queries.
- */
-bool handleSpecialNamespaces(OperationContext* txn, Request& request, const QueryMessage& q) {
-    const char* ns = strstr(request.getns(), ".$cmd.sys.");
-    if (!ns)
-        return false;
-
-    ns += 10;
-
-    BSONObjBuilder reply;
-
-    const auto upgradeToRealCommand = [txn, &q, &reply](StringData commandName) {
-        BSONObjBuilder cmdBob;
-        cmdBob.append(commandName, 1);
-        cmdBob.appendElements(q.query);  // fields are validated by Commands
-        auto interposedCmd = cmdBob.done();
-
-        // Rewrite upgraded pseudoCommands to run on the 'admin' database.
-        const NamespaceString interposedNss("admin", "$cmd");
-        runAgainstRegistered(txn, interposedNss.ns().c_str(), interposedCmd, reply, q.queryOptions);
-    };
-
-    if (strcmp(ns, "inprog") == 0) {
-        upgradeToRealCommand("currentOp");
-    } else if (strcmp(ns, "killop") == 0) {
-        upgradeToRealCommand("killOp");
-    } else if (strcmp(ns, "unlock") == 0) {
-        reply.append("err", "can't do unlock through mongos");
-    } else {
-        warning() << "unknown sys command [" << ns << "]";
-        return false;
-    }
-
-    replyToQuery(0, request.session(), request.m(), reply.done());
-    return true;
+    Command::execCommandClient(opCtx, c, queryOptions, ns, jsobj, anObjBuilder);
 }
 
 }  // namespace
 
-void Strategy::queryOp(OperationContext* txn, Request& request) {
-    verify(!NamespaceString(request.getns()).isCommand());
-
+void Strategy::queryOp(OperationContext* opCtx, const NamespaceString& nss, DbMessage* dbm) {
     globalOpCounters.gotQuery();
 
-    QueryMessage q(request.d());
+    const QueryMessage q(*dbm);
 
-    NamespaceString ns(q.ns);
-    Client* client = txn->getClient();
-    AuthorizationSession* authSession = AuthorizationSession::get(client);
-    Status status = authSession->checkAuthForFind(ns, false);
-    audit::logQueryAuthzCheck(client, ns, q.query, status.code());
+    Client* const client = opCtx->getClient();
+    AuthorizationSession* const authSession = AuthorizationSession::get(client);
+
+    Status status = authSession->checkAuthForFind(nss, false);
+    audit::logQueryAuthzCheck(client, nss, q.query, status.code());
     uassertStatusOK(status);
 
     LOG(3) << "query: " << q.ns << " " << redact(q.query) << " ntoreturn: " << q.ntoreturn
            << " options: " << q.queryOptions;
 
-    if (q.ntoreturn == 1 && strstr(q.ns, ".$cmd"))
-        throw UserException(8010, "something is wrong, shouldn't see a command here");
-
     if (q.queryOptions & QueryOption_Exhaust) {
         uasserted(18526,
-                  string("the 'exhaust' query option is invalid for mongos queries: ") + q.ns +
-                      " " + q.query.toString());
+                  str::stream() << "The 'exhaust' query option is invalid for mongos queries: "
+                                << nss.ns()
+                                << " "
+                                << q.query.toString());
     }
 
     // Determine the default read preference mode based on the value of the slaveOk flag.
-    ReadPreference readPreferenceOption = (q.queryOptions & QueryOption_SlaveOk)
-        ? ReadPreference::SecondaryPreferred
-        : ReadPreference::PrimaryOnly;
-    ReadPreferenceSetting readPreference(readPreferenceOption, TagSet());
+    const ReadPreferenceSetting readPreference = [&]() {
+        BSONElement rpElem;
+        auto readPrefExtractStatus = bsonExtractTypedField(
+            q.query, QueryRequest::kWrappedReadPrefField, mongo::Object, &rpElem);
+        if (readPrefExtractStatus == ErrorCodes::NoSuchKey) {
+            return ReadPreferenceSetting(q.queryOptions & QueryOption_SlaveOk
+                                             ? ReadPreference::SecondaryPreferred
+                                             : ReadPreference::PrimaryOnly);
+        }
 
-    BSONElement rpElem;
-    auto readPrefExtractStatus =
-        bsonExtractTypedField(q.query, QueryRequest::kWrappedReadPrefField, mongo::Object, &rpElem);
-
-    if (readPrefExtractStatus.isOK()) {
-        auto parsedRps = ReadPreferenceSetting::fromBSON(rpElem.Obj());
-        uassertStatusOK(parsedRps.getStatus());
-        readPreference = parsedRps.getValue();
-    } else if (readPrefExtractStatus != ErrorCodes::NoSuchKey) {
         uassertStatusOK(readPrefExtractStatus);
-    }
 
-    auto canonicalQuery = CanonicalQuery::canonicalize(txn, q, ExtensionsCallbackNoop());
-    uassertStatusOK(canonicalQuery.getStatus());
+        return uassertStatusOK(ReadPreferenceSetting::fromBSON(rpElem.Obj()));
+    }();
+
+    auto canonicalQuery =
+        uassertStatusOK(CanonicalQuery::canonicalize(opCtx, q, ExtensionsCallbackNoop()));
 
     // If the $explain flag was set, we must run the operation on the shards as an explain command
     // rather than a find command.
-    if (canonicalQuery.getValue()->getQueryRequest().isExplain()) {
-        const QueryRequest& qr = canonicalQuery.getValue()->getQueryRequest();
-        BSONObj findCommand = qr.asFindCommand();
+    const QueryRequest& queryRequest = canonicalQuery->getQueryRequest();
+    if (queryRequest.isExplain()) {
+        const BSONObj findCommand = queryRequest.asFindCommand();
 
         // We default to allPlansExecution verbosity.
-        auto verbosity = ExplainCommon::EXEC_ALL_PLANS;
+        const auto verbosity = ExplainCommon::EXEC_ALL_PLANS;
 
         const bool secondaryOk = (readPreference.pref != ReadPreference::PrimaryOnly);
-        rpc::ServerSelectionMetadata metadata(secondaryOk, readPreference);
+        const rpc::ServerSelectionMetadata metadata(secondaryOk, readPreference);
 
         BSONObjBuilder explainBuilder;
-        uassertStatusOK(
-            Strategy::explainFind(txn, findCommand, qr, verbosity, metadata, &explainBuilder));
+        uassertStatusOK(Strategy::explainFind(
+            opCtx, findCommand, queryRequest, verbosity, metadata, &explainBuilder));
 
         BSONObj explainObj = explainBuilder.done();
         replyToQuery(0,  // query result flags
-                     request.session(),
-                     request.m(),
+                     client->session(),
+                     dbm->msg(),
                      static_cast<const void*>(explainObj.objdata()),
                      explainObj.objsize(),
                      1,  // numResults
@@ -233,8 +181,8 @@ void Strategy::queryOp(OperationContext* txn, Request& request) {
     // 0 means the cursor is exhausted. Otherwise we assume that a cursor with the returned id can
     // be retrieved via the ClusterCursorManager.
     auto cursorId =
-        ClusterFind::runQuery(txn,
-                              *canonicalQuery.getValue(),
+        ClusterFind::runQuery(opCtx,
+                              *canonicalQuery,
                               readPreference,
                               &batch,
                               nullptr /*Argument is for views which OP_QUERY doesn't support*/);
@@ -253,31 +201,71 @@ void Strategy::queryOp(OperationContext* txn, Request& request) {
         obj.appendSelfToBufBuilder(reply.bufBuilderForResults());
         numResults++;
     }
-    reply.send(request.session(),
+
+    reply.send(client->session(),
                0,  // query result flags
-               request.m(),
+               dbm->msg(),
                numResults,
                0,  // startingFrom
                cursorId.getValue());
 }
 
-void Strategy::clientCommandOp(OperationContext* txn, Request& request) {
-    const QueryMessage q(request.d());
+void Strategy::clientCommandOp(OperationContext* opCtx,
+                               const NamespaceString& nss,
+                               DbMessage* dbm) {
+    const QueryMessage q(*dbm);
+
+    Client* const client = opCtx->getClient();
 
     LOG(3) << "command: " << q.ns << " " << redact(q.query) << " ntoreturn: " << q.ntoreturn
            << " options: " << q.queryOptions;
 
     if (q.queryOptions & QueryOption_Exhaust) {
         uasserted(18527,
-                  string("the 'exhaust' query option is invalid for mongos commands: ") + q.ns +
-                      " " + q.query.toString());
+                  str::stream() << "The 'exhaust' query option is invalid for mongos commands: "
+                                << nss.ns()
+                                << " "
+                                << q.query.toString());
     }
 
-    const NamespaceString nss(request.getns());
-    invariant(nss.isCommand() || nss.isSpecialCommand());
+    uassert(16978,
+            str::stream() << "Bad numberToReturn (" << q.ntoreturn
+                          << ") for $cmd type ns - can only be 1 or -1",
+            q.ntoreturn == 1 || q.ntoreturn == -1);
 
-    if (handleSpecialNamespaces(txn, request, q))
-        return;
+    // Handle the $cmd.sys pseudo-commands
+    if (nss.isSpecialCommand()) {
+        const auto upgradeToRealCommand = [&](StringData commandName) {
+            BSONObjBuilder cmdBob;
+            cmdBob.append(commandName, 1);
+            cmdBob.appendElements(q.query);  // fields are validated by Commands
+            auto interposedCmd = cmdBob.done();
+
+            // Rewrite upgraded pseudoCommands to run on the 'admin' database.
+            const NamespaceString interposedNss("admin", "$cmd");
+            BSONObjBuilder reply;
+            runAgainstRegistered(
+                opCtx, interposedNss.ns().c_str(), interposedCmd, reply, q.queryOptions);
+            replyToQuery(0, client->session(), dbm->msg(), reply.done());
+        };
+
+        if (nss.coll() == "$cmd.sys.inprog") {
+            upgradeToRealCommand("currentOp");
+            return;
+        } else if (nss.coll() == "$cmd.sys.killop") {
+            upgradeToRealCommand("killOp");
+            return;
+        } else if (nss.coll() == "$cmd.sys.unlock") {
+            replyToQuery(0,
+                         client->session(),
+                         dbm->msg(),
+                         BSON("err"
+                              << "can't do unlock through mongos"));
+            return;
+        }
+
+        // No pseudo-command found, fall through to execute as a regular query
+    }
 
     BSONObj cmdObj = q.query;
 
@@ -311,7 +299,7 @@ void Strategy::clientCommandOp(OperationContext* txn, Request& request) {
     const int maxTimeMS =
         uassertStatusOK(QueryRequest::parseMaxTimeMS(cmdObj[QueryRequest::cmdOptionMaxTimeMS]));
     if (maxTimeMS > 0) {
-        txn->setDeadlineAfterNowBy(Milliseconds{maxTimeMS});
+        opCtx->setDeadlineAfterNowBy(Milliseconds{maxTimeMS});
     }
 
     int loops = 5;
@@ -321,9 +309,9 @@ void Strategy::clientCommandOp(OperationContext* txn, Request& request) {
             OpQueryReplyBuilder reply;
             {
                 BSONObjBuilder builder(reply.bufBuilderForResults());
-                runAgainstRegistered(txn, q.ns, cmdObj, builder, q.queryOptions);
+                runAgainstRegistered(opCtx, q.ns, cmdObj, builder, q.queryOptions);
             }
-            reply.sendCommandReply(request.session(), request.m());
+            reply.sendCommandReply(client->session(), dbm->msg());
             return;
         } catch (const StaleConfigException& e) {
             if (loops <= 0)
@@ -333,41 +321,43 @@ void Strategy::clientCommandOp(OperationContext* txn, Request& request) {
 
             log() << "Retrying command " << redact(q.query) << causedBy(e);
 
-            // For legacy reasons, ns may not actually be set in the exception :-(
-            string staleNS = e.getns();
-            if (staleNS.size() == 0)
-                staleNS = q.ns;
+            // For legacy reasons, ns may not actually be set in the exception
+            const std::string staleNS(e.getns().empty() ? std::string(q.ns) : e.getns());
 
-            ShardConnection::checkMyConnectionVersions(txn, staleNS);
-            if (loops < 4)
-                versionManager.forceRemoteCheckShardVersionCB(txn, staleNS);
+            ShardConnection::checkMyConnectionVersions(opCtx, staleNS);
+            if (loops < 4) {
+                const NamespaceString staleNSS(staleNS);
+                if (staleNSS.isValid()) {
+                    Grid::get(opCtx)->catalogCache()->invalidateShardedCollection(staleNSS);
+                }
+            }
         } catch (const DBException& e) {
             OpQueryReplyBuilder reply;
             {
                 BSONObjBuilder builder(reply.bufBuilderForResults());
                 Command::appendCommandStatus(builder, e.toStatus());
             }
-            reply.sendCommandReply(request.session(), request.m());
+            reply.sendCommandReply(client->session(), dbm->msg());
             return;
         }
     }
 }
 
-void Strategy::commandOp(OperationContext* txn,
+void Strategy::commandOp(OperationContext* opCtx,
                          const string& db,
                          const BSONObj& command,
                          int options,
                          const string& versionedNS,
                          const BSONObj& targetingQuery,
                          const BSONObj& targetingCollation,
-                         vector<CommandResult>* results) {
+                         std::vector<CommandResult>* results) {
     QuerySpec qSpec(db + ".$cmd", command, BSONObj(), 0, 1, options);
 
     ParallelSortClusteredCursor cursor(
         qSpec, CommandInfo(versionedNS, targetingQuery, targetingCollation));
 
     // Initialize the cursor
-    cursor.init(txn);
+    cursor.init(opCtx);
 
     set<ShardId> shardIds;
     cursor.getQueryShardIds(shardIds);
@@ -383,34 +373,36 @@ void Strategy::commandOp(OperationContext* txn,
     }
 }
 
-void Strategy::getMore(OperationContext* txn, Request& request) {
-    const char* ns = request.getns();
-    const int ntoreturn = request.d().pullInt();
+void Strategy::getMore(OperationContext* opCtx, const NamespaceString& nss, DbMessage* dbm) {
+    const int ntoreturn = dbm->pullInt();
     uassert(
         34424, str::stream() << "Invalid ntoreturn for OP_GET_MORE: " << ntoreturn, ntoreturn >= 0);
-    const long long id = request.d().pullInt64();
+    const long long cursorId = dbm->pullInt64();
+
+    globalOpCounters.gotGetMore();
+
+    Client* const client = opCtx->getClient();
 
     // TODO: Handle stale config exceptions here from coll being dropped or sharded during op for
     // now has same semantics as legacy request.
-    const NamespaceString nss(ns);
-    auto statusGetDb = grid.catalogCache()->getDatabase(txn, nss.db().toString());
+
+    auto statusGetDb = Grid::get(opCtx)->catalogCache()->getDatabase(opCtx, nss.db());
     if (statusGetDb == ErrorCodes::NamespaceNotFound) {
-        replyToQuery(ResultFlag_CursorNotFound, request.session(), request.m(), 0, 0, 0);
+        replyToQuery(ResultFlag_CursorNotFound, client->session(), dbm->msg(), 0, 0, 0);
         return;
     }
-
     uassertStatusOK(statusGetDb);
 
     boost::optional<long long> batchSize;
     if (ntoreturn) {
         batchSize = ntoreturn;
     }
-    GetMoreRequest getMoreRequest(
-        NamespaceString(ns), id, batchSize, boost::none, boost::none, boost::none);
 
-    auto cursorResponse = ClusterFind::runGetMore(txn, getMoreRequest);
+    GetMoreRequest getMoreRequest(nss, cursorId, batchSize, boost::none, boost::none, boost::none);
+
+    auto cursorResponse = ClusterFind::runGetMore(opCtx, getMoreRequest);
     if (cursorResponse == ErrorCodes::CursorNotFound) {
-        replyToQuery(ResultFlag_CursorNotFound, request.session(), request.m(), 0, 0, 0);
+        replyToQuery(ResultFlag_CursorNotFound, client->session(), dbm->msg(), 0, 0, 0);
         return;
     }
     uassertStatusOK(cursorResponse.getStatus());
@@ -425,8 +417,8 @@ void Strategy::getMore(OperationContext* txn, Request& request) {
     }
 
     replyToQuery(0,
-                 request.session(),
-                 request.m(),
+                 client->session(),
+                 dbm->msg(),
                  buffer.buf(),
                  buffer.len(),
                  numResults,
@@ -434,26 +426,30 @@ void Strategy::getMore(OperationContext* txn, Request& request) {
                  cursorResponse.getValue().getCursorId());
 }
 
-void Strategy::killCursors(OperationContext* txn, Request& request) {
-    DbMessage& dbMessage = request.d();
-    const int numCursors = dbMessage.pullInt();
+void Strategy::killCursors(OperationContext* opCtx, DbMessage* dbm) {
+    const int numCursors = dbm->pullInt();
     massert(34425,
             str::stream() << "Invalid killCursors message. numCursors: " << numCursors
                           << ", message size: "
-                          << dbMessage.msg().dataSize()
+                          << dbm->msg().dataSize()
                           << ".",
-            dbMessage.msg().dataSize() == 8 + (8 * numCursors));
+            dbm->msg().dataSize() == 8 + (8 * numCursors));
     uassert(28794,
             str::stream() << "numCursors must be between 1 and 29999.  numCursors: " << numCursors
                           << ".",
             numCursors >= 1 && numCursors < 30000);
-    ConstDataCursor cursors(dbMessage.getArray(numCursors));
-    Client* client = txn->getClient();
-    AuthorizationSession* authSession = AuthorizationSession::get(client);
-    ClusterCursorManager* manager = grid.getCursorManager();
+
+    globalOpCounters.gotOp(dbKillCursors, false);
+
+    ConstDataCursor cursors(dbm->getArray(numCursors));
+
+    Client* const client = opCtx->getClient();
+    AuthorizationSession* const authSession = AuthorizationSession::get(client);
+    ClusterCursorManager* const manager = Grid::get(opCtx)->getCursorManager();
 
     for (int i = 0; i < numCursors; ++i) {
-        CursorId cursorId = cursors.readAndAdvance<LittleEndian<int64_t>>();
+        const CursorId cursorId = cursors.readAndAdvance<LittleEndian<int64_t>>();
+
         boost::optional<NamespaceString> nss = manager->getNamespaceForCursorId(cursorId);
         if (!nss) {
             LOG(3) << "Can't find cursor to kill.  Cursor id: " << cursorId << ".";
@@ -478,60 +474,61 @@ void Strategy::killCursors(OperationContext* txn, Request& request) {
                    << "', cursor id: " << cursorId << ".";
             continue;
         }
+
         LOG(3) << "Killed cursor.  Namespace: '" << *nss << "', cursor id: " << cursorId << ".";
     }
 }
 
-void Strategy::writeOp(OperationContext* txn, int op, Request& request) {
-    // make sure we have a last error
-    dassert(&LastError::get(cc()));
-
+void Strategy::writeOp(OperationContext* opCtx, DbMessage* dbm) {
     OwnedPointerVector<BatchedCommandRequest> commandRequestsOwned;
-    vector<BatchedCommandRequest*>& commandRequests = commandRequestsOwned.mutableVector();
+    std::vector<BatchedCommandRequest*>& commandRequests = commandRequestsOwned.mutableVector();
 
-    msgToBatchRequests(request.m(), &commandRequests);
+    msgToBatchRequests(dbm->msg(), &commandRequests);
 
-    for (vector<BatchedCommandRequest*>::iterator it = commandRequests.begin();
-         it != commandRequests.end();
-         ++it) {
+    auto& clientLastError = LastError::get(opCtx->getClient());
+
+    for (auto it = commandRequests.begin(); it != commandRequests.end(); ++it) {
         // Multiple commands registered to last error as multiple requests
-        if (it != commandRequests.begin())
-            LastError::get(cc()).startRequest();
+        if (it != commandRequests.begin()) {
+            clientLastError.startRequest();
+        }
 
-        BatchedCommandRequest* commandRequest = *it;
+        BatchedCommandRequest* const commandRequest = *it;
 
-        // Adjust namespaces for command
-        NamespaceString fullNS(commandRequest->getNS());
-        string cmdNS = fullNS.getCommandNS();
-        // We only pass in collection name to command
-        commandRequest->setNS(fullNS);
-
-        BSONObjBuilder builder;
-        BSONObj requestBSON = commandRequest->toBSON();
+        BatchedCommandResponse commandResponse;
 
         {
             // Disable the last error object for the duration of the write cmd
-            LastError::Disabled disableLastError(&LastError::get(cc()));
-            runAgainstRegistered(txn, cmdNS.c_str(), requestBSON, builder, 0);
+            LastError::Disabled disableLastError(&clientLastError);
+
+            // Adjust namespace for command
+            const NamespaceString& fullNS(commandRequest->getNS());
+            const std::string cmdNS = fullNS.getCommandNS();
+
+            BSONObj commandBSON = commandRequest->toBSON();
+
+            BSONObjBuilder builder;
+            runAgainstRegistered(opCtx, cmdNS.c_str(), commandBSON, builder, 0);
+
+            bool parsed = commandResponse.parseBSON(builder.done(), nullptr);
+            (void)parsed;  // for compile
+            dassert(parsed && commandResponse.isValid(nullptr));
         }
 
-        BatchedCommandResponse commandResponse;
-        bool parsed = commandResponse.parseBSON(builder.done(), NULL);
-        (void)parsed;  // for compile
-        dassert(parsed && commandResponse.isValid(NULL));
-
         // Populate the lastError object based on the write response
-        LastError::get(cc()).reset();
-        bool hadError =
-            batchErrorToLastError(*commandRequest, commandResponse, &LastError::get(cc()));
+        clientLastError.reset();
+
+        const bool hadError =
+            batchErrorToLastError(*commandRequest, commandResponse, &clientLastError);
 
         // Check if this is an ordered batch and we had an error which should stop processing
-        if (commandRequest->getOrdered() && hadError)
+        if (commandRequest->getOrdered() && hadError) {
             break;
+        }
     }
 }
 
-Status Strategy::explainFind(OperationContext* txn,
+Status Strategy::explainFind(OperationContext* opCtx,
                              const BSONObj& findCommand,
                              const QueryRequest& qr,
                              ExplainCommon::Verbosity verbosity,
@@ -546,7 +543,7 @@ Status Strategy::explainFind(OperationContext* txn,
     Timer timer;
 
     std::vector<Strategy::CommandResult> shardResults;
-    Strategy::commandOp(txn,
+    Strategy::commandOp(opCtx,
                         qr.nss().db().toString(),
                         explainCmdBob.obj(),
                         options,
@@ -566,7 +563,7 @@ Status Strategy::explainFind(OperationContext* txn,
     const char* mongosStageName = ClusterExplain::getStageNameForReadOp(shardResults, findCommand);
 
     return ClusterExplain::buildExplainResult(
-        txn, shardResults, mongosStageName, millisElapsed, out);
+        opCtx, shardResults, mongosStageName, millisElapsed, out);
 }
 
 }  // namespace mongo
