@@ -221,18 +221,24 @@ connect = function(url, user, pass) {
     }
 
     if (!url.startsWith("mongodb://")) {
-        var colon = url.lastIndexOf(":");
-        var slash = url.lastIndexOf("/");
+        const colon = url.lastIndexOf(":");
+        const slash = url.lastIndexOf("/");
+        if (url.split("/").length > 1) {
+            url = url.substring(0, slash).replace(/\//g, "%2F") + url.substring(slash);
+        }
+        if (slash == 0) {
+            throw Error("Failed to parse mongodb:// URL: " + url);
+        }
         if (slash == -1 && colon == -1) {
             url = "mongodb://127.0.0.1:27017/" + url;
-        } else {
+        } else if (slash != -1) {
             url = "mongodb://" + url;
         }
     }
 
     chatty("connecting to: " + url);
     var m = new Mongo(url);
-    db = m.getDB(m.defaultDB);
+    var db = m.getDB(m.defaultDB);
 
     if (user && pass) {
         if (!db.auth(user, pass)) {
