@@ -63,6 +63,7 @@
  * Member variables:
  *  nodes {Array.<Mongo>} - connection to replica set members
  */
+
 var ReplSetTest = function(opts) {
     'use strict';
 
@@ -85,7 +86,11 @@ var ReplSetTest = function(opts) {
     var _unbridgedPorts;
     var _unbridgedNodes;
 
-    this.kDefaultTimeoutMS = 10 * 60 * 1000;
+    var _causalConsistency;
+
+    // Some code still references kDefaultTimeoutMS as a (non-static) member variable, so make sure
+    // it's still accessible that way.
+    this.kDefaultTimeoutMS = ReplSetTest.kDefaultTimeoutMS;
     var oplogName = 'oplog.rs';
 
     // Publicly exposed variables
@@ -114,6 +119,7 @@ var ReplSetTest = function(opts) {
         var twoPrimaries = false;
         self.nodes.forEach(function(node) {
             try {
+                node.setSlaveOk();
                 var n = node.getDB('admin').runCommand({ismaster: 1});
                 if (n.ismaster == true) {
                     if (self.liveNodes.master) {
@@ -122,7 +128,6 @@ var ReplSetTest = function(opts) {
                         self.liveNodes.master = node;
                     }
                 } else {
-                    node.setSlaveOk();
                     self.liveNodes.slaves.push(node);
                 }
             } catch (err) {
@@ -136,6 +141,23 @@ var ReplSetTest = function(opts) {
         return self.liveNodes.master || false;
     }
 
+    function asCluster(conn, fn) {
+        if (self.keyFile) {
+            return authutil.asCluster(conn, self.keyFile, fn);
+        } else {
+            return fn();
+        }
+    }
+
+    /**
+     * Returns 'true' if the test has been configured to run without journaling enabled.
+     */
+    function _isRunningWithoutJournaling() {
+        return self.nojournal || jsTestOptions().noJournal ||
+            jsTestOptions().storageEngine == 'inMemory' ||
+            jsTestOptions().storageEngine == 'ephemeralForTest';
+    }
+
     /**
      * Wait for a rs indicator to go to a particular state or states.
      *
@@ -262,13 +284,31 @@ var ReplSetTest = function(opts) {
     }
 
     /**
-     * Returns the optime for the specified host by issuing replSetGetStatus.
+     * Returns true if the OpTime is empty, else false.
+     *
+     * Empty OpTime Formats:
+     *   PV0: Timestamp(0,0)
+     *   PV1: {ts: Timestamp(0,0), t: NumberLong(-1)}
+     */
+    function _isEmptyOpTime(opTime) {
+        if (!opTime.hasOwnProperty("ts") || !opTime.hasOwnProperty("t")) {
+            return (opTime.getTime() == 0 && opTime.getInc() == 0);
+        }
+        return (opTime.ts.getTime() == 0 && opTime.ts.getInc() == 0 && opTime.t == -1);
+    }
+
+    /**
+     * Returns the OpTime for the specified host by issuing replSetGetStatus.
      */
     function _getLastOpTime(conn) {
         var replSetStatus =
             assert.commandWorked(conn.getDB("admin").runCommand({replSetGetStatus: 1}));
         var connStatus = replSetStatus.members.filter(m => m.self)[0];
-        return connStatus.optime;
+        var opTime = connStatus.optime;
+        if (_isEmptyOpTime(opTime)) {
+            throw new Error("last OpTime is empty -- connection: " + conn);
+        }
+        return opTime;
     }
 
     /**
@@ -292,13 +332,21 @@ var ReplSetTest = function(opts) {
         var replSetStatus =
             assert.commandWorked(conn.getDB("admin").runCommand({replSetGetStatus: 1}));
 
-        var runningWithoutJournaling = TestData.noJournal || "inMemory" == TestData.storageEngine ||
-            "ephemeralForTest" == TestData.storageEngine;
+        // Older servers in multiversion suites do not return an 'optimes' array, so we use
+        // the only OpTime they provide.
+        if (!replSetStatus.optimes) {
+            return _getLastOpTime(conn);
+        }
+
         var opTimeType = "durableOpTime";
-        if (runningWithoutJournaling) {
+        if (_isRunningWithoutJournaling()) {
             opTimeType = "appliedOpTime";
         }
-        return replSetStatus.optimes[opTimeType];
+        var opTime = replSetStatus.optimes[opTimeType];
+        if (_isEmptyOpTime(opTime)) {
+            throw new Error("last durable OpTime is empty -- connection: " + conn);
+        }
+        return opTime;
     }
 
     /*
@@ -311,6 +359,13 @@ var ReplSetTest = function(opts) {
         return ts1.getTime() < ts2.getTime();
     }
 
+    /*
+     * Returns true if the node can be elected primary of a replica set.
+     */
+    function _isElectable(node) {
+        return !node.arbiterOnly && (node.priority === undefined || node.priority != 0);
+    }
+
     /**
      * Returns list of nodes as host:port strings.
      */
@@ -377,8 +432,10 @@ var ReplSetTest = function(opts) {
             var member = {};
             member._id = i;
 
-            var port = this.ports[i];
-            member.host = this.host + ":" + port;
+            member.host = this.host;
+            if (!member.host.contains('/')) {
+                member.host += ":" + this.ports[i];
+            }
 
             var nodeOpts = this.nodeOptions["n" + i];
             if (nodeOpts) {
@@ -394,8 +451,8 @@ var ReplSetTest = function(opts) {
             cfg.members.push(member);
         }
 
-        if (jsTestOptions().useLegacyReplicationProtocol) {
-            cfg.protocolVersion = 0;
+        if (jsTestOptions().forceReplicationProtocolVersion !== undefined) {
+            cfg.protocolVersion = jsTestOptions().forceReplicationProtocolVersion;
         }
 
         if (_configSettings) {
@@ -420,12 +477,19 @@ var ReplSetTest = function(opts) {
      *
      * @param options - The options passed to {@link MongoRunner.runMongod}
      */
-    this.startSet = function(options) {
+    this.startSet = function(options, restart) {
         print("ReplSetTest starting set");
 
+        if (options && options.keyFile) {
+            self.keyFile = options.keyFile;
+        }
+        if (options && options.nojournal != undefined) {
+            self.nojournal = true;
+        }
+
         var nodes = [];
         for (var n = 0; n < this.ports.length; n++) {
-            nodes.push(this.start(n, options));
+            nodes.push(this.start(n, options, restart));
         }
 
         this.nodes = nodes;
@@ -478,21 +542,35 @@ var ReplSetTest = function(opts) {
 
     /**
      * Blocks until all nodes agree on who the primary is.
+     * If 'expectedPrimaryNodeId' is provided, ensure that every node is seeing this node as the
+     * primary. Otherwise, ensure that all the nodes in the set agree with the first node on the
+     * identity of the primary.
      */
-    this.awaitNodesAgreeOnPrimary = function(timeout) {
+    this.awaitNodesAgreeOnPrimary = function(timeout, nodes, expectedPrimaryNodeId) {
         timeout = timeout || self.kDefaultTimeoutMS;
+        nodes = nodes || self.nodes;
+        expectedPrimaryNodeId = expectedPrimaryNodeId || -1;
+        if (expectedPrimaryNodeId === -1) {
+            print("AwaitNodesAgreeOnPrimary: Waiting for nodes to agree on any primary.");
+        } else {
+            print("AwaitNodesAgreeOnPrimary: Waiting for nodes to agree on " +
+                  nodes[expectedPrimaryNodeId].name + " as primary.");
+        }
 
         assert.soonNoExcept(function() {
-            var primary = -1;
+            var primary = expectedPrimaryNodeId;
 
-            for (var i = 0; i < self.nodes.length; i++) {
-                var replSetGetStatus =
-                    self.nodes[i].getDB("admin").runCommand({replSetGetStatus: 1});
+            for (var i = 0; i < nodes.length; i++) {
+                var replSetGetStatus = nodes[i].getDB("admin").runCommand({replSetGetStatus: 1});
                 var nodesPrimary = -1;
                 for (var j = 0; j < replSetGetStatus.members.length; j++) {
                     if (replSetGetStatus.members[j].state === ReplSetTest.State.PRIMARY) {
                         // Node sees two primaries.
                         if (nodesPrimary !== -1) {
+                            print("AwaitNodesAgreeOnPrimary: Retrying because " + nodes[i].name +
+                                  " thinks both " + nodes[nodesPrimary].name + " and " +
+                                  nodes[j].name + " are primary.");
+
                             return false;
                         }
                         nodesPrimary = j;
@@ -500,6 +578,8 @@ var ReplSetTest = function(opts) {
                 }
                 // Node doesn't see a primary.
                 if (nodesPrimary < 0) {
+                    print("AwaitNodesAgreeOnPrimary: Retrying because " + nodes[i].name +
+                          " does not see a primary.");
                     return false;
                 }
 
@@ -507,10 +587,14 @@ var ReplSetTest = function(opts) {
                     // If we haven't seen a primary yet, set it to this.
                     primary = nodesPrimary;
                 } else if (primary !== nodesPrimary) {
+                    print("AwaitNodesAgreeOnPrimary: Retrying because " + nodes[i].name +
+                          " thinks the primary is " + nodes[nodesPrimary].name + " instead of " +
+                          nodes[primary].name);
                     return false;
                 }
             }
 
+            print("AwaitNodesAgreeOnPrimary: Nodes agreed on primary " + nodes[primary].name);
             return true;
         }, "Awaiting nodes to agree on primary", timeout);
     };
@@ -620,43 +704,184 @@ var ReplSetTest = function(opts) {
         if (config.hasOwnProperty(wcMajorityJournalField)) {
             return config;
         }
-        var runningWithoutJournaling = TestData.noJournal || TestData.storageEngine == "inMemory" ||
-            TestData.storageEngine == "ephemeralForTest";
-        if (runningWithoutJournaling) {
+
+        if (_isRunningWithoutJournaling()) {
             config[wcMajorityJournalField] = false;
         }
+
         return config;
     };
 
     this._setDefaultConfigOptions = function(config) {
-        if (jsTestOptions().useLegacyReplicationProtocol &&
+        if (jsTestOptions().forceReplicationProtocolVersion !== undefined &&
             !config.hasOwnProperty("protocolVersion")) {
-            config.protocolVersion = 0;
+            config.protocolVersion = jsTestOptions().forceReplicationProtocolVersion;
         }
         // Update config for non journaling test variants
         this._updateConfigIfNotDurable(config);
     };
 
-    this.initiate = function(cfg, initCmd, timeout) {
+    /**
+     * Runs replSetInitiate on the first node of the replica set.
+     * Ensures that a primary is elected (not necessarily node 0).
+     * initiate() should be preferred instead of this, but this is useful when the connections
+     * aren't authorized to run replSetGetStatus.
+     * TODO(SERVER-14017): remove this in favor of using initiate() everywhere.
+     */
+    this.initiateWithAnyNodeAsPrimary = function(cfg, initCmd) {
         var master = this.nodes[0].getDB("admin");
         var config = cfg || this.getReplSetConfig();
         var cmd = {};
         var cmdKey = initCmd || 'replSetInitiate';
-        timeout = timeout || self.kDefaultTimeoutMS;
 
+        // Throw an exception if nodes[0] is unelectable in the given config.
+        if (!_isElectable(config.members[0])) {
+            throw Error("The node at index 0 must be electable");
+        }
+
+        // Start up a single node replica set then reconfigure to the correct size (if the config
+        // contains more than 1 node), so the primary is elected more quickly.
+        var originalMembers, originalSettings;
+        if (config.members && config.members.length > 1) {
+            originalMembers = config.members.slice();
+            config.members = config.members.slice(0, 1);
+            originalSettings = config.settings;
+            delete config.settings;  // Clear settings to avoid tags referencing sliced nodes.
+        }
         this._setDefaultConfigOptions(config);
 
         cmd[cmdKey] = config;
         printjson(cmd);
 
         assert.commandWorked(master.runCommand(cmd), tojson(cmd));
-        this.awaitSecondaryNodes(timeout);
+        this.getPrimary();  // Blocks until there is a primary.
+
+        // Reconfigure the set to contain the correct number of nodes (if necessary).
+        if (originalMembers) {
+            config.members = originalMembers;
+            if (originalSettings) {
+                config.settings = originalSettings;
+            }
+            config.version = 2;
+
+            // Nodes started with the --configsvr flag must have configsvr = true in their config.
+            if (this.nodes[0].hasOwnProperty("fullOptions") &&
+                this.nodes[0].fullOptions.hasOwnProperty("configsvr")) {
+                config.configsvr = true;
+            }
+
+            cmd = {replSetReconfig: config};
+            print("Reconfiguring replica set to add in other nodes");
+            printjson(cmd);
+
+            // replSetInitiate and replSetReconfig commands can fail with a NodeNotFound error
+            // if a heartbeat times out during the quorum check. We retry three times to reduce
+            // the chance of failing this way.
+            assert.retry(() => {
+                var res;
+                try {
+                    res = master.runCommand(cmd);
+                    if (res.ok === 1) {
+                        return true;
+                    }
+                } catch (e) {
+                    // reconfig can lead to a stepdown if the primary looks for a majority before
+                    // a majority of nodes have successfully joined the set. If there is a stepdown
+                    // then the reconfig request will be killed and respond with a network error.
+                    if (isNetworkError(e)) {
+                        return true;
+                    }
+                    throw e;
+                }
+
+                assert.commandFailedWithCode(
+                    res, ErrorCodes.NodeNotFound, "replSetReconfig during initiate failed");
+                return false;
+            }, "replSetReconfig during initiate failed", 3, 5 * 1000);
+        }
 
         // Setup authentication if running test with authentication
         if ((jsTestOptions().keyFile) && cmdKey == 'replSetInitiate') {
             master = this.getPrimary();
             jsTest.authenticateNodes(this.nodes);
         }
+
+        this.awaitSecondaryNodes();
+    };
+
+    /**
+     * Runs replSetInitiate on the replica set and requests the first node to step up as primary.
+     * This version should be prefered where possible but requires all connections in the
+     * ReplSetTest to be authorized to run replSetGetStatus.
+     */
+    this.initiateWithNodeZeroAsPrimary = function(cfg, initCmd) {
+        this.initiateWithAnyNodeAsPrimary(cfg, initCmd);
+
+        // stepUp() calls awaitReplication() which requires all nodes to be authorized to run
+        // replSetGetStatus.
+        asCluster(this.nodes, function() {
+            self.stepUp(self.nodes[0]);
+        });
+    };
+
+    /**
+     * Runs replSetInitiate on the replica set and requests the first node to step up as
+     * primary.
+     */
+    this.initiate = function(cfg, initCmd) {
+        this.initiateWithNodeZeroAsPrimary(cfg, initCmd);
+    };
+
+    /**
+     * Steps up 'node' as primary.
+     * Waits for all nodes to reach the same optime before sending the replSetStepUp command
+     * to 'node'.
+     * Calls awaitReplication() which requires all connections in 'nodes' to be authenticated.
+     */
+    this.stepUp = function(node) {
+        var secondaryOpTimeType = ReplSetTest.OpTimeType.LAST_DURABLE;
+        if (_isRunningWithoutJournaling()) {
+            secondaryOpTimeType = ReplSetTest.OpTimeType.LAST_APPLIED;
+        }
+
+        this.awaitReplication(ReplSetTest.kDefaultTimeoutMS, secondaryOpTimeType);
+        this.awaitNodesAgreeOnPrimary();
+        if (this.getPrimary() === node) {
+            return;
+        }
+
+        if (this.protocolVersion === 0 || jsTestOptions().forceReplicationProtocolVersion === 0) {
+            // Ensure the specified node is primary.
+            for (let i = 0; i < this.nodes.length; i++) {
+                let primary = this.getPrimary();
+                if (primary === node) {
+                    break;
+                }
+                try {
+                    // Make sure the nodes do not step back up for 10 minutes.
+                    assert.commandWorked(
+                        primary.adminCommand({replSetStepDown: 10 * 60, force: true}));
+                } catch (ex) {
+                    print("Caught exception while stepping down node '" + tojson(node.host) +
+                          "': " + tojson(ex));
+                }
+                this.awaitReplication(ReplSetTest.kDefaultTimeoutMS, secondaryOpTimeType);
+                this.awaitNodesAgreeOnPrimary();
+            }
+
+            // Reset the rest of the nodes so they can run for election during the test.
+            for (let i = 0; i < this.nodes.length; i++) {
+                // Cannot call replSetFreeze on the primary.
+                if (this.nodes[i] === node) {
+                    continue;
+                }
+                assert.commandWorked(this.nodes[i].adminCommand({replSetFreeze: 0}));
+            }
+        } else {
+            assert.commandWorked(node.adminCommand({replSetStepUp: 1}));
+            this.awaitNodesAgreeOnPrimary();
+        }
+        assert.eq(this.getPrimary(), node, node.host + " was not primary after stepUp");
     };
 
     /**
@@ -677,21 +902,45 @@ var ReplSetTest = function(opts) {
     };
 
     this.reInitiate = function() {
-        var config = this.getReplSetConfig();
-        var newVersion = this.getReplSetConfigFromNode().version + 1;
-        config.version = newVersion;
+        var config = this.getReplSetConfigFromNode();
+        var newConfig = this.getReplSetConfig();
+        // Only reset members.
+        config.members = newConfig.members;
+        config.version += 1;
 
         this._setDefaultConfigOptions(config);
 
         try {
             assert.commandWorked(this.getPrimary().adminCommand({replSetReconfig: config}));
         } catch (e) {
-            if (tojson(e).indexOf("error doing query: failed") < 0) {
+            if (!isNetworkError(e)) {
                 throw e;
             }
         }
     };
 
+    /**
+     * Blocks until all nodes in the replica set have the same config version as the primary.
+     **/
+    this.awaitNodesAgreeOnConfigVersion = function(timeout) {
+        timeout = timeout || this.kDefaultTimeoutMS;
+
+        assert.soonNoExcept(function() {
+            var primaryVersion = self.getPrimary().adminCommand({ismaster: 1}).setVersion;
+
+            for (var i = 0; i < self.nodes.length; i++) {
+                var version = self.nodes[i].adminCommand({ismaster: 1}).setVersion;
+                assert.eq(version,
+                          primaryVersion,
+                          "waiting for secondary node " + self.nodes[i].host +
+                              " with config version of " + version +
+                              " to match the version of the primary " + primaryVersion);
+            }
+
+            return true;
+        }, "Awaiting nodes to agree on config version", timeout);
+    };
+
     /**
      * Waits for the last oplog entry on the primary to be visible in the committed snapshop view
      * of the oplog on *all* secondaries.
@@ -787,7 +1036,7 @@ var ReplSetTest = function(opts) {
                               ", but expected config version #" + masterConfigVersion);
 
                         if (slaveConfigVersion > masterConfigVersion) {
-                            master = this.getPrimary();
+                            master = self.getPrimary();
                             masterConfigVersion =
                                 master.getDB("local")['system.replset'].findOne().version;
                             masterName = master.toString().substr(14);  // strip "connection to "
@@ -843,7 +1092,7 @@ var ReplSetTest = function(opts) {
                       " secondaries synced at optime " + tojson(masterLatestOpTime));
                 return true;
             } catch (e) {
-                print("ReplSetTest awaitReplication: caught exception " + e + ';\n' + e.stack);
+                print("ReplSetTest awaitReplication: caught exception " + e);
 
                 // We might have a new master now
                 awaitLastOpTimeWrittenFn();
@@ -886,16 +1135,6 @@ var ReplSetTest = function(opts) {
 
     // Call the provided checkerFunction, after the replica set has been write locked.
     this.checkReplicaSet = function(checkerFunction, ...checkerFunctionArgs) {
-
-        function generateUniqueDbName(dbNames, prefix) {
-            var uniqueDbName;
-            Random.setRandomSeed();
-            do {
-                uniqueDbName = prefix + Random.randInt(100000);
-            } while (dbNames.has(uniqueDbName));
-            return uniqueDbName;
-        }
-
         assert.eq(typeof checkerFunction,
                   "function",
                   "Expected checkerFunction parameter to be a function");
@@ -904,16 +1143,10 @@ var ReplSetTest = function(opts) {
         var primary = this.getPrimary();
         assert(primary, 'calling getPrimary() failed');
 
-        // Since we cannot determine if there is a background index in progress (SERVER-26624),
-        // we flush indexing as follows:
-        //  1. Iterate through all collections and run collMod against each (collMod will block
-        //     replication to wait for any active background index builds to complete)
-        //  2. Insert a document into a dummy collection with a writeConcern for all nodes (which
-        //     will block on completion of the background index build + collMod)
-        var dbNameSet = new Set(primary.getDBNames());
-        var uniqueDbName = generateUniqueDbName(dbNameSet, "flush_all_background_indexes_");
-
-        for (let dbName of dbNameSet.values()) {
+        // Since we cannot determine if there is a background index in progress (SERVER-26624), we
+        // use the "collMod" command to wait for any index builds that may be in progress on the
+        // primary or on one of the secondaries to complete.
+        for (let dbName of primary.getDBNames()) {
             if (dbName === "local") {
                 continue;
             }
@@ -926,28 +1159,25 @@ var ReplSetTest = function(opts) {
                     // skip view evaluation, and therefore won't fail on an invalid view.
                     if (!collInfo.name.startsWith('system.')) {
                         // 'usePowerOf2Sizes' is ignored by the server so no actual collection
-                        // modification takes place.
-                        assert.commandWorked(
-                            dbHandle.runCommand({collMod: collInfo.name, usePowerOf2Sizes: true}));
+                        // modification takes place. We intentionally await replication without
+                        // doing any I/O to avoid any overhead from allocating or deleting data
+                        // files when using the MMAPv1 storage engine. We call awaitReplication()
+                        // later on to ensure the collMod is replicated to all nodes.
+                        assert.commandWorked(dbHandle.runCommand({
+                            collMod: collInfo.name,
+                            usePowerOf2Sizes: true,
+                        }));
                     }
                 });
         }
 
-        let dummyDB = primary.getDB(uniqueDbName);
-        let dummyColl = dummyDB.dummy;
-        assert.writeOK(dummyColl.insert(
-            {x: 1}, {writeConcern: {w: this.nodeList().length, wtimeout: self.kDefaultTimeoutMS}}));
-
-        // We drop the dummy database for cleanup purposes only.
-        assert.commandWorked(dummyDB.dropDatabase());
-
         var activeException = false;
 
+        // Lock the primary to prevent the TTL monitor from deleting expired documents in
+        // the background while we are getting the dbhashes of the replica set members.
+        assert.commandWorked(primary.adminCommand({fsync: 1, lock: 1}),
+                             'failed to lock the primary');
         try {
-            // Lock the primary to prevent the TTL monitor from deleting expired documents in
-            // the background while we are getting the dbhashes of the replica set members.
-            assert.commandWorked(primary.adminCommand({fsync: 1, lock: 1}),
-                                 'failed to lock the primary');
             this.awaitReplication(60 * 1000 * 5);
             checkerFunction.apply(this, checkerFunctionArgs);
         } catch (e) {
@@ -1059,6 +1289,9 @@ var ReplSetTest = function(opts) {
             // liveNodes must have been populated.
             var primary = rst.liveNodes.master;
             var combinedDBs = new Set(primary.getDBNames());
+            // replSetConfig will be undefined for master/slave passthrough.
+            const replSetConfig =
+                rst.getReplSetConfigFromNode ? rst.getReplSetConfigFromNode() : undefined;
 
             rst.getSecondaries().forEach(secondary => {
                 secondary.getDBNames().forEach(dbName => combinedDBs.add(dbName));
@@ -1147,8 +1380,10 @@ var ReplSetTest = function(opts) {
                     // Check that the following collection stats are the same across replica set
                     // members:
                     //  capped
-                    //  nindexes
+                    //  nindexes, except on nodes with buildIndexes: false
                     //  ns
+                    const hasSecondaryIndexes = !replSetConfig ||
+                        replSetConfig.members[rst.getNodeId(secondary)].buildIndexes !== false;
                     primaryCollections.forEach(collName => {
                         var primaryCollStats =
                             primary.getDB(dbName).runCommand({collStats: collName});
@@ -1158,7 +1393,8 @@ var ReplSetTest = function(opts) {
                         assert.commandWorked(secondaryCollStats);
 
                         if (primaryCollStats.capped !== secondaryCollStats.capped ||
-                            primaryCollStats.nindexes !== secondaryCollStats.nindexes ||
+                            (hasSecondaryIndexes &&
+                             primaryCollStats.nindexes !== secondaryCollStats.nindexes) ||
                             primaryCollStats.ns !== secondaryCollStats.ns) {
                             print(msgPrefix +
                                   ', the primary and secondary have different stats for the ' +
@@ -1229,8 +1465,10 @@ var ReplSetTest = function(opts) {
             this.query = function(ts) {
                 var coll = this.getOplogColl();
                 var query = {ts: {$gte: ts ? ts : new Timestamp()}};
-                // Set the cursor to read backwards, from last to first.
-                this.cursor = coll.find(query).sort({$natural: -1});
+                // Set the cursor to read backwards, from last to first. We also set the cursor not
+                // to time out since it may take a while to process each batch and a test may have
+                // changed "cursorTimeoutMillis" to a short time period.
+                this.cursor = coll.find(query).sort({$natural: -1}).noCursorTimeout();
             };
 
             this.getFirstDoc = function() {
@@ -1281,10 +1519,12 @@ var ReplSetTest = function(opts) {
                     var otherOplogEntry = readers[i].next();
                     if (!bsonBinaryEqual(oplogEntry, otherOplogEntry)) {
                         var query = prevOplogEntry ? {ts: {$lte: prevOplogEntry.ts}} : {};
-                        rst.nodes.forEach(node => this.dumpOplog(node, query));
-                        assert(false,
-                               msgPrefix + ", non-matching oplog entry for nodes: " +
-                                   firstReader.mongo.host + " " + readers[i].mongo.host);
+                        rst.nodes.forEach(node => this.dumpOplog(node, query, 100));
+                        var log = msgPrefix +
+                            ", non-matching oplog entries for the following nodes: \n" +
+                            firstReader.mongo.host + ": " + tojsononeline(oplogEntry) + "\n" +
+                            readers[i].mongo.host + ": " + tojsononeline(otherOplogEntry);
+                        assert(false, log);
                     }
                 }
                 prevOplogEntry = oplogEntry;
@@ -1537,7 +1777,7 @@ var ReplSetTest = function(opts) {
             return;
         }
 
-        if (_alldbpaths) {
+        if ((!opts || !opts.noCleanData) && _alldbpaths) {
             print("ReplSetTest stopSet deleting all dbpaths");
             for (var i = 0; i < _alldbpaths.length; i++) {
                 resetDbpath(_alldbpaths[i]);
@@ -1639,7 +1879,11 @@ var ReplSetTest = function(opts) {
      * Constructor, which instantiates the ReplSetTest object from an existing set.
      */
     function _constructFromExistingSeedNode(seedNode) {
-        var conf = _replSetGetConfig(new Mongo(seedNode));
+        const conn = new Mongo(seedNode);
+        if (jsTest.options().keyFile) {
+            self.keyFile = jsTest.options().keyFile;
+        }
+        var conf = asCluster(conn, () => _replSetGetConfig(conn));
         print('Recreating replica set from config ' + tojson(conf));
 
         var existingNodes = conf.members.map(member => member.host);
@@ -1648,12 +1892,22 @@ var ReplSetTest = function(opts) {
     }
 
     if (typeof opts === 'string' || opts instanceof String) {
-        _constructFromExistingSeedNode(opts);
+        retryOnNetworkError(function() {
+            // The primary may unexpectedly step down during startup if under heavy load
+            // and too slowly processing heartbeats. When it steps down, it closes all of
+            // its connections.
+            _constructFromExistingSeedNode(opts);
+        }, 10);
     } else {
         _constructStartNewInstances(opts);
     }
 };
 
+/**
+ *  Global default timeout (10 minutes).
+ */
+ReplSetTest.kDefaultTimeoutMS = 10 * 60 * 1000;
+
 /**
  * Set of states that the replica set can be in. Used for the wait functions.
  */
