@@ -81,6 +81,11 @@ Status addSSLServerOptions(moe::OptionSection* options) {
     options->addOptionChaining(
         "net.ssl.CAFile", "sslCAFile", moe::String, "Certificate Authority file for SSL");
 
+    options->addOptionChaining("net.ssl.clusterCAFile",
+                               "sslClusterCAFile",
+                               moe::String,
+                               "CA used for verifying remotes during outbound connections");
+
     options->addOptionChaining(
         "net.ssl.CRLFile", "sslCRLFile", moe::String, "Certificate Revocation List file for SSL");
 
@@ -168,6 +173,15 @@ Status addSSLClientOptions(moe::OptionSection* options) {
     options->addOptionChaining(
         "ssl.FIPSMode", "sslFIPSMode", moe::Switch, "activate FIPS 140-2 mode at startup");
 
+    options
+        ->addOptionChaining(
+            "ssl.disabledProtocols",
+            "sslDisabledProtocols",
+            moe::String,
+            "Comma separated list of TLS protocols to disable [TLS1_0,TLS1_1,TLS1_2]")
+        .hidden();
+
+
     return Status::OK();
 }
 
@@ -219,6 +233,64 @@ Status canonicalizeSSLServerOptions(moe::Environment* params) {
     return Status::OK();
 }
 
+/**
+ * Older versions of mongod/mongos accepted --sslDisabledProtocols values
+ * in the form 'noTLS1_0,noTLS1_1'.  kAcceptNegativePrefix allows us to
+ * continue accepting this format on mongod/mongos while only supporting
+ * the "standard" TLS1_X format in the shell.
+ */
+enum DisabledProtocolsMode {
+    kStandardFormat,
+    kAcceptNegativePrefix,
+};
+
+Status storeDisabledProtocols(const std::string& disabledProtocols,
+                              DisabledProtocolsMode mode = kStandardFormat) {
+    if (disabledProtocols == "none") {
+        // Allow overriding the default behavior below of implicitly disabling TLS 1.0.
+        return Status::OK();
+    }
+
+    // The disabledProtocols field is composed of a comma separated list of protocols to
+    // disable. First, tokenize the field.
+    const auto tokens = StringSplitter::split(disabledProtocols, ",");
+
+    // All universally accepted tokens, and their corresponding enum representation.
+    const std::map<std::string, SSLParams::Protocols> validConfigs{
+        {"TLS1_0", SSLParams::Protocols::TLS1_0},
+        {"TLS1_1", SSLParams::Protocols::TLS1_1},
+        {"TLS1_2", SSLParams::Protocols::TLS1_2},
+    };
+
+    // These noTLS* tokens exist for backwards compatibility.
+    const std::map<std::string, SSLParams::Protocols> validNoConfigs{
+        {"noTLS1_0", SSLParams::Protocols::TLS1_0},
+        {"noTLS1_1", SSLParams::Protocols::TLS1_1},
+        {"noTLS1_2", SSLParams::Protocols::TLS1_2},
+    };
+
+    // Map the tokens to their enum values, and push them onto the list of disabled protocols.
+    for (const std::string& token : tokens) {
+        auto mappedToken = validConfigs.find(token);
+        if (mappedToken != validConfigs.end()) {
+            sslGlobalParams.sslDisabledProtocols.push_back(mappedToken->second);
+            continue;
+        }
+
+        if (mode == DisabledProtocolsMode::kAcceptNegativePrefix) {
+            auto mappedNoToken = validNoConfigs.find(token);
+            if (mappedNoToken != validNoConfigs.end()) {
+                sslGlobalParams.sslDisabledProtocols.push_back(mappedNoToken->second);
+                continue;
+            }
+        }
+
+        return Status(ErrorCodes::BadValue, "Unrecognized disabledProtocols '" + token + "'");
+    }
+
+    return Status::OK();
+}
+
 Status storeSSLServerOptions(const moe::Environment& params) {
     if (params.count("net.ssl.mode")) {
         std::string sslModeParam = params["net.ssl.mode"].as<string>();
@@ -260,6 +332,12 @@ Status storeSSLServerOptions(const moe::Environment& params) {
                 .generic_string();
     }
 
+    if (params.count("net.ssl.clusterCAFile")) {
+        sslGlobalParams.sslClusterCAFile =
+            boost::filesystem::absolute(params["net.ssl.clusterCAFile"].as<std::string>())
+                .generic_string();
+    }
+
     if (params.count("net.ssl.CRLFile")) {
         sslGlobalParams.sslCRLFile =
             boost::filesystem::absolute(params["net.ssl.CRLFile"].as<std::string>())
@@ -267,34 +345,21 @@ Status storeSSLServerOptions(const moe::Environment& params) {
     }
 
     if (params.count("net.ssl.sslCipherConfig")) {
+        warning()
+            << "net.ssl.sslCipherConfig is deprecated. It will be removed in a future release.";
+        if (!sslGlobalParams.sslCipherConfig.empty()) {
+            return Status(ErrorCodes::BadValue,
+                          "net.ssl.sslCipherConfig is incompatible with the openSSLCipherConfig "
+                          "setParameter");
+        }
         sslGlobalParams.sslCipherConfig = params["net.ssl.sslCipherConfig"].as<string>();
     }
 
     if (params.count("net.ssl.disabledProtocols")) {
-        // The disabledProtocols field is composed of a comma separated list of protocols to
-        // disable. First, tokenize the field.
-        std::vector<std::string> tokens =
-            StringSplitter::split(params["net.ssl.disabledProtocols"].as<string>(), ",");
-
-        // All accepted tokens, and their corresponding enum representation. The noTLS* tokens
-        // exist for backwards compatibility.
-        const std::map<std::string, SSLParams::Protocols> validConfigs{
-            {"TLS1_0", SSLParams::Protocols::TLS1_0},
-            {"noTLS1_0", SSLParams::Protocols::TLS1_0},
-            {"TLS1_1", SSLParams::Protocols::TLS1_1},
-            {"noTLS1_1", SSLParams::Protocols::TLS1_1},
-            {"TLS1_2", SSLParams::Protocols::TLS1_2},
-            {"noTLS1_2", SSLParams::Protocols::TLS1_2}};
-
-        // Map the tokens to their enum values, and push them onto the list of disabled protocols.
-        for (const std::string& token : tokens) {
-            auto mappedToken = validConfigs.find(token);
-            if (mappedToken != validConfigs.end()) {
-                sslGlobalParams.sslDisabledProtocols.push_back(mappedToken->second);
-            } else {
-                return Status(ErrorCodes::BadValue,
-                              "Unrecognized disabledProtocols '" + token + "'");
-            }
+        const auto status = storeDisabledProtocols(params["net.ssl.disabledProtocols"].as<string>(),
+                                                   DisabledProtocolsMode::kAcceptNegativePrefix);
+        if (!status.isOK()) {
+            return status;
         }
     }
 
@@ -389,6 +454,14 @@ Status storeSSLClientOptions(const moe::Environment& params) {
     if (params.count("ssl.FIPSMode")) {
         sslGlobalParams.sslFIPSMode = true;
     }
+    if (params.count("ssl.disabledProtocols")) {
+        const auto status =
+            storeDisabledProtocols(params["ssl.disabledProtocols"].as<std::string>());
+        if (!status.isOK()) {
+            return status;
+        }
+    }
+
     return Status::OK();
 }
 
