@@ -33,6 +33,7 @@
 #include "mongo/util/signal_handlers.h"
 
 #include <signal.h>
+#include <time.h>
 
 #if !defined(_WIN32)
 #include <unistd.h>
@@ -43,6 +44,7 @@
 #include "mongo/platform/process_id.h"
 #include "mongo/stdx/thread.h"
 #include "mongo/util/assert_util.h"
+#include "mongo/util/concurrency/idle_thread_block.h"
 #include "mongo/util/exit.h"
 #include "mongo/util/log.h"
 #include "mongo/util/quick_exit.h"
@@ -163,13 +165,26 @@ sigset_t asyncSignals;
 void signalProcessingThread() {
     setThreadName("signalProcessingThread");
 
+    time_t signalTimeSeconds = -1;
+    time_t lastSignalTimeSeconds = -1;
+
     while (true) {
         int actualSignal = 0;
-        int status = sigwait(&asyncSignals, &actualSignal);
+        int status = [&] {
+            MONGO_IDLE_THREAD_BLOCK;
+            return sigwait(&asyncSignals, &actualSignal);
+        }();
         fassert(16781, status == 0);
         switch (actualSignal) {
             case SIGUSR1:
                 // log rotate signal
+                signalTimeSeconds = time(0);
+                if (signalTimeSeconds <= lastSignalTimeSeconds) {
+                    // ignore multiple signals in the same or earlier second.
+                    break;
+                }
+
+                lastSignalTimeSeconds = signalTimeSeconds;
                 fassert(16782, rotateLogs(serverGlobalParams.logRenameOnRotate));
                 logProcessDetailsForLogRotate();
                 break;
